/* cdfbadcut -- This program reduces a particular dimension by taking
 * as the dimension value the first or last value of a test field that
 * is within the bad data value limits.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

static char hbin[HBMAX][LINE],hbout[HBMAX][LINE];
static char cline[LINE];
static float bad,badlim;
static long nsbin,nvbin,nsbout,nvbout;
static float *sbin,*vbin,*sbout,*vbout;
static int nsflds,nvflds;
static struct field *fp,findex,ftest,findexout,fsinn[HBMAX],
     fvin[HBMAX],fsout[HBMAX],fvout[HBMAX];
static int poop,loop;
static char dnlist[4][LINE];
static int dslist[4];
static int first;
static long *levmap;

main(argc,argv)
int argc;
char *argv[];
{

/* check argument list */
  if (argc != 4 || (strcmp(argv[1],"-f") != 0 && strcmp(argv[1],"-l") != 0)) {
    fprintf(stderr,"Usage: cdfbadcut -[f|l] dimension test_field\n");
    exit(1);
  }
  if (strcmp(argv[1],"-f") == 0) first = 1;
  else first = 0;

/* get header from standard input */
  gethdr(stdin,hbin,HBMAX);
  if (getfmt(hbin,HBMAX) != 'f') {
    fprintf(stderr,"cdfbadcut: float format expected\n");
    exit(1);
  }

/* check to see that fields exist */
  if ((fp = seekfld(hbin,HBMAX,'s',argv[2])) == NULL || fp->dim != 1 ||
       strcmp(fp->fname,fp->dname1) != 0) {
    fprintf(stderr,"cdfbadcut: no index field named %s\n",argv[2]);
    exit(1);
  }
  findex = *fp;
  if ((fp = seekfld(hbin,HBMAX,'v',argv[3])) == NULL) {
    fprintf(stderr,"cdfbadcut: no variable field named %s\n",argv[3]);
    exit(1);
  }
  ftest = *fp;

/* get bad data values */
  if (seekpar(hbin,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbin,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;

/* build the new header */
  nullhdr(hbout,HBMAX,"float");
  copycmt(hbout,HBMAX,hbin,HBMAX);
  copypar(hbout,HBMAX,hbin,HBMAX);
  sprintf(cline,"cdfbadcut: %s %s %s\n",argv[1],argv[2],argv[3]);
  addcline(hbout,HBMAX,cline);

/* static fields */
  nsflds = loop = 0;
  while ((fp = getfld(hbin,HBMAX,'s',loop++)) != NULL) {
    if (strcmp(fp->fname,argv[2]) != 0) {
      zapfld(fp,argv[2],'s');
      fsinn[nsflds++] = *fp;
    } 
  }

/* variable fields */
  nvflds = 0;
  while ((fp = getfld(hbin,HBMAX,'v',nvflds)) != NULL) {
    fvin[nvflds++] = *fp;
    zapfld(fp,argv[2],'v');
  }

/* set up zapped index field with same characteristics as zapped test field */
  fp = seekfld(hbout,HBMAX,'v',argv[3]);
  addfld(hbout,HBMAX,'v',findex.fname,findex.smul,findex.sadd,findex.bytes,
       fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
       fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
  findexout = *seekfld(hbout,HBMAX,'v',findex.fname);

/* get structures for non-special output fields */
  loop = 0;
  while ((fp = getfld(hbout,HBMAX,'s',loop)) != NULL) {
    fsout[loop++] = *fp;
  }
  loop = 0;
  while ((fp = getfld(hbout,HBMAX,'v',loop)) != NULL) {
    fvout[loop++] = *fp;
  }

/* write new header */
  puthdr(stdout,hbout,HBMAX);

/* allocate buffers */
  nsbin = elemcnt(hbin,HBMAX,'s');
  sbin = getbuff(nsbin);
  nsbout = elemcnt(hbout,HBMAX,'s');
  sbout = getbuff(nsbout);
  nvbin = elemcnt(hbin,HBMAX,'v');
  vbin = getbuff(nvbin);
  nvbout = elemcnt(hbout,HBMAX,'v');
  vbout = getbuff(nvbout);
  if ((levmap = (long*)malloc(sizeof(long)*findexout.tsize)) == NULL) {
    fprintf(stderr,"cdfbadcut: can't get enough memory\n");
    exit(1);
  }

/* read the static slice */
  getslice(stdin,nsbin,sbin);

/* transfer static slice data */
  for (loop = 0; loop < nsflds; loop++) {
    if (fsinn[loop].tsize == fsout[loop].tsize) {
      for (poop = 0; poop < fsinn[loop].tsize; poop++) {
        sbout[fsout[loop].ptr + poop] = sbin[fsinn[loop].ptr + poop];
      }
    }
    else {
      for (poop = 0; poop < fsout[loop].tsize; poop++) {
        sbout[fsout[loop].ptr + poop] = bad;
      }
    }
  }

/* write the static slice */
  putslice(stdout,nsbout,sbout);

/* loop on variable slice read */
  while (getslice(stdin,nvbin,vbin) != EOF) {

/* compute level map */
    zaptest(argv[2],&ftest,vbin,first,levmap);

/* transfer variable slice data */
    for (loop = 0; loop < nvflds; loop++) {
      if (fvin[loop].tsize == fvout[loop].tsize) {
        for (poop = 0; poop < fvin[loop].tsize;poop++) {
          vbout[fvout[loop].ptr + poop] = vbin[fvin[loop].ptr + poop];
        }
      }
      else zapdata(argv[2],fvin + loop,fvout + loop,&ftest,vbin,vbout,levmap);
    }

/* special treatment for new variable field */
    for (loop = 0; loop < findexout.tsize; loop++) {
      if (levmap[loop] < 0) {
        vbout[findexout.ptr + loop] = bad;
      }
      else {
        vbout[findexout.ptr + loop] = sbin[findex.ptr + levmap[loop]];
      }
    }

/* write variable slice */
    putslice(stdout,nvbout,vbout);
  }
  exit(0);
}

/* zapdata -- transfer data, reducing dimensionality */
zapdata(dname,fpin,fpout,fptest,bin,bout,levmap)
char *dname;
struct field *fpin,*fpout,*fptest;
float *bin,*bout;
long *levmap;
{
  long remember,yoop,zoop,dtest,start,incr,size;

/* test for same dimensionality between test field and field to be reduced */
  dtest = 0;
  if (fpin->dim != fptest->dim) dtest++;
  if (fpin->tsize != fptest->tsize) dtest++;
  if (fpin->dim > 0 && strcmp(fpin->dname1,fptest->dname1) != 0) dtest++;
  if (fpin->dim > 1 && strcmp(fpin->dname2,fptest->dname2) != 0) dtest++;
  if (fpin->dim > 2 && strcmp(fpin->dname3,fptest->dname3) != 0) dtest++;
  if (fpin->dim > 3 && strcmp(fpin->dname4,fptest->dname4) != 0) dtest++;

/* put bad data if not same dimensionality */
  if (dtest) {
    for (zoop = 0; zoop < fpout->tsize; zoop++) {
      bout[fpout->ptr + zoop] = bad;
    }
  }

/* otherwise, do the reduction */
  else {
    zoop = 0;
    while (subspace1(fptest,dname,zoop,&start,&incr,&size) != FAIL) {

/* do the transfer */
      remember = levmap[zoop];
      if (remember >= 0) bout[fpout->ptr + zoop++] =
           bin[fpin->ptr + start + incr*remember];
      else bout[fpout->ptr + zoop++] = bad;
    }
  }
}

/* zaptest -- compute the level map */
zaptest(dname,fptest,bin,orderflag,levmap)
char *dname;
struct field *fptest;
float *bin;
long orderflag;
long *levmap;
{
  long zoop,yoop,start,incr,size,remember;

/* loop on instances of the dimension */
  zoop = 0;
  while (subspace1(fptest,dname,zoop,&start,&incr,&size) != FAIL) {

/* do the test */
    remember = -1;
    for (yoop = 0; yoop < size; yoop++) {
      if (fabs(bin[fptest->ptr + start + incr*yoop]) < badlim) {
        remember = yoop;
        if (orderflag) break;
      }
    }
    levmap[zoop++] = remember;
  }

/* fail if no instances */
  if (zoop == 0) {
    fprintf(stderr,"cdfbadcut: %s doesn't contain dimension %s\n",
         fptest->fname,dname);
    exit(1);
  }
}

/* zapfld -- zap the old dimension out of the field and add to new header */
zapfld(fp,dimension,ftype)
struct field *fp;
char *dimension;
char ftype;
{
  int newdim;

/* set up lists */
  strcpy(dnlist[0],fp->dname1);
  strcpy(dnlist[1],fp->dname2);
  strcpy(dnlist[2],fp->dname3);
  strcpy(dnlist[3],fp->dname4);
  dslist[0] = fp->dsize1;
  dslist[1] = fp->dsize2;
  dslist[2] = fp->dsize3;
  dslist[3] = fp->dsize4;

/* check for dimension name and eliminate */
  newdim = 0;
  for (poop = 0; poop < fp->dim; poop++) {
    if (strcmp(dimension,dnlist[poop]) != 0) {
      strcpy(dnlist[newdim],dnlist[poop]);
      dslist[newdim++] = dslist[poop];
    }
  }

/* add the field to the output buffer */
  addfld(hbout,HBMAX,ftype,fp->fname,fp->smul,fp->sadd,fp->bytes,newdim,
       dnlist[0],dslist[0],
       dnlist[1],dslist[1],
       dnlist[2],dslist[2],
       dnlist[3],dslist[3]);
}
