/* cdfcatf concatenates the variable sections of any number of files onto a
 * complete file. This only works if all variable slice formats are the same. 
 */
#include <stdio.h>
#include <string.h>
#include "cdfhdr.h"

static int i;			
static char hbuff[HBMAX][LINE];  /* buffer for first ascii header */
static char cbuff[HBMAX][LINE];  /* buffer for subsequent headers */
static char cline[LINE];         /* comment line buffer */
static long vsize;               /* size of variable slice */
static long ssize;               /* size of static slice */
static long num;    /* number of items read returned from getslice */
static float *sbuff,*vbuff;      /* static and variable slice buffers */
static FILE *fp;

main(argc,argv)
int argc;
char **argv;
{
  
  /* check usage */
  if (argc < 2) {
    fprintf(stderr,"Usage: cdfcatf file_1 file_2 ...\n");
    exit(1);
  }
  
  /* open and get header of first file */
  if((fp = fopen(argv[1],"r")) != NULL) {
    gethdr(fp,hbuff,HBMAX);
    /* add comment */
    addcline(hbuff,HBMAX,"cdfcatf:\n");
    strcpy(cline,"");
    for (i = 1; i < argc; i++) {
      if ((strlen(cline) + strlen(argv[i])) > (LINE - 6))
	break;
      strcat(cline," ");
      strcat(cline,argv[i]);
    }
    strcat(cline,"\n");
    addcline(hbuff,HBMAX,cline);
    
    /* output first file, it doesn't change */
    puthdr(stdout,hbuff,HBMAX);
    
    /* static section */
    ssize = elemcnt(hbuff,HBMAX,'s');
    sbuff = (float *) malloc(sizeof(float)*ssize);
    if (sbuff == NULL) {
      fprintf(stderr,"cdfcatf: can't get enough memory\n");
      exit(1);
    }
    getslice(fp,ssize,sbuff);
    putslice(stdout,ssize,sbuff);
    
    /* now var section */
    vsize = elemcnt(hbuff,HBMAX,'v');
    vbuff = (float *) malloc(sizeof(float)*vsize);
    if (vbuff == NULL) {
      fprintf(stderr,"cdfcatf: can't get enough memory\n");
      exit(1);
    }
    num = getslice(fp,vsize,vbuff);
    while(num != EOF) {
      putslice(stdout,vsize,vbuff);
      num = getslice(fp,vsize,vbuff);
    }
    
    /* close the initial file */
    fclose(fp);
  }
  else {
    fprintf(stderr,"cdfcatf: fatal error, couldn't open %s\n",
	    argv[1]);
    exit(1);
  }
  
  /* for each remaining filename on the command line */
  for(i = 2; i < argc; i++) {
    
    /* open, check that var format matches 
     * and write var slices until eof */
    if((fp = fopen(argv[i],"r")) == NULL) {
      fprintf(stderr,"cdfcatf; fatal error, ");
      fprintf(stderr,"couldn't open %s\n",argv[i]);
      exit(1);
    }
    gethdr(fp,cbuff,HBMAX);
    
    /* read away static section */
    ssize = elemcnt(cbuff,HBMAX,'s');
    getslice(fp,ssize,sbuff);
    
    /* if header info for var section differs except in amount
     * of white space */
    if(vdiff(hbuff,cbuff) == 0) {
      fprintf(stderr,"cdfcatf: fatal error, formats differ\n");
      exit(1);
    }
    
    /* if format OK, write var slices until EOF */ 
    num = getslice(fp,vsize,vbuff);
    while(num != EOF) {
      putslice(stdout,vsize,vbuff);
      num = getslice(fp,vsize,vbuff);
    }
    
    /* close the file */
    fclose(fp);
  }
  exit(0);
}
/* vdiff accepts two header buffers and compares their var sections to see that
 * they match. returns 1 if so, 0 if not
 */
vdiff(buff1, buff2)
     char buff1[HBMAX][LINE];	/* original file's buffer */
     char buff2[HBMAX][LINE];	/* a subsequent file's buffer */
{
  int pos;	/* line number of section of buffer */
  struct field *fld1,*fld2;
  /* pointers to struct with field info */
  
  /* for each field in the static section */
  pos = 0;
  
  /* get a field from each header */
  while((fld1 = getfld(buff1,HBMAX,'s',pos)) != NULL) {
    
    /* compare struct members to see that they agree */
    if((fld2 = getfld(buff2,HBMAX,'s',pos++)) == NULL ||
       strcmp(fld1->fname,fld2->fname) || 
       fld1->smul != fld2->smul ||
       fld1->sadd != fld2->sadd ||
       fld1->bytes != fld2->bytes ||
       fld1->dim != fld2->dim ||
       fld1->dsize1 != fld2->dsize1 ||
       fld1->dsize2 != fld2->dsize2 ||
       fld1->dsize3 != fld2->dsize3 ||
       fld1->tsize != fld2->tsize ||
       fld1->ptr != fld2->ptr ||
       strcmp(fld1->dname1, fld2->dname1) ||
       strcmp(fld1->dname2, fld2->dname2) ||
       strcmp(fld1->dname3, fld2->dname3)) {
      
      return(0);
    }
  }
  
  /* now we have end of static section file 1, check that also end of
   * static section file2 before going on */
  if((fld2 = getfld(buff2,HBMAX,'s',pos)) != NULL) {
    return(0);
  }
  
  /* for each field in var section */
  pos = 0;
  
  /* get a field from each header */
  while((fld1 = getfld(buff1,HBMAX,'v',pos)) != NULL) {
    
    /* compare struct members to see that they agree */
    if((fld2 = getfld(buff2,HBMAX,'v',pos++)) == NULL ||
       strcmp(fld1->fname,fld2->fname) || 
       fld1->smul != fld2->smul ||
       fld1->sadd != fld2->sadd ||
       fld1->bytes != fld2->bytes ||
       fld1->dim != fld2->dim ||
       fld1->dsize1 != fld2->dsize1 ||
       fld1->dsize2 != fld2->dsize2 ||
       fld1->dsize3 != fld2->dsize3 ||
       fld1->tsize != fld2->tsize ||
       fld1->ptr != fld2->ptr ||
       strcmp(fld1->dname1, fld2->dname1) ||
       strcmp(fld1->dname2, fld2->dname2) ||
       strcmp(fld1->dname3, fld2->dname3)) {
      
      return(0);
    }
  }
  
  /* now we have end of variable section file 1, check that also end of
   * variable section file2 */
  if((fld2 = getfld(buff2,HBMAX,'v',pos)) != NULL) {
    return(0);
  }
  return(1);
}
