/* cdfclaus -- This program converts CLAUS infrared brightness
 * temperature data (see http://badc.nerc.ac.uk/data/claus/)
 * to Candis format.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "quickcandis.h"

main(int argc, char *argv[])
{
  int i,j,ix,iy,lon,lat,lonmax,latmax,fh,flon,flat,hires,inchar;
  float latlonincr,fyear,fmonth,fday,fhour;
  float *outbuff;
  char hline[82],year[5],month[3],day[3],hour[3];

  /* create a candis file */
  fh = candiscreate("-",0);

  /* get claus file header information -- first, magic number */
  if (fgets(hline,80,stdin) == (char*)NULL) {
    fprintf(stderr,"cdfclaus: unexpected EOF on input\n");
    exit(1);
  }
  if (strncmp(hline,"P5",2) != 0) {
    fprintf(stderr,"cdfclaus: input is not a CLAUS file\n");
    exit(1);
  }

  /* informational lines */
  for (i = 0; i < 7; i++) {
    if (fgets(hline,80,stdin) == (char*)NULL) {
      fprintf(stderr,"cdfclaus: unexpected EOF on input\n");
      exit(1);
    }

    /* replace newlines etc with nulls */
    for (j = 0; j < 80; j++) {
      if (hline[j] == '\n' || hline[j] == '\r' || hline[j] == '\f')
	hline[j] = '\0';
    }
    candiscomment(fh,hline);

    /* extract date information from line 3 */
    if (i == 2) {
      for (j = 0; j < 4; j++) year[j] = hline[j + 17];
      for (j = 0; j < 2; j++) {
	month[j] = hline[j + 21];
	day[j] = hline[j + 23];
	hour[j] = hline[j + 25];
      }
      year[4] = '\0';
      month[2] = '\0';
      day[2] = '\0';
      hour[2] = '\0';
      fyear = atof(year);
      fmonth = atof(month);
      fday = atof(day);
      fhour = atof(hour);
    }
  }

  /* get file dimensions */
  if (fgets(hline,80,stdin) == (char*)NULL) {
    fprintf(stderr,"cdfclaus: unexpected EOF on input\n");
    exit(1);
  }
  sscanf(hline,"%d %d",&lonmax,&latmax);
  if (lonmax == 720 && latmax == 359) hires = 0;
  else if (lonmax == 1080 && latmax == 539) hires = 1;
  else {
    fprintf(stderr,"cdfclaus: incorrect image dimensions %d x %d\n",
	    lonmax,latmax);
    exit(1);
  }

  /* get header end indicator */
  if (fgets(hline,80,stdin) == (char*)NULL) {
    fprintf(stderr,"cdfclaus: unexpected EOF on input\n");
    exit(1);
  }
  if (strncmp(hline,"255",3)) {
    fprintf(stderr,"cdfclaus: header end indicator %s is incorrect\n",hline);
    exit(1);
  }

  /* allocate buffer space for image */
  if ((outbuff = (float*)malloc(sizeof(float)*lonmax*latmax)) ==
      (float*)NULL) {
    fprintf(stderr,"cdfclaus: cannot allocate buffer memory\n");
    exit(1);
  }

  /* make dimension fields */
  if (hires) latlonincr = 0.3333333;
  else latlonincr = 0.5;
  flon = candisdim(fh,"lon",lonmax,0.,latlonincr);
  flat = candisdim(fh,"lat",latmax,-90. + latlonincr,latlonincr);

  /* add date and time fields */
  candisfld(fh,"year",0,0,0,0,0,&fyear);
  candisfld(fh,"month",0,0,0,0,0,&fmonth);
  candisfld(fh,"day",0,0,0,0,0,&fday);
  candisfld(fh,"hour",0,0,0,0,0,&fhour);

  /* transfer the data */
  for (iy = 0; iy < latmax; iy++) {
    for (ix = 0; ix < lonmax; ix++) {
      lon = ix;
      lat = latmax - 1 - iy;
      i = lon*latmax + lat;
      if (i < 0 || i >= lonmax*latmax) {
	fprintf(stderr,"cdfclaus: i = %d out of bounds for lon,lat = %d %d\n",
		i,lon,lat);
	exit(1);
      }
      if ((inchar = getc(stdin)) == EOF) {
	fprintf(stderr,"cdfclaus: unexpected EOF on input data\n");
	exit(1);
      }
      outbuff[i] = 340. - (inchar - 1)*170./254.;
    }
  }

  /* write the brightness temperature field */
  candisfld(fh,"temp",2,flon,flat,0,0,outbuff);

  /* done! */
  candiswrite(fh);
  exit(0);
}
