/* cdfdefint -- This program computes a definite integral on the
 * specified dimension of all fields with that dimension, leaving
 * them with one fewer dimensions.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

static char hbin[HBMAX][LINE],hbout[HBMAX][LINE]; /* header buffers */
static char pline[LINE],cline[LINE];     /* line buffers */
static float *sbin,*sbout,*vbin,*vbout;  /* pointers to slice buffers */
static long nsbin,nsbout,nvbin,nvbout;   /* lengths of slice buffers */
static float bad,badlim;                 /* bad data stuff */
static int ds[4];                        /* dimension sizes */
static char dn[4][LINE];                 /* dimension names */
static long goop,poop,loop;              /* looping variables */
static long g1,g2,g3,g4;                 /* temporary indices */
static long nflds;                       /* number of variable fields */
static int nlevels;                      /* number of good int levels */
static struct field *fp,*ifld;           /* field structure pointer */
static int nstat,nvar;                   /* numbers of fields */
static float *sfin,*vfin[HBMAX];         /* input field pointers */
static float *sfout,*vfout[HBMAX];       /* output field pointers */
static char smod[HBMAX],vmod[HBMAX];     /* modification flags */
static struct field ft[HBMAX];           /* table of field structures */
static long start1,incr1,size1;          /* subspace1 arguments */
static float *deltadim;                  /* index field */
static float sum,element;                /* temporaries */
static char rmdim();                     /* dimension removing function */

main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc != 2) {
    fprintf(stderr,"Usage: cdfdefint dimension_name\n");
    exit(1);
  }
/* get header buffer from standard input */
  gethdr(stdin,hbin,HBMAX);
/* check for float type */
  if (getfmt(hbin,HBMAX) != 'f') {
    fprintf(stderr,"cdfdefint: float format required\n");
    exit(1);
  }
/* allocate in buffers */
  nsbin = elemcnt(hbin,HBMAX,'s');
  sbin = getbuff(nsbin);
  nvbin = elemcnt(hbin,HBMAX,'v');
  vbin = getbuff(nvbin);
/* get static slice */
  getslice(stdin,nsbin,sbin);
/* get bad data values */
  if (seekpar(hbin,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbin,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;
/* compute deltadim from index field */
  sprintf(cline,"d%s",argv[1]);
  if ((ifld = seekfld(hbin,HBMAX,'s',argv[1])) == NULL) {
    fprintf(stderr,"cdfdefint: dimension %s doesn't exist\n",argv[1]);
    exit(1);
  }
  if (ifld->tsize < 2 || ifld->dim != 1) {
    fprintf(stderr,"cdfdefint: field %s too small or not one dimensional\n",
	    argv[1]);
    exit(1);
  }
  deltadim = &sbin[ifld->ptr];

/* make new header */
  nullhdr(hbout,HBMAX,"float");
  copycmt(hbout,HBMAX,hbin,HBMAX);
  sprintf(cline,"cdfdefint %s\n",argv[1]);
  addcline(hbout,HBMAX,cline);
  copypar(hbout,HBMAX,hbin,HBMAX);
  nstat = 0;
  while ((fp = getfld(hbin,HBMAX,'s',nstat)) != NULL) {
    smod[nstat++] = rmdim(fp,'s',argv[1]);
  }
  nvar = 0;
  while ((fp = getfld(hbin,HBMAX,'v',nvar)) != NULL) {
    vmod[nvar++] = rmdim(fp,'v',argv[1]);
  }
/* write new header */
  puthdr(stdout,hbout,HBMAX);
/* out allocate buffers */
  nsbout = elemcnt(hbout,HBMAX,'s');
  sbout = getbuff(nsbout);
  nvbout = elemcnt(hbout,HBMAX,'v');
  vbout = getbuff(nvbout);
/* transform static slice data */
  loop = 0;
  while ((fp = getfld(hbin,HBMAX,'s',loop)) != NULL) {
    strcpy(cline,fp->fname);
    sfout = getptr(hbout,HBMAX,sbout,'s','d',cline);
    sfin = getptr2(hbin,HBMAX,sbin,'s','d',cline,&fp);
/* modified field -- integrate specified dimension */
    if (smod[loop++]) {
      poop = 0;
      while(subspace1(fp,argv[1],poop,&start1,&incr1,&size1) != FAIL) {
	if (size1 <= 1) {
	  fprintf(stderr,"cdfdefint: integrated dimension size = 1\n");
	  exit(1);
	}
        sum = 0.;
        nlevels = 0;
        for (goop = 0; goop < size1; goop++) {
          element = sfin[start1 + incr1*goop];
          if (fabs(element) < badlim) {
	    g1 = goop - 1;
	    g2 = goop;
	    if (g1 < 0) {
	      g1 = 0;
	      g2 = 1;
	    }
	    g3 = goop;
	    g4 = goop + 1;
	    if (g4 >= size1) {
	      g4 = size1 - 1;
	      g3 = size1 - 2;
	    }
            sum += 0.5*element*(deltadim[g4] - deltadim[g3] +
			       deltadim[g2] - deltadim[g1]);
            nlevels++;
          }
        }
        if (nlevels > 0) sfout[poop++] = sum;
        else sfout[poop++] = bad;
      }
    }
/* unmodified field -- simple transfer */
    else {
      for (poop = 0; poop < fp->tsize; poop++) sfout[poop] = sfin[poop];
    }
  }
/* put the modified static slice */
  putslice(stdout,nsbout,sbout);
/* compile information on variable slice fields */
  nflds = 0;
  while((fp = getfld(hbin,HBMAX,'v',nflds)) != NULL) {
    vfout[nflds] = getptr(hbout,HBMAX,vbout,'v','d',fp->fname);
    vfin[nflds] = getptr(hbin,HBMAX,vbin,'v','d',fp->fname);
    ft[nflds++] = *fp;
  }
/* loop on variable slices */
  while (getslice(stdin,nvbin,vbin) != EOF) {
/* transform variable slice data */
    for (loop = 0; loop < nflds; loop++) {
/* modified field -- integrate specified dimension */
      if (vmod[loop]) {
        poop = 0;
        while(subspace1(&ft[loop],argv[1],poop,&start1,&incr1,&size1)
	      != FAIL) {
          sum = 0.;
          nlevels = 0;
          for (goop = 0; goop < size1; goop++) {
            element = vfin[loop][start1 + incr1*goop];
            if (fabs(element) < badlim) {
	      g1 = goop - 1;
	      g2 = goop;
	      if (g1 < 0) {
		g1 = 0;
		g2 = 1;
	      }
	      g3 = goop;
	      g4 = goop + 1;
	      if (g4 >= size1) {
		g4 = size1 - 1;
		g3 = size1 - 2;
	      }
	      sum += 0.5*element*(deltadim[g4] - deltadim[g3] +
			       deltadim[g2] - deltadim[g1]);
              nlevels++;
            }
          }
          if (nlevels > 0) vfout[loop][poop++] = sum;
          else vfout[loop][poop++] = bad;
        }
      }
/* unmodified field -- simple transfer */
      else {
        for (poop = 0; poop < ft[loop].tsize; poop++) {
	  vfout[loop][poop] = vfin[loop][poop];
	}
      }
    }
/* put the variable slice */
    putslice(stdout,nvbout,vbout);
  }
  exit(0);
}

/* rmdim -- remove a dimension if it exists and add modified field
 * to output.
 */
static char rmdim(fp,type,dimname)
struct field *fp;
char type;
char *dimname;
{
  char flag;
  int dimout;
/* work through dimensions */
  dimout = 0;
  flag = 0;
  if (fp->dim > 0) {
    if (strcmp(dimname,fp->dname1) == 0) {
      flag = 1;
    }
    else {
      ds[dimout] = fp->dsize1;
      strcpy(dn[dimout++],fp->dname1);
    }
  }
  if (fp->dim > 1) {
    if (strcmp(dimname,fp->dname2) == 0) {
      flag = 1;
    }
    else {
      ds[dimout] = fp->dsize2;
      strcpy(dn[dimout++],fp->dname2);
    }
  }
  if (fp->dim > 2) {
    if (strcmp(dimname,fp->dname3) == 0) {
      flag = 1;
    }
    else {
      ds[dimout] = fp->dsize3;
      strcpy(dn[dimout++],fp->dname3);
    }
  }
  if (fp->dim > 3) {
    if (strcmp(dimname,fp->dname4) == 0) {
      flag = 1;
    }
    else {
      ds[dimout] = fp->dsize4;
      strcpy(dn[dimout++],fp->dname4);
    }
  }
/* add new field entry */
  addfld(hbout,HBMAX,type,fp->fname,fp->smul,fp->sadd,fp->bytes,dimout,
       dn[0],ds[0],dn[1],ds[1],dn[2],ds[2],dn[3],ds[3]);
/* return flag */
  return(flag);
}
