/* cdfderiv -- Take the derivative of a specified field with respect
 * to a specified index field.
 */

#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"
#define TINY 1.e-30

static char *hbuff[HBMAX][LINE];  /* header buffer */
static char cline[LINE];          /* character line buffer */
static long nsbuff,nvbin,nvbout;  /* slice buffer sizes */
static float *sbuff,*vbuff;       /* slice buffer pointers */
static float *ifield;             /* pointer to specified index field */
static float *sfield;             /* pointer to specified field */
static float *dfield;             /* pointer to derivative field */
static long instance;             /* looping var over instances */
static long loop;                 /* the other looping variable */
static struct field *fp;          /* field structure temporary pointer */
static struct field *fps;         /* field structure pointer for specified
                                     field */
static float bad,badlim;          /* bad data values */
static long start,incr,size;      /* loop variables */
static float den;                 /* derivative temporaries */
static long i1,i2,ix1,ix2;        /* index temporaries */
static char *argv1, *argv2, *argv3;  /* for input argument processing */
static int cflag, rflag;          /* flags for cyclic and repeat cyclic case */


main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc < 4 || argc > 5) {
    fprintf(stderr,"Usage: [-c|r] cdfderiv derived_field input_field dimension\n");
    exit(1);
  }
/* get header */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfderiv: float format expected\n");
    exit(1);
  }
/* get bad data values */
  if (seekpar(hbuff,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbuff,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;
/* compute input buffer lengths */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  nvbin = elemcnt(hbuff,HBMAX,'v');
/* add stuff to header, comment first */
  if (argc == 4) {
    sprintf(cline,"%s: %s %s %s\n",argv[0],argv[1],argv[2],argv[3]);
    argv1 = argv[1];
    argv2 = argv[2];
    argv3 = argv[3];
  }
  else {
    sprintf(cline,"%s: %s %s %s %s\n",argv[0],argv[1],argv[2],argv[3],argv[4]);
    argv1 = argv[2];
    argv2 = argv[3];
    argv3 = argv[4];
    if (strcmp(argv[1],"-c") == 0) cflag = 1;
    else if (strcmp(argv[1],"-r") == 0) rflag = 1;
    else {
      fprintf(stderr,"cdfderiv: option %s not recognized\n",argv[1]);
      exit(1);
    }
  }
  addcline(hbuff,HBMAX,cline);
/* derivative field next -- same dimensions as specified field */
  if ((fp = seekfld(hbuff,HBMAX,'v',argv2)) == NULL) {
    fprintf(stderr,"cdfderiv: %s field not found in variable slice\n",argv2);
    exit(1);
  }
  addfld(hbuff,HBMAX,'v',argv1,fp->smul,fp->sadd,fp->bytes,fp->dim,
       fp->dname1,fp->dsize1,
       fp->dname2,fp->dsize2,
       fp->dname3,fp->dsize3,
       fp->dname4,fp->dsize4);
/* allocate buffers */
  nvbout = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbuff);
  vbuff = getbuff(nvbout);
/* assign field and structure pointers */
  sfield = getptr2(hbuff,HBMAX,vbuff,'v','d',argv2,&fps);
  dfield = getptr(hbuff,HBMAX,vbuff,'v','d',argv1);
  if ((ifield = getptr2(hbuff,HBMAX,sbuff,'s','r',argv3,&fp)) == NULL) {
    fprintf(stderr,"cdfderiv: %s field doesn't exist\n",argv3);
    exit(1);
  }
  if ((fp->dim != 1) || (strcmp(fp->fname,fp->dname1) != 0)) {
    fprintf(stderr,"cdfderiv: %s field is not an index field\n",argv3);
    exit(1);
  }
/* write header */
  puthdr(stdout,hbuff,HBMAX);
/* transfer static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);
/* loop on variable slices */
  while (getslice(stdin,nvbin,vbuff) != EOF) {
/* loop on subspace instances */
    instance = 0;
    while (subspace1(fps,argv3,instance,&start,&incr,&size) != FAIL) {
      if (size < 2) {
	fprintf(stderr,"cdfderiv: size of active dimension must be > 1\n");
	exit(1);
      }
/* loop over points and do centered differences except on ends in non-
 * cyclic case */
      for (loop = 0; loop < size; loop++) {
	if (loop == 0) {
	  if (cflag) {
	    i1 = start + incr*(size - 1);
	    ix1 = size - 1;
	  }
	  else if (rflag) {
	    i1 = start + incr*(size - 2);
	    ix1 = size - 2;
	  }
	  else {
	    i1 = start + incr*loop;
	    ix1 = loop;
	  }
	  i2 = start + incr*(loop + 1);
	  ix2 = loop + 1;
	}
	else if (loop == size - 1) {
	  i1 = start + incr*(loop - 1);
	  ix1 = loop - 1;
	  if (cflag) {
	    i2 = start;
	    ix2 = 0;
	  }
	  else if (rflag) {
	    i2 = start + incr;
	    ix2 = 1;
	  }
	  else {
	    i2 = start + incr*loop;
	    ix2 = loop;
	  }
	}
	else {
	  i1 = start + incr*(loop - 1);
	  i2 = start + incr*(loop + 1);
	  ix1 = loop - 1;
	  ix2 = loop + 1;
	}
	if (rflag || cflag) {
	  den = ifield[2] - ifield[0];
	}
	else {
	  den = ifield[ix2] - ifield[ix1];
	}
	if (fabs(den) < TINY) divbyzero(instance);
	if (fabs(sfield[i1]) < badlim && fabs(sfield[i2]) < badlim) {
	  dfield[start + incr*loop] = (sfield[i2] - sfield[i1])/den;
	}
	else {
	  dfield[start + incr*loop] = bad;
	}
      }
/* increment instance */
      instance++;
    }
/* write variable slice */
    putslice(stdout,nvbout,vbuff);
  }
  exit(0);
}

divbyzero(instance)
int instance;
{
  fprintf(stderr,"cdfderiv: zero index field interval -- calc %d\n",instance);
  exit(1);
}
