/* cdffill -- Fill in bad data gaps by linear interpolation. */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbin[HBMAX][LINE],hbout[HBMAX][LINE];  /* header buffers */
static char cline[LINE];                       /* line buffer */
static float *sbin,*vbin;                      /* slice buffers */
static long nsbin,nvbin;                       /* slice buffer lengths */
static struct field *fp;                       /* field pointer temporary */
static char *dimname;                          /* name of dimension */
static long loop,poop,goop;                    /* looping indices */
static long instance;                          /* loop index for subspace1 */
static long instart,inincr,insize;             /* input subspace1 parameters */
static float bad,badlim;                       /* bad data values */
static float *work;                            /* working field pointer */
static long badflag,good;                   /* flags for interpolation test */
static float work0,work1,frac;                 /* temps for interpolation */

main (argc,argv)
int argc;
char *argv[];
{

/* check args */
  if (argc != 2) {
    fprintf(stderr,"Usage: cdffill dimension\n");
    exit(1);
  }
  dimname = argv[1];

/* get a header from standard input */
  gethdr(stdin,hbin,HBMAX);
  if (getfmt(hbin,HBMAX) != 'f') {
    fprintf(stderr,"cdfinterp: float format expected\n");
    exit(1);
  }

/* get bad data values */
  if (seekpar(hbin,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbin,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;

/* check for existence of index field of interest */
  if ((fp = seekfld(hbin,HBMAX,'s',dimname)) == NULL) {
    fprintf(stderr,"cdfinterp: can't find index field %s\n",dimname);
    exit(1);
  }
  if (fp->dim != 1 || strcmp(fp->fname,fp->dname1) != 0) {
    fprintf(stderr,"cdfinterp: %s not an index field\n",dimname);
    exit(1);
  }

/* make a new header */
  nullhdr(hbout,HBMAX,"float");

/* comments */
  copycmt(hbout,HBMAX,hbin,HBMAX);
  sprintf(cline,"cdffill: %s\n",argv[1]);
  addcline(hbout,HBMAX,cline);

/* copy parameters */
  copypar(hbout,HBMAX,hbin,HBMAX);

/* static fields */
  copyfld(hbout,HBMAX,hbin,HBMAX,'s');

/* variable fields */
  copyfld(hbout,HBMAX,hbin,HBMAX,'v');

/* make slice buffers */
  nsbin = elemcnt(hbin,HBMAX,'s');
  sbin = getbuff(nsbin);
  nvbin = elemcnt(hbin,HBMAX,'v');
  vbin = getbuff(nvbin);

/* write new header to standard output */
  puthdr(stdout,hbout,HBMAX);

/* read static slice from standard input and write to standard output */
  getslice(stdin,nsbin,sbin);
  putslice(stdout,nsbin,sbin);

/* loop on variable slices */
  while (getslice(stdin,nvbin,vbin) != EOF) {

/* loop of variable fields */
    loop = 0;
    while ((fp = getfld(hbin,HBMAX,'v',loop++)) != NULL) {

/* loop on instance */
      instance = 0;
      while (subspace1(fp,dimname,instance++,
           &instart,&inincr,&insize) != FAIL) {
        work = getptr(hbin,HBMAX,vbin,'v','d',fp->fname);

/* loop on dimension of interest */
        good = -1;
        badflag = 0;
        for (poop = 0; poop < insize; poop++) {

/* if datum is good */
          if (fabs(work[instart + inincr*poop]) < bad) {

/* if previous bad data */
            if (badflag) {

/* exclude case where first datum is bad */
              if (good >= 0) {

/* interpolate! */
                work0 = work[instart + inincr*good];
                work1 = work[instart + inincr*poop];
                for (goop = good + 1; goop < poop; goop++) {
                  frac = (float)(goop - good)/(float)(poop - good);
                  work[instart + inincr*goop] = frac*work1 + (1. - frac)*work0;
                }
              }
              badflag = 0;
            }
            good = poop;
          }

/* if datum is bad, set bad data flag */
          else badflag = 1;
        }
      }
    }

/* write the new variable slice */
    putslice(stdout,nvbin,vbin);
  }
  exit(0);
}
