/* cdfhisto -- This program makes histograms. */

#include <stdio.h> 
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define LIMIT 100      /* Maximum size of command line argument.   */
#define LOW 0          /* dvals element that stores the lowest     */
/* range value.                             */
#define INC 2          /* dvals element that stores the range inc- */
/* crement.                                 */
#define NUM 1          /* dvals element that stores the number of  */
/* divisions in the range.                  */

/* Candis variables */
static char hbuff[HBMAX][LINE]; /* Buffer containing the header           */
static char newhbuff[HBMAX][LINE]; /* Buffer containing the histogram file*/
/* header.                             */
static char c1[LINE];           /* A character buffer that is used to     */
/* construct entries for the comment and  */
/* parameter sections of the header.      */
static float *sbuff, *vbuff;    /* Pointers to the static and variable    */
/* field buffers.                         */
static float *newsbuff, *newvbuff; /* Pointers to the static variable     */
/* field buffers of the histogram.     */
static long nsbuff, nvbuff;     /* Sizes (in elements) of static and      */
/* variable buffers.                      */
static long newnsbuff, newnvbuff; /* Sizes (in elements) of static and    */
/* variable buffers for the histogram.  */
static float bad, badlim;       /* Values of bad data parameters.         */
static int kflag;               /* copy input flag                        */
static struct field *fp,*hptr;
/* size of histogram */
static long hsize;

/* Cdfhisto functions  */
static void usage();                /* Prints an error message and reports*/
/* the appropriate format of the command line. */
static void getcmdline();         /* Gets the command line arguments from */
/* argv and stores them in nfields, hname,*/
/* fnams, dvals.                        */
static long calclistpos();        /* Given a coordinate in a 4x4 matrix,  */
/* calclistpos calculates where this    */
/* value should be placed in a linear   */
/* representation of the array.         */

main(argc, argv)
int argc;
char *argv[];
{
  /* Cdfhisto variables */
  long i,fnum, dpos;       /* Variable used in for-next loops.     */
  long nfields;            /* Number of dimensions in the histogram.*/
  char hname[LIMIT];       /* Histogram filename.                  */
  char fnams[4][LIMIT];    /* Names of the candidate dimensions.   */
  char gnams[4][LIMIT];    /* Actual names of the dimensions.      */
  float dvals[4][3];       /* Low and high ends of the histogram   */
  /* ranges and the increments.           */
  float *fieldptr[4];      /* Pointers to the variable fields.     */
  float *dimptr[4];        /* Pointers to the histogram dimensions.*/
  float *histo;            /* The histogram.                       */
  long position;            /* A position in the histogram (where   */
  /* the dimensions coincide).            */
  static long cords[4] ={
    0,0,0,0 };             /* A coordinate in an 4x4 array.        */
  char strbuf[LIMIT];      /* Used to convert longs in strings.    */
  char strbuf2[LIMIT];
  int inrange;             /* 1 if in range, 0 if out of range.    */
  long dsize1, dsize2, dsize3, dsize4;      /* Field dimension info.*/
  long tsize;                  /* The total size of a given field. */

  /* Get command line arguments */
  getcmdline(argc, argv, &nfields, hname, fnams, gnams, dvals);

  /* get header from stdin */
  gethdr(stdin, hbuff, HBMAX);

  /* check format */
  if (getfmt(hbuff, HBMAX) != 'f')
  {
    fprintf(stderr, "cdfhisto: wrong format type\n");
    exit(1);
  }

  /* check validity of dimension names */
  for (fnum = 0; fnum < nfields; ++fnum)
  {
    if (strcmp(fnams[fnum], "") != 0)
    {
      if ((fp = seekfld(hbuff, HBMAX, 'v', fnams[fnum])) == NULL)
      {
        fprintf(stderr,"cdfhisto: %s is not a valid dimension name\n",
             fnams[fnum]);
        exit(1);
      }
    }
  }

  /* Check field sizes and dimensions by storing the sizes of the. */
  /* dimensions of the first field and comparing them to the       */
  /* remaining fields.                                             */
  fp = seekfld(hbuff, HBMAX, 'v', fnams[0]);
  dsize1 = fp->dsize1;
  dsize2 = fp->dsize2;
  dsize3 = fp->dsize3;
  dsize4 = fp->dsize4;
  tsize = fp->tsize;
  for (fnum = 1; fnum < nfields; ++fnum)
  {
    fp = seekfld(hbuff, HBMAX, 'v', fnams[fnum]);
    if ((fp->dsize1 != dsize1) || (fp->dsize2 != dsize2)
        || (fp->dsize3 != dsize3) || (fp->dsize4 != dsize4))
    {
      fprintf(stderr,"cdfhisto: dimensions are not same size.");
      exit(1);
    }
  }

  /* Allocate slice buffers */
  nsbuff = elemcnt(hbuff, HBMAX, 's');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuff, HBMAX, 'v');
  vbuff = getbuff(nvbuff);

  /* Read the static slice */
  getslice(stdin, nsbuff, sbuff);

  /* Get pointers to the variable fields */
  for (fnum = 0; fnum < nfields; ++fnum) {
    fieldptr[fnum] = getptr(hbuff, HBMAX, vbuff, 'v', 'd', fnams[fnum]);
  }

  /* Make a header for the histogram file. */
  nullhdr(newhbuff, HBMAX, "float");

  /* Add comments to the histogram header after copying old comments. */
  copycmt(newhbuff, HBMAX, hbuff, HBMAX);
  sprintf(strbuf,"%s %s %s\n", argv[0+kflag], argv[1+kflag], argv[2+kflag]);
  addcline(newhbuff, HBMAX, strbuf);
  for (fnum = 0; fnum < nfields; ++fnum)
  {
    sprintf(strbuf2, "  %s %s %s %s\n", argv[(fnum*4)+3+kflag],
        argv[(fnum*4)+4+kflag],argv[(fnum*4)+5+kflag], argv[(fnum*4)+6+kflag]);
    addcline(newhbuff, HBMAX, strbuf2);
  }

  /* If -k set, then copy params and fields as well */
  if (kflag) {
    copypar(newhbuff,HBMAX,hbuff,HBMAX);
    copyfld(newhbuff,HBMAX,hbuff,HBMAX,'s');
    copyfld(newhbuff,HBMAX,hbuff,HBMAX,'v');
  }

  /* Add index parameters to histogram header. */
  for (fnum = 0; fnum < nfields; ++fnum)
  {
    sprintf(strbuf, "%f", dvals[fnum][INC]);
    sprintf(strbuf2, "d%s", gnams[fnum]);
    addpar(newhbuff, HBMAX, strbuf2, strbuf);
    sprintf(strbuf, "%f", dvals[fnum][LOW] + .5*dvals[fnum][INC]);
    sprintf(strbuf2, "%s0", gnams[fnum]);
    addpar(newhbuff, HBMAX, strbuf2, strbuf);
  }

  /* Add index fields to static slice in reverse order. */
  for (fnum = nfields - 1 ; fnum >= 0; fnum--)
    addfld(newhbuff, HBMAX, 's', gnams[fnum], 1000., 0., 's', 1,
        gnams[fnum], (int) (dvals[fnum][NUM]));

  /* Add histogram field to the variable slice.  The index fields
   * are in reverse order.  (Why?) */
  if (nfields == 1)
    addfld(newhbuff, HBMAX, 'v', hname, 1000., 0., 's', 1,
        gnams[0], (int) (dvals[0][NUM]));
  else if (nfields == 2)
    addfld(newhbuff, HBMAX, 'v', hname, 1000., 0., 's', 2,
        gnams[1], (int) (dvals[1][NUM]), gnams[0], (int) (dvals[0][NUM]));
  else if (nfields == 3)
    addfld(newhbuff, HBMAX, 'v', hname, 1000., 0., 's', 3,
        gnams[2], (int) (dvals[2][NUM]), gnams[1], (int) (dvals[1][NUM]),
        gnams[0], (int) (dvals[0][NUM]));
  else if (nfields == 4)
    addfld(newhbuff, HBMAX, 'v', hname, 1000., 0., 's', 4,
        gnams[3], (int) dvals[3][NUM], gnams[2], (int) dvals[2][NUM],
        gnams[1], (int) dvals[1][NUM], gnams[0], (int) dvals[0][NUM]);

  /* Allocate space for slice buffers. */
  newnsbuff = elemcnt(newhbuff, HBMAX, 's');
  newsbuff = getbuff(newnsbuff);
  newnvbuff = elemcnt(newhbuff, HBMAX, 'v');
  newvbuff = getbuff(newnvbuff);

  /* If -k flag set, copy over old stuff in static slice */
  if (kflag) for (i = 0; i < nsbuff; i++) newsbuff[i] = sbuff[i];

  /* Allocate pointers to the dimensions. */
  for (fnum = 0; fnum < nfields; ++fnum)
  {
    dimptr[fnum] = getptr(newhbuff, HBMAX, newsbuff, 's', 'd', gnams[fnum]);
  }

  /* Get histogram pointer */
  histo = getptr2(newhbuff, HBMAX, newvbuff, 'v', 'd', hname, &hptr);
  hsize = hptr->tsize;

  /* Fill in values of index fields. */
  /* the points should be centered in the bins thus the 1/2 */
  for (fnum = 0; fnum < nfields; ++fnum)
  {
    for (position = 0; position < (int) (dvals[fnum][NUM]); ++position)
    {
      (dimptr[fnum])[position]=dvals[fnum][LOW]+dvals[fnum][INC]*(position+.5);
    }
  }

  /* Write histogram header to histogram file. */
  puthdr(stdout, newhbuff, HBMAX);

  /* Write histogram static slice to histogram file. */
  putslice(stdout, newnsbuff, newsbuff);

  /* Fill histogram */
  while (getslice(stdin, nvbuff, vbuff) != EOF)
  {

    /* But first transfer variable slice stuff if -k */
    if (kflag) {
      for (i = 0; i < nvbuff; i++) {
	newvbuff[i] = vbuff[i];
      }
    }

    /* Zero histogram */
    for (i = 0; i < hsize; i++) histo[i] = 0.;

    /* Then do the rest */
    for (dpos = 0; dpos < tsize; ++dpos)
    {
      for (fnum = 0; fnum < nfields; ++fnum)
      {
        if ((fieldptr[fnum][dpos] > dvals[fnum][LOW])
            && (fieldptr[fnum][dpos] 
            < (dvals[fnum][INC]*dvals[fnum][NUM]+dvals[fnum][LOW])))
        {
          cords[fnum] = (long) ((fieldptr[fnum][dpos] - 
              dvals[fnum][LOW])/dvals[fnum][INC]);
          inrange = 1;
        }
        else 
        {
          inrange = 0;
        }
      }

      if (inrange == 1)
      {
        position = calclistpos(cords, dvals, nfields);
        histo[position] += 1.;
      }
    }

    /* Write histogram to variable slice. */
    putslice(stdout, newnvbuff, newvbuff);
  }
  exit(0);
}

static void usage()
{
  fprintf(stderr,
	  "Usage: cdfhisto [-k] h_dim h_name field1 low1 num1 incr1 ...\n");
  exit(1);
}

static void getcmdline(argc, argv, nfields, hname, fnams, gnams, dvals)
int argc;
char *argv[];
long *nfields;
char hname[LIMIT];
char fnams[4][LIMIT];
char gnams[4][LIMIT];
float dvals[4][3];
{
  long count;         /* Used as a variable in for-next loops etc.    */
  /* Check command line arguments */
  if (argc > 1)
  {
    /* check if we need to keep input fields */
    if(strcmp(argv[1],"-k") == 0 ) kflag = 1;
    else kflag = 0;
    *nfields = atoi(argv[1+kflag]);
    if ((*nfields >= 1)  && (*nfields <= 4))
    {
      if (argc != ((*nfields*4)+3+kflag))
        usage();
    }
    else usage();
  }
  else usage();

  /* Other error checking could be included here, such as making sure*/
  /* arguments are numbers when they are supposed to be numbers, that*/
  /* there is an appropriate filename etc.                           */
  /* Getcmdln variables */

  strcpy(hname, argv[2+kflag]);

  for (count = 0; count < *nfields; ++count)
  {
    /* Copy the names of 1 to 4 dimensions into the fnams array.   */
    strcpy(fnams[count], argv[(count*4)+3+kflag]);
    strcpy(gnams[count],fnams[count]);
    strcat(gnams[count],"-dim");

    /* Copy the lower, upper bounds and increments into the dvals array*/
    dvals[count][LOW] = atof(argv[(count*4)+4+kflag]);
    dvals[count][NUM] = atof(argv[(count*4)+5+kflag]);
    dvals[count][INC] = atof(argv[(count*4)+6+kflag]);
  }
}

static long calclistpos(cords, dvals, nfields)
long cords[4];
float dvals[4][3];
long nfields;
{
  long x, y, z, w, nx, ny, nz;

  x = cords[0];
  y = cords[1];
  z = cords[2];
  w = cords[3];
  nx = (int) (dvals[0][NUM]);
  ny = (int) (dvals[1][NUM]);
  nz = (int) (dvals[2][NUM]);

  if (nfields == 1)
    return(x);
  else if (nfields == 2)
    return((y)*nx+(x));
  else if (nfields == 3)
    return((z)*(nx*ny)+(y)*nx+(x-1));
  else
    return((w)*(nx*ny*nz)+(z)*(nx*ny)+(y)*(nx)+(x));

}
