/* cdflagr -- Lagrangian trajectory calculator.
 * Modified 30 Aug 89 so integration is done both forward and backward
 * in time.  This program is getting baroque.
 */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define VSMAX 100          /* max number of variable slices */
#define IFMAX 3            /* max number of index fields */
#define NFMAX 100          /* max number of variable fields */
#define I(i,j,k) (coef[0]*i + coef[1]*j + coef[2]*k)
                           /* eulerian field index macro */
#define J(i,j) (i*nt + j)  /* lagrangian field index macro */

/* command line stuff */
static int nt;                    /* number of time points to be computed */
static int ntplus,ntminus;    /* time extension in plus and minus directions */
static float tstart,dt;           /* starting time, time interval */

/* header information */
static char hbin[HBMAX][LINE],hbout[HBMAX][LINE];  /* header buffers */
static char cb[HBMAX/2][LINE];    /* comment buffer */
static char alpha[LINE];          /* line buffer */

/* slice variables */
static float *sbin,*sbout;        /* static slice buffers */
static float *vin[VSMAX],*vout;   /* variable slice buffers */
static int nvs;                   /* number of variable slices */
static int ivs1,ivs2;             /* variable slice indices */
static long nsin,nsout,nvin,nvout;   /* slice lengths */

/* input index field information */
static char ifld[IFMAX][LINE];    /* index field names */
static int indexfld,is[IFMAX];       /* # index fields, sizes */

/* various looping indices and temporaries */
static int loop,poop,goop,temp;   /* general looping indices and temporaries */
static long lloop;                /* long loop index (just in case!) */
static int i1,i2,i3,jj1,j2,j3,k1,k2,k3;
                         /* looping variables for euler array access */
static int ix,iy,iz;              /* eulerian locators of lagrangian fields */
static float wx,wy,wz,wt;      /* weighted remainders for eulerian location */
static int iparcel;               /* parcel index */
static int nparcels;        /* number of parcel trajectories to be computed */
static int ngenflds;              /* number of general variable fields */
static int it;                    /* time index */
static int rs[IFMAX];             /* reduction indices */
static int coef[IFMAX];        /* coefficients for evaluating field indices */
static float ctime1,ctime2;       /* lag and lead times */
static float lu,lv,lw;            /* interpolated velocities */
static float time;                /* time loop temporary */
static float limit,redval,pos;    /* temporary floats */
static struct field *fp;          /* pointer to field structure */
static float bad,badlim;          /* bad data value and limit */

/* pointers to fields - these can be treated as 1-D arrays after allocation */
static float *u1,*v1,*w1,*u2,*v2,*w2;  /* lag and lead velocity fields */
static float *gen1,*gen2;         /* lag and lead general field */
static float *lx,*ly,*lz,*lgen;   /* lagrangian fields */
static float *ex,*ey,*ez;         /* input index fields */
static char *good;                /* flag indicating good parcel */

/* offsets are positions of fields relative to start of slice */
static int ifptr[IFMAX];          /* input index field offsets */
static int vptr[IFMAX];           /* offsets of input velocity fields */
static int tptr;                  /* offset of input time field */
static int inoff[NFMAX];          /* general input field offsets */
static int outoff[NFMAX];         /* general output field offsets */
static int xoff[IFMAX];           /* output position offsets */

/* function declarations */
static float interpol();

main(argc,argv)
int argc;
char *argv[];
{
/* simple argument check (more later) */
  if (argc < 7) {
    fprintf(stderr,"Usage: cdflagr u [v [w]] tname t dt ntminus ntplus");
    fprintf(stderr," [rname rval ... ]\n");
    exit(1);
  }
/* get header from standard input */
  gethdr(stdin,hbin,HBMAX);
  if (getfmt(hbin,HBMAX) != 'f')
       die("cdflagr: input file must be float format\n");
/* extract bad and badlim from parameter section of header */
  if (seekpar(hbin,HBMAX,"bad",alpha) == OK) bad = atof(alpha);
  else bad = BAD;
  if (seekpar(hbin,HBMAX,"badlim",alpha) == OK) badlim = atof(alpha);
  else badlim = BADLIM;
/* get static slice */
  nsin = elemcnt(hbin,HBMAX,'s');
  nvin = elemcnt(hbin,HBMAX,'v');
  if ((sbin = (float *) malloc(sizeof(float)*nsin)) == NULL)
       die("cdflagr: can't get enough memory\n");
  getslice(stdin,nsin,sbin);
/* extract info on index fields */
  for (loop = 0; loop < IFMAX; loop++) ifptr[loop] = 0;
  loop = indexfld = 0;
  while ((fp = getfld(hbin,HBMAX,'s',loop++)) != NULL) {
/* must be one-dimensional */
    if (fp->dim != 1) continue;
/* must have name and dimension name agree (definition of index field) */
    if (strcmp(fp->fname,fp->dname1) != 0) continue;
/* got an index field */
    if (indexfld >= IFMAX) die("cdflagr: too many index fields\n");
    strcpy(ifld[indexfld],fp->fname);
    is[indexfld] = fp->dsize1;
    ifptr[indexfld] = fp->ptr;
    indexfld++;
  }
  if (indexfld < 1) die("cdflagr: not enough index fields\n");
/* initialize index field pointers */
  ex = sbin + ifptr[0];
  ey = sbin + ifptr[1];
  ez = sbin + ifptr[2];
/* interpret command line */
  if (argc < indexfld + 6) die("cdflagr: not enough command line args\n");
/* velocities first */
  for (loop = 0; loop < IFMAX; loop++) vptr[loop] = 0;
  for (loop = 0; loop < indexfld; loop++) {
    if ((fp = seekfld(hbin,HBMAX,'v',argv[loop + 1])) == NULL) {
      fprintf(stderr,"cdflagr: can't find velocity field %s\n",argv[loop+1]);
      exit(1);
    }
    if (fp->dim != indexfld) die("cdflagr: inconsistent dimensions in header\n");
    vptr[loop] = fp->ptr;
  }
/* time field name */
  if ((fp = seekfld(hbin,HBMAX,'v',argv[indexfld + 1])) == NULL)
       die("cdflagr: requested time field name not defined\n");
  tptr = fp->ptr;
/* time auxiliary variables */
  tstart = atof(argv[indexfld + 2]);
  dt = atof(argv[indexfld + 3]);
  if (dt <= 0.) die("cdflagr: dt must be positive\n");
  ntminus = atoi(argv[indexfld + 4]);
  ntplus = atoi(argv[indexfld + 5]);
  if (ntplus < 0 || ntminus > 0) {
    die("cdflagr: must have ntminus <= 0 and ntplus >= 0\n");
  }
  if (ntminus == 0) ntminus = -1;
  if (ntplus == 0) ntplus = 1;
  ntminus++;
  nt = ntplus - ntminus;
/* remaining arguments are reduce fields and values */
  temp = argc - indexfld - 6;
  if (temp % 2 != 0) die("cdflagr: reduce field doesn't have a value\n");
  if (temp > 6) die("cdflagr: too many reduce fields\n");
/* initialize reduction indices and index coeffs to zero */
  for (loop = 0; loop < IFMAX; loop++) {
    rs[loop] = 0;
    coef[loop] = 0;
  }
/* loop on possible reduce fields (just index fields that have been found) */
  for (loop = 0; loop < indexfld; loop++) {
/* no match indicated by -1 indexfld */
    rs[loop] = -1;
/* loop on requested reduction fields */
    for (poop = indexfld + 6; poop < argc; poop += 2) {
      redval = atof(argv[poop + 1]);
/* check for a match */
      if (strcmp(argv[poop],ifld[loop]) == 0) {
/* found match -- now find index field index value */
        limit = 1.e30;
        for (goop = 0; goop < is[loop]; goop++) {
          if (fabs(sbin[ifptr[loop] + goop] - redval) < limit) {
            limit = fabs(sbin[ifptr[loop] + goop] - redval);
            rs[loop] = goop;
          }
        }
      }
    }
  }
/* evaluate index coefficients (assume u-vel structure is representative) */
  fp = seekfld(hbin,HBMAX,'v',argv[1]);
/* loop on index fields */
  for (loop = 0; loop < indexfld; loop++) {
/* is index field the same as first dimension name? */
    if ((indexfld > 0) && (strcmp(ifld[loop],fp->dname1) == 0)) {
      coef[loop] = 1;
      if (indexfld > 1) coef[loop] *= fp->dsize2;
      if (indexfld > 2) coef[loop] *= fp->dsize3;
    }
/* what about second dimension name? */
    else if ((indexfld > 1) && (strcmp(ifld[loop],fp->dname2) == 0)) {
      coef[loop] = 1;
      if (indexfld > 2) coef[loop] *= fp->dsize3;
    }
/* what about third dimension name? */
    else if ((indexfld > 2) && (strcmp(ifld[loop],fp->dname3) == 0)) {
      coef[loop] = 1;
    }
/* no match between dimension names and index names is an error */
    else die("cdflagr: no match between an index name and a dimension\n");
  }
/* allocate good parcel flag based on size of u-vel array */
  if ((good = (char *) malloc(sizeof(char)*fp->tsize)) == NULL)
       die("cdflagr: can't get enough memory\n");
/* get variable slices */
  nvs = 0;
/* get space for first variable slice */
  if ((vin[nvs] = (float *) malloc(sizeof(float)*nvin)) == NULL)
       die("cdflagr: can't get enough memory\n");
/* loop on slice reads */
  while (getslice(stdin,nvin,vin[nvs]) != EOF) {
    nvs++;
    if (nvs >= VSMAX) die("cdflagr: too many variable slices\n");
/* get space for next variable slice */
    if ((vin[nvs] = (float *) malloc(sizeof(float)*nvin)) == NULL)
         die("cdflagr: can't get enough memory\n");
  }
  if (nvs == 0) die("cdflagr: no variable slices\n");
/* free last buffer, as it isn't needed */
  free(vin[nvs]);
/* survey number of parcels */
/* first, get ranges of limits covering starting points */
/* first index */
  if (rs[0] == -1) {
    jj1 = 0;
    k1 = is[0];
  }
  else if (rs[0] == 0) {
    jj1 = 0;
    k1 = 1;
  }
  else {
    jj1 = rs[0];
    k1 = jj1 + 1;
  }
/* second index */
  if (rs[1] == -1) {
    j2 = 0;
    k2 = is[1];
  }
  else if (rs[1] == 0) {
    j2 = 0;
    k2 = 1;
  }
  else {
    j2 = rs[1];
    k2 = j2 + 1;
  }
/* third index */
  if (rs[2] == -1) {
    j3 = 0;
    k3 = is[2];
  }
  else if (rs[2] == 0) {
    j3 = 0;
    k3 = 1;
  }
  else {
    j3 = rs[2];
    k3 = j3 + 1;
  }
/* find first slice time that exceeds requested time */
  for (ivs2 = 0; ivs2 < nvs; ivs2++) {
    if (*(vin[ivs2] + tptr) > tstart) break;
  }
  ivs1 = ivs2 - 1;
  if (ivs2 >= nvs) ivs2 = nvs - 1;
  if (ivs1 >= nvs) ivs1 = nvs - 1;
  if (ivs1 < 0) ivs1 = 0;
/* define velocity field pointers */
  u1 = vin[ivs1] + vptr[0];
  v1 = vin[ivs1] + vptr[1];
  w1 = vin[ivs1] + vptr[2];
  u2 = vin[ivs2] + vptr[0];
  v2 = vin[ivs2] + vptr[1];
  w2 = vin[ivs2] + vptr[2];
/* loop on starting regions, and count points that have good velocities */
  nparcels = 0;
  for (i1 = jj1; i1 < k1; i1++) {
    for (i2 = j2; i2 < k2; i2++) {
      for (i3 = j3; i3 < k3; i3++) {
/* eliminate those points with undefined velocities */
        good[I(i1,i2,i3)] = 0;
        if (fabs(u1[I(i1,i2,i3)]) > badlim) continue;
        if ((indexfld > 1) && (fabs(v1[I(i1,i2,i3)]) > badlim)) continue;
        if ((indexfld > 2) && (fabs(w1[I(i1,i2,i3)]) > badlim)) continue;
        if (fabs(u2[I(i1,i2,i3)]) > badlim) continue;
        if ((indexfld > 1) && (fabs(v2[I(i1,i2,i3)]) > badlim)) continue;
        if ((indexfld > 2) && (fabs(w2[I(i1,i2,i3)]) > badlim)) continue;
        good[I(i1,i2,i3)] = 1;
        nparcels++;
      }
    }
  }
  if (nparcels == 0) die("cdflagr: no parcels\n");
/* make new header */
  nullhdr(hbout,HBMAX,"float");
/* transfer comments */
  temp = getcmt(hbin,HBMAX,cb);
  for (loop = 0; loop < temp; loop++) addcline(hbout,HBMAX,cb[loop]);
/* add comment lines */
  strcpy(alpha,"cdflagr:");
  for (loop = 1; loop < indexfld + 2; loop++) {
    strcat(alpha," ");
    strcat(alpha,argv[loop]);
  }
  strcat(alpha,"\n");
  addcline(hbout,HBMAX,alpha);
  strcpy(alpha,"  ");
  for (loop = indexfld + 2; loop < argc; loop++) {
    strcat(alpha," ");
    strcat(alpha,argv[loop]);
  }
  strcat(alpha,"\n");
  addcline(hbout,HBMAX,alpha);
/* add parameters */
  sprintf(alpha,"%f",tstart + ntminus*dt);
  addpar(hbout,HBMAX,"time0",alpha);
  sprintf(alpha,"%f",dt);
  addpar(hbout,HBMAX,"dtime",alpha);
  sprintf(alpha,"%f",badlim);
  addpar(hbout,HBMAX,"badlim",alpha);
  sprintf(alpha,"%f",bad);
  addpar(hbout,HBMAX,"bad",alpha);
/* add static index fields */
  addfld(hbout,HBMAX,'s',"parcel",1.,0.,'s',1,"parcel",nparcels);
  addfld(hbout,HBMAX,'s',"time",1000.,0.,'l',1,"time",nt);
/* add variable fields -- parcel position first ... */
  for (loop = 0; loop < IFMAX; loop++) xoff[loop] = 0;
  for (loop = 0; loop < indexfld; loop++) {
    addfld(hbout,HBMAX,'v',ifld[loop],1000.,0.,'l',2,
         "parcel",nparcels,"time",nt);
/* store the offsets */
    fp = seekfld(hbout,HBMAX,'v',ifld[loop]);
    xoff[loop] = fp->ptr;
  }
/* ... then existing variable fields of dimension index */
  ngenflds = loop = 0;
  while ((fp = getfld(hbin,HBMAX,'v',loop++)) != NULL) {
    if (fp->dim != indexfld) continue;
    addfld(hbout,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,2,
         "parcel",nparcels,"time",nt);
/* again, store the offsets */
    if (loop >= NFMAX) die("cdflagr: too many variable fields\n");
    inoff[ngenflds] = fp->ptr;
    fp = seekfld(hbout,HBMAX,'v',fp->fname);
    outoff[ngenflds++] = fp->ptr;
  }
/* allocate space for the output buffers */
  nsout = elemcnt(hbout,HBMAX,'s');
  nvout = elemcnt(hbout,HBMAX,'v');
  if ((sbout = (float *) malloc(sizeof(float)*nsout)) == NULL)
       die("cdflagr: can't get enough memory\n");
  if ((vout = (float *) malloc(sizeof(float)*nvout)) == NULL)
       die("cdflagr: can't get enough memory\n");
/* fill in the output index fields */
  fp = seekfld(hbout,HBMAX,'s',"time");
  for (loop = 0; loop < fp->dsize1; loop++)
       sbout[fp->ptr + loop] = tstart + (loop + ntminus)*dt;
  fp = seekfld(hbout,HBMAX,'s',"parcel");
  for (loop = 0; loop < fp->dsize1; loop++)
       sbout[fp->ptr + loop] = loop;
/* fill variable slice with missing data value as default */
  for (lloop = 0; lloop < nvout; lloop++) vout[lloop] = bad;
/* write the header to the standard output */
  puthdr(stdout,hbout,HBMAX);
/* write static slice to the standard output */
  putslice(stdout,nsout,sbout);
/* initialize lagrangian position fields */
  lx = vout + xoff[0];
  ly = vout + xoff[1];
  lz = vout + xoff[2];
/* loop forward in time */
  for (it = -ntminus; it < nt; it++) {
    step(dt,-1);
  }
/* loop backward in time */
  for (it = -ntminus; it >= 0; it--) {
    step(-dt,1);
  }
/* write the variable slice to the standard output */
  putslice(stdout,nvout,vout);
  exit(0);
}

/* step -- this routine does a time step */
step(dt1,incr)
float dt1;
int incr;
{
/* figure out which input slices are the lead and lag slices */
  time = tstart + (it + ntminus)*fabs(dt1);
  for (ivs2 = 0; ivs2 < nvs; ivs2++) {
    if (*(vin[ivs2] + tptr) > time) break;
  }
  ivs1 = ivs2 - 1;
  if (ivs2 >= nvs) ivs2 = nvs - 1;
  if (ivs1 >= nvs) ivs1 = nvs - 1;
  if (ivs1 < 0) ivs1 = 0;
/* find the relative weighting between the two */
  ctime1 = *(vin[ivs1] + tptr);
  ctime2 = *(vin[ivs2] + tptr);
  if (ctime1 == ctime2) wt = 0.5;
  else wt = (time - ctime1)/(ctime2 - ctime1);
/* re-define velocity field pointers for current time */
  u1 = vin[ivs1] + vptr[0];
  v1 = vin[ivs1] + vptr[1];
  w1 = vin[ivs1] + vptr[2];
  u2 = vin[ivs2] + vptr[0];
  v2 = vin[ivs2] + vptr[1];
  w2 = vin[ivs2] + vptr[2];
/* loop again on the parcel number */
  iparcel = -1;
  for (i1 = jj1; i1 < k1; i1++) {
    for (i2 = j2; i2 < k2; i2++) {
      for (i3 = j3; i3 < k3; i3++) {
/* treat only parcels previously found to have initially good velocities */
        if (good[I(i1,i2,i3)] == 0) continue;
        iparcel++;
/* initialize position vector if initial time step */
        if (it == -ntminus) {
          lx[J(iparcel,-ntminus)] = ex[i1];
          if (indexfld > 1) ly[J(iparcel,-ntminus)] = ey[i2];
          if (indexfld > 2) lz[J(iparcel,-ntminus)] = ez[i3];
        }
/* if not initial step, integrate parcel positions forward */
        else {
/* if previous position bad, quit this parcel */
          if (fabs(lx[J(iparcel,it + incr)]) > badlim) continue;
          if ((indexfld > 1) && (fabs(ly[J(iparcel,it + incr)]) > badlim))
               continue;
          if ((indexfld > 2) && (fabs(lz[J(iparcel,it + incr)]) > badlim))
               continue;
/* set up spatial interpolation parameters for previous position */
          elocat(lx[J(iparcel,it + incr)],ex,is[0],&ix,&wx);
          if (indexfld > 1) elocat(ly[J(iparcel,it + incr)],ey,is[1],&iy,&wy);
          if (indexfld > 2) elocat(lz[J(iparcel,it + incr)],ez,is[2],&iz,&wz);
/* interpolate the velocities and forward difference integrate */
/* quit integration loop if we run into missing data or go out of bounds */
          lu = interpol(u1,u2);
          if (fabs(lu) > badlim) continue;
          pos = lx[J(iparcel,it + incr)] + lu*dt1;
          if ((pos < ex[0]) || (pos > ex[is[0] - 1])) continue;
          lx[J(iparcel,it)] = pos;
          if (indexfld > 1) {
            lv = interpol(v1,v2);
            if (fabs(lv) > badlim) continue;
            pos = ly[J(iparcel,it + incr)] + lv*dt1;
            if ((pos < ey[0]) || (pos > ey[is[1] - 1])) continue;
            ly[J(iparcel,it)] = pos;
          }
          if (indexfld > 2) {
            lw = interpol(w1,w2);
            if (fabs(lw) > badlim) continue;
            pos = lz[J(iparcel,it + incr)] + lw*dt1;
            if ((pos < ez[0]) || (pos > ez[is[2] - 1])) continue;
            lz[J(iparcel,it)] = pos;
          }
        }
/* set up spatial interpolation parameters for current position */
        elocat(lx[J(iparcel,it)],ex,is[0],&ix,&wx);
        if (indexfld > 1) elocat(ly[J(iparcel,it)],ey,is[1],&iy,&wy);
        if (indexfld > 2) elocat(lz[J(iparcel,it)],ez,is[2],&iz,&wz);
/* find the parcel values of all the general variables */
        for (loop = 0; loop < ngenflds; loop++) {
/* set up pointers to the lead and lag fields and lagrangian field */
          gen1 = vin[ivs1] + inoff[loop];
          gen2 = vin[ivs2] + inoff[loop];
          lgen = vout + outoff[loop];
/* interpolate the sucker */
          lgen[J(iparcel,it)] = interpol(gen1,gen2);
        }
      }
    }
  }
}

/* death routine */
die(message)
char *message;
{
  fprintf(stderr,"%s",message);
  exit(1);
}

/* determine position and weighted remainder */
elocat(lval,e,emax,i,w)
float lval,e[],*w;
int emax,*i;
{
  int j;
  float denom,rem,temp;
  denom = e[1] - e[0];
  temp = (lval - e[0])/denom;
  j = temp;
  if (j < 0) j = 0;
  if (j > emax - 2) j = emax - 2;
  rem = temp - j;
  if (rem < 0.) rem = 0.;
  if (rem > 1.) rem = 1.;
  *i = j;
  *w = rem;
}

/* linearly interpolate field values in time and space */
/* understands global variables ix, iy, iz, wx, wy, wz, wt */
static float interpol(f1,f2)
float *f1,*f2;
{
  int jx,jy,jz;
  int bad1,bad2;
  float qx,qy,qz;
  float temp1,temp2;
  float v1,v2,v3,v4,v5,v6,v7,v8;
  bad1 = bad2 = 0;
  if (indexfld == 1) {
    jx = ix + 1;
    qx = 1. - wx;
    if (fabs(v1 = f1[I(ix,0,0)]) > badlim) bad1 = 1;
    if (fabs(v2 = f1[I(jx,0,0)]) > badlim) bad1 = 1;
    temp1 = v1*qx + v2*wx;
    if (fabs(v1 = f2[I(ix,0,0)]) > badlim) bad2 = 1;
    if (fabs(v2 = f2[I(jx,0,0)]) > badlim) bad2 = 1;
    temp2 = v1*qx + v2*wx;
  }
  else if (indexfld == 2) {
    jx = ix + 1;
    jy = iy + 1;
    qx = 1. - wx;
    qy = 1. - wy;
    if (fabs(v1 = f1[I(ix,iy,0)]) > badlim) bad1 = 1;
    if (fabs(v2 = f1[I(jx,iy,0)]) > badlim) bad1 = 1;
    if (fabs(v3 = f1[I(ix,jy,0)]) > badlim) bad1 = 1;
    if (fabs(v4 = f1[I(jx,jy,0)]) > badlim) bad1 = 1;
    temp1 = (v1*qx + v2*wx)*qy
          + (v3*qx + v4*wx)*wy;
    if (fabs(v1 = f2[I(ix,iy,0)]) > badlim) bad2 = 1;
    if (fabs(v2 = f2[I(jx,iy,0)]) > badlim) bad2 = 1;
    if (fabs(v3 = f2[I(ix,jy,0)]) > badlim) bad2 = 1;
    if (fabs(v4 = f2[I(jx,jy,0)]) > badlim) bad2 = 1;
    temp2 = (v1*qx + v2*wx)*qy
          + (v3*qx + v4*wx)*wy;
  }
  else if (indexfld == 3) {
    jx = ix + 1;
    jy = iy + 1;
    jz = iz + 1;
    qx = 1. - wx;
    qy = 1. - wy;
    qz = 1. - wz;
    if (fabs(v1 = f1[I(ix,iy,iz)]) > badlim) bad1 = 1;
    if (fabs(v2 = f1[I(jx,iy,iz)]) > badlim) bad1 = 1;
    if (fabs(v3 = f1[I(ix,jy,iz)]) > badlim) bad1 = 1;
    if (fabs(v4 = f1[I(jx,jy,iz)]) > badlim) bad1 = 1;
    if (fabs(v5 = f1[I(ix,iy,jz)]) > badlim) bad1 = 1;
    if (fabs(v6 = f1[I(jx,iy,jz)]) > badlim) bad1 = 1;
    if (fabs(v7 = f1[I(ix,jy,jz)]) > badlim) bad1 = 1;
    if (fabs(v8 = f1[I(jx,jy,jz)]) > badlim) bad1 = 1;
    temp1 = ((v1*qx + v2*wx)*qy
           + (v3*qx + v4*wx)*wy)*qz
          + ((v5*qx + v6*wx)*qy
           + (v7*qx + v8*wx)*wy)*wz;
    if (fabs(v1 = f2[I(ix,iy,iz)]) > badlim) bad2 = 1;
    if (fabs(v2 = f2[I(jx,iy,iz)]) > badlim) bad2 = 1;
    if (fabs(v3 = f2[I(ix,jy,iz)]) > badlim) bad2 = 1;
    if (fabs(v4 = f2[I(jx,jy,iz)]) > badlim) bad2 = 1;
    if (fabs(v5 = f2[I(ix,iy,jz)]) > badlim) bad2 = 1;
    if (fabs(v6 = f2[I(jx,iy,jz)]) > badlim) bad2 = 1;
    if (fabs(v7 = f2[I(ix,jy,jz)]) > badlim) bad2 = 1;
    if (fabs(v8 = f2[I(jx,jy,jz)]) > badlim) bad2 = 1;
    temp2 = ((v1*qx + v2*wx)*qy
           + (v3*qx + v4*wx)*wy)*qz
          + ((v5*qx + v6*wx)*qy
           + (v7*qx + v8*wx)*wy)*wz;
  }
  if (bad1 && bad2) return(bad);
  else if (bad2) return(temp1);
  else if (bad1) return(temp2);
  else return(temp1*(1. - wt) + temp2*wt);
}
