/* cdflook.c -- Print header and element counts of a common data format
 * file from standard input in float format on the standard output.
 */
#include <stdio.h>
#include "cdfhdr.h"

#define HBBIG 2*HBMAX

static char hbuff[HBBIG][LINE];
static float *heap;
static int loop,poop;
static int nvars;
static long ns,nv;
static struct field *fb;
static struct field fa[HBBIG];

main(argc,argv)
int argc;
char *argv[];
{
/* check for arguments */
  if (argc != 1) {
    fprintf(stderr,"Usage: cdflook < cdf_file\n");
    exit(1);
  }
/* get header buffer from standard input */
  gethdr(stdin,hbuff,HBBIG);
/* print it to standard output */
  puthdr(stdout,hbuff,HBBIG);
/* check if format is float before proceeding */
  if (getfmt(hbuff,HBBIG) != 'f') {
    fprintf(stderr,"cdflook: non-float format\n");
    exit(1);
  }
/* compute expected number of elements */
  ns = elemcnt(hbuff,HBBIG,'s');
  nv = elemcnt(hbuff,HBBIG,'v');
/* allocate space for static slice */
  if ((heap = (float *)malloc(sizeof(float)*ns)) == NULL) {
    fprintf(stderr,"cdflook: can't allocate static slice buffer\n");
    exit(1);
  }
/* read static slice and print element count */
  if (getslice(stdin,ns,heap) == EOF) {
    fprintf(stderr,"cdflook: unexpected end of file reading static slice\n");
    exit(1);
  }
  fprintf(stdout,"%ld elements in static slice\n",ns);
/* print values of scalar variables in static slice */
  for (poop = 0;; poop++) {
    if ((fb = getfld(hbuff,HBBIG,'s',poop)) == NULL) break;
    if (fb->dim == 0) {
      fprintf(stdout,"    %s = %f\n",fb->fname,heap[fb->ptr]);
    }
  }
/* free static slice */
  free(heap);
/* allocate space for variable slices */
  if ((heap = (float *)malloc(sizeof(float)*nv)) == NULL) {
    fprintf(stderr,"cdflook: can't allocate variable slice buffer\n");
    exit(1);
  }
/* store results of getfld scan */
  for (poop = 0;; poop++) {
    nvars = poop;
    if ((fb = getfld(hbuff,HBBIG,'v',poop)) == NULL) break;
    fa[poop] = *fb;
  }
/* read each variable slice and print element count */
  for (loop = 1;; loop++) {
    if (getslice(stdin,nv,heap) == EOF) {
      exit(0);
    }
/* print values of scalar variables */
    fprintf(stdout,"%ld elements in variable slice %d\n",nv,loop);
    for (poop = 0; poop < nvars; poop++) {
      if (fa[poop].dim == 0) {
        fprintf(stdout,"    %s = %f\n",fa[poop].fname,heap[fa[poop].ptr]);
      }
    }
  }
}
