/* cdfmean -- Take the means over the specified dimension of
 * a specified list of variables.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

static char hbuff[HBMAX][LINE];              /* header buffer */
static char cline[3*LINE];                   /* line buffer */
static float *sbuff,*vbuff;                  /* slice buffers */
static long nsbin,nsbout,nvbin,nvbout;       /* slice buffer lengths */
static struct field *fp;                     /* field pointer temporary */
static float bad,badlim;                     /* bad data values */
static char *dimension,*suffix,*vars[HBMAX]; /* input strings */
static long loop,poop;                       /* looping indices */
static long nvars;                           /* number of variables affected */
static long indexsize;                       /* size of index field */
static long bingo;                           /* flag indicating dim exists */
static char ndname[4][LINE],*odname[4];      /* new and old dimension names */
static long ndsize[4],odsize[4];             /* new and old dimension sizes */
static long dimpos;                          /* counter dimension position */
static float *field,*mfield;                 /* subjects of operation */
static long start,incr,size,indexfld,instance;  /* for subspace1 call */
static double fmean;                         /* for averaging */
static long goodelems;                       /* for averaging */
static char buffer[3*LINE];                  /* name buffer for new name */
static char *newname();                      /* cat suffix to name */

main (argc,argv)
int argc;
char *argv[];
{

/* check args */
  if (argc < 4) {
    fprintf(stderr,"Usage: cdfmean dimension suffix var1 var2 ...\n");
    exit(1);
  }
  dimension = argv[1];
  suffix = argv[2];
  nvars = argc - 3;
  for (loop = 0; loop < nvars; loop++) {
    vars[loop] = argv[loop + 3];
  }

/* get a header from standard input */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfmean: float format expected\n");
    exit(1);
  }

/* get bad data values */
  if (seekpar(hbuff,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbuff,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;

/* check to see if dimension exists */
  if ((fp = seekfld(hbuff,HBMAX,'s',dimension)) == NULL ||
       strcmp(fp->fname,fp->dname1) != 0) {
    fprintf(stderr,"cdfmean: %s doesn't exist or is not an index field\n",
         dimension);
    exit(1);
  }
  indexsize = fp->dsize1;

/* get input buffer sizes */
  nsbin = elemcnt(hbuff,HBMAX,'s');
  nvbin = elemcnt(hbuff,HBMAX,'v');

/* check consistency for each variable -- does it have the right dimension? */
  for (loop = 0; loop < nvars; loop++) {
    if ((fp = seekfld(hbuff,HBMAX,'v',vars[loop])) == NULL) {
      fprintf(stderr,"cdfmean: variable field %s doesn't exist\n",vars[loop]);
      exit(1);
    }
    bingo = 0;
    if (fp->dim > 0 && strcmp(fp->dname1,dimension) == 0) bingo = 1;
    if (fp->dim > 1 && strcmp(fp->dname2,dimension) == 0) bingo = 2;
    if (fp->dim > 2 && strcmp(fp->dname3,dimension) == 0) bingo = 3;
    if (fp->dim > 3 && strcmp(fp->dname4,dimension) == 0) bingo = 4;
    if (bingo == 0) {
      fprintf(stderr,"cdfmean: variable %s doesn't have dimension %s\n",
           vars[loop],dimension);
      exit(1);
    }

/* construct new entry information for reduced variable */
    dimpos = 0;
    odname[0] = fp->dname1;
    odname[1] = fp->dname2;
    odname[2] = fp->dname3;
    odname[3] = fp->dname4;
    odsize[0] = fp->dsize1;
    odsize[1] = fp->dsize2;
    odsize[2] = fp->dsize3;
    odsize[3] = fp->dsize4;
    for (poop = 0; poop < fp->dim; poop++) {
      if (poop != bingo - 1) {
        strcpy(ndname[dimpos],odname[poop]);
        ndsize[dimpos] = odsize[poop];
        dimpos++;
      }
    }

/* add the new field */
    addfld(hbuff,HBMAX,'v',newname(vars[loop],suffix),fp->smul,fp->sadd,
         fp->bytes,fp->dim - 1,ndname[0],ndsize[0],ndname[1],ndsize[1],
         ndname[2],ndsize[2],ndname[3],ndsize[3]);
  }

/* add a comment */
  sprintf(cline,"cdfmean: %s %s\n",dimension,suffix);
  addcline(hbuff,HBMAX,cline);
  loop = 3;
  while (loop < argc) {
    strcpy(cline,"");
    for (poop = 0; poop < 3; poop++) {
      if (loop >= argc) break;
      strcat(cline," ");
      strcat(cline,vars[loop - 3]);
      loop++;
    }
    strcat(cline,"\n");
    addcline(hbuff,HBMAX,cline);
  }

/* get output buffer sizes and create buffers */
  nsbout = elemcnt(hbuff,HBMAX,'s');
  nvbout = elemcnt(hbuff,HBMAX,'v');
  sbuff = getbuff(nsbout);
  vbuff = getbuff(nvbout);

/* put header and transfer static slice */
  puthdr(stdout,hbuff,HBMAX);
  getslice(stdin,nsbin,sbuff);
  putslice(stdout,nsbout,sbuff);

/* loop on variable slices */
  while (getslice(stdin,nvbin,vbuff) != EOF) {

/* loop on variables to be averaged */
    for (loop = 0; loop < nvars; loop++) {
      field = getptr2(hbuff,HBMAX,vbuff,'v','d',vars[loop],&fp);
      mfield = getptr(hbuff,HBMAX,vbuff,'v','d',newname(vars[loop],suffix));

/* loop on subspace cutting thru field in direction of averaging dimension */
      instance = 0;
      while (subspace1(fp,dimension,instance,&start,&incr,&size) != FAIL) {
        fmean = 0.;
        goodelems = 0;
        for (poop = 0; poop < size; poop++) {
          indexfld = start + incr*poop;
          if (fabs(field[indexfld]) < badlim) {
            fmean += field[indexfld];
            goodelems++;
          }
        }
        if (goodelems > 0) mfield[instance++] = fmean/goodelems;
        else mfield[instance++] = bad;
      }
    }

/* write variable slice */
    putslice(stdout,nvbout,vbuff);
  }
  exit(0);
}

/* newname -- concatenate name and suffix together */
static char *newname(name,suffix)
char *name,*suffix;
{
  strcpy(buffer,name);
  strcat(buffer,suffix);
  return(buffer);
}
