/* cdforder                                                           */
/* cdforder indexfield1 indexfield2 ....                              */
/* cdforder will rearange the order of the index fields and the order */
/* of the dimnames within the fields                                  */

#include <stdio.h>
#include <string.h>
#include "cdfhdr.h"

#define MAXINFLD  20

static int inflddim[MAXINFLD],dim1,dim2,dim3,dim4;    /* dimsize info */
static char buffer[HBMAX][LINE],bufferw[HBMAX][LINE];
                                    /* buffer for header data */
static char df1[LINE],df2[LINE],df3[LINE],df4[LINE];  /* dimname info/order */
static char infldo[MAXINFLD][LINE]; /* index field data */
static char infldx[MAXINFLD][LINE]; /* index field temp buffer */
static int ninfld;                  /* number of input fields */
static float *sbuffin,*sbuffout;    /* buffers that contain slice data */

main(argc,argv)
int argc;
char *argv[];
{
  int loop,loop1,pos,vpos,flag;   /* used in looping and flagging */
  int a1,b1,c1,d1,*a2,*b2,*c2,*d2;/* used in looping */
  int ds1,ds2,ds3,ds4,add1,add2;  /* used in looping thru fields and dims */
  int dsa,dsb,dsc,dsd;            /* same */
  long nel;                       /* # of elements to get and to write */
  char inflda[MAXINFLD][LINE];    /* contains index field info */
  char cl[LINE];                  /* for comment line and misc */
  char nofld[LINE],field;         /* temp storage and null fld */
  float *din,*dout;               /* pointers for data in and out */
  struct field *fp;               /* pointer to the field structure */
  struct field *fb;               /* another pointer to field structure */

  strcpy(nofld,"\0");
/* assume command line arguments are index fields in desired order */
  if (argc > 1) {
/* move stuff out of argument line */
    if (argc > MAXINFLD) {
      fprintf(stderr,"cdforder: too many fields given\n");
      fprintf(stderr,"          update MAXINFLD in prog\n");
      exit(1);
    }
    ninfld=argc-1;
    for (loop = 0; loop < MAXINFLD; loop++) {
      if (loop < ninfld) strcpy(infldo[loop],argv[loop+1]);
      else strcpy(infldo[loop],nofld);
    }
  }
/* get header from stdin */
  gethdr(stdin,buffer,HBMAX);
/* check format */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfisocut: file must be float!\n");
    exit(1);
  }
/* no command line arguments -- in this case reverse order of index fields */
  if (argc == 1) {
/* first figure out how many index fields there are */
    ninfld = loop = 0;
    while (1) {
      fp=getfld(buffer,HBMAX,'s',loop++);
      if (fp == NULL) break;
      if (fp->dim == 1 && strcmp(fp->fname,fp->dname1) == 0) {
	strcpy(infldx[ninfld++],fp->fname);
      }
    }
/* now create a fake input list from actual index fields with rev order */
    for (loop = 0; loop < MAXINFLD; loop++) {
      if (loop < ninfld) strcpy(infldo[loop],infldx[ninfld - 1 - loop]);
      else strcpy(infldo[loop],nofld);
    }
  }
/* make sure names given are index fields */
  loop=0;
  while (loop<ninfld) {
    fp=seekfld(buffer,HBMAX,'s',infldo[loop]);
    if ((fp==NULL)||(strcmp(fp->fname,fp->dname1)!=0)) {
      fprintf(stderr,"cdforder: %s is an invalid field\n",infldo[loop]);
      exit(1);
    }
    inflddim[loop++]=fp->dsize1;
  }
/* make sure every index field was given */
  loop=flag=0;
  while ((fp=getfld(buffer,HBMAX,'s',loop))!=NULL) {
    if (strcmp(fp->fname,fp->dname1)==0) {
      strcpy(inflda[loop],fp->fname);
      flag++;
    }
    loop++;
  }
  if (flag!=ninfld) {
    fprintf(stderr,"cdforder: all index fields were not accounted for\n");
    exit(1);
  }
/* make output header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer comments */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
/* add comment line */
  sprintf(cl,"cdforder: ");
  loop=1;
  if (argc > 4) {
    for (loop=1;loop<4;loop++) {
      strcat(cl,argv[loop]);
      strcat(cl," ");
    }
    strcat(cl,"\n");
    addcline(bufferw,HBMAX,cl);
    sprintf(cl,"  ");
  }
  while (loop < argc) {
    strcat(cl,argv[loop++]);
    strcat(cl," ");
  }
  strcat(cl,"\n");
  addcline(bufferw,HBMAX,cl);
/* transfer parameters */
  copypar(bufferw,HBMAX,buffer,HBMAX);
/* transfer static and variable fields */
  field='s';
/* loop thru static then variable area */
  for (loop1=0;loop1<2;loop1++) {
    pos=0;
/* loop thru the fields */
    while ((fp=getfld(buffer,HBMAX,field,pos++))!=NULL) {
/* if it is an index field then ignore 'til later */
      if (strcmp(fp->fname,fp->dname1)!=0) {
/* if a 0 or 1-dim field then send with no changes */
        if (fp->dim == 0)
          addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,0);
        else if (fp->dim == 1) {
          addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,1,
                 fp->dname1,fp->dsize1);
        }
/* if it is a 2, 3 or 4-dim field update then send */
        else if (fp->dim == 2) {
          inorder(fp->dname1,fp->dname2,nofld,nofld);
          addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,2,
                 df1,dim1,df2,dim2);
        }
        else if (fp->dim == 3) {
          inorder(fp->dname1,fp->dname2,fp->dname3,nofld);
          addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,3,
                 df1,dim1,df2,dim2,df3,dim3);
        }
        else if (fp->dim == 4) {
          inorder(fp->dname1,fp->dname2,fp->dname3,fp->dname4);
          addfld(bufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,4,
                 df1,dim1,df2,dim2,df3,dim3,df4,dim4);
        }
      }
    }
/* when thru with rest of static fields then send index fields in order */
    if (field=='s') {
      loop=0;
      while (loop<ninfld) {
        fp=seekfld(buffer,HBMAX,'s',infldo[loop++]);
        addfld(bufferw,HBMAX,'s',fp->fname,fp->smul,fp->sadd,fp->bytes,1,
               fp->dname1,fp->dsize1);
      }
    }
/* switch to variable fields */
    field='v';
  }
/* output header */
  puthdr(stdout,bufferw,HBMAX);
/* allocate memory for the static slices */
  field='s';
  nel=elemcnt(buffer,HBMAX,field);
  sbuffin=getbuff(nel);
  sbuffout=getbuff(nel);
/* cycle thru the static then variable slice */
  flag=0;
  while (getslice(stdin,nel,sbuffin)!=EOF) {
    if (FLOATSWAP) floatswap(nel,sbuffin);
    ++flag;
    vpos=0;
    while ((fp=getfld(buffer,HBMAX,field,vpos++))!=NULL) {
      fb=seekfld(bufferw,HBMAX,field,fp->fname);
      din=getptr(buffer,HBMAX,sbuffin,field,'d',fp->fname);
      dout=getptr(bufferw,HBMAX,sbuffout,field,'d',fp->fname);
      if ((fp->dim == 0)||(fp->dim == 1)) {
        for (pos=0;pos<fp->tsize;pos++)
          dout[pos]=din[pos];
      }
/* find the dimsizes for looping thru the fields and looping info */
      else if (fp->dim > 1) {
        ds1=ds2=ds3=ds4=dsa=dsb=dsc=dsd=1;
        ds1=fp->dsize1;
        dsa=fb->dsize1;
        a2 = &a1;
        b2 = &b1;
        c2 = &c1;
        d2 = &d1;
        if (strcmp(fp->dname1,fb->dname2)==0) b2 = &a1;
        else if (strcmp(fp->dname1,fb->dname3)==0) c2 = &a1;
        else if (strcmp(fp->dname1,fb->dname4)==0) d2 = &a1;
        if (fp->dim >= 2) {
          if (strcmp(fp->dname2,fb->dname1)==0) a2 = &b1;
          else if (strcmp(fp->dname2,fb->dname3)==0) c2 = &b1;
          else if (strcmp(fp->dname2,fb->dname4)==0) d2 = &b1;
          ds2=fp->dsize2;
          dsb=fb->dsize2;
        }
        if (fp->dim >= 3) {
          if (strcmp(fp->dname3,fb->dname1)==0) a2 = &c1;
          else if (strcmp(fp->dname3,fb->dname2)==0) b2 = &c1;
          else if (strcmp(fp->dname3,fb->dname4)==0) d2 = &c1;
          ds3=fp->dsize3;
          dsc=fb->dsize3;
        }
        if (fp->dim >= 4) {
          if (strcmp(fp->dname4,fb->dname1)==0) a2 = &d1;
          else if (strcmp(fp->dname4,fb->dname2)==0) b2 = &d1;
          else if (strcmp(fp->dname4,fb->dname3)==0) c2 = &d1;
          ds4=fp->dsize4;
          dsd=fb->dsize4;
        }
/* loop thru the dimensions */
        for (a1=0;a1<ds1;a1++) {
          for (b1=0;b1<ds2;b1++) {
            for (c1=0;c1<ds3;c1++) {
              for (d1=0;d1<ds4;d1++) {
/* transfer the data */
                add1=(d1+c1*ds4+b1*ds4*ds3+a1*ds4*ds3*ds2);
                add2=((*d2)+(*c2)*dsd+(*b2)*dsd*dsc+(*a2)*dsd*dsc*dsb);
                dout[add2]=din[add1];
              }
            }
          }
        }
      }
    }
/* output slice and if static slice then change buffers to variable slice */
    if (FLOATSWAP) floatswap(nel,sbuffout);
    putslice(stdout,nel,sbuffout);
    if (flag==1) {
      free(sbuffin);
      free(sbuffout);
      nel=elemcnt(buffer,HBMAX,'v');
      sbuffin=getbuff(nel);
      sbuffout=getbuff(nel);
      field='v';
    }
  }
  exit(0);
}

/* rearranges the dim names within a field to be in order required */
inorder(d1,d2,d3,d4)
char d1[],d2[],d3[],d4[];
{
  int x,y,temp,pos[5];
  char dname[4][LINE];

/* place in an array for easier handling */
  strcpy(dname[0],d1);
  strcpy(dname[1],d2);
  strcpy(dname[2],d3);
  strcpy(dname[3],d4);
  pos[0]=pos[1]=pos[2]=pos[3]=pos[4]=MAXINFLD-1;
  for (x=0;x<4;x++) {
    for (y=0;y<ninfld;y++) {
      if (strcmp(dname[x],infldo[y])==0)
        pos[x]=y;
    }
  }
/* compare to list given on command line */
  for (x=0;x<4;x++) {
    for (y=0;y<4;y++) {
      if (pos[y]>pos[y+1]) {
        temp=pos[y];
        pos[y]=pos[y+1];
        pos[y+1]=temp;
      }
    }
  }
/* transfer the new order (of names) to new variables */
  strcpy(df1,infldo[pos[0]]);
  strcpy(df2,infldo[pos[1]]);
  strcpy(df3,infldo[pos[2]]);
  strcpy(df4,infldo[pos[3]]);
/* do same for dimsizes */
  dim1=inflddim[pos[0]];
  dim2=inflddim[pos[1]];
  dim3=inflddim[pos[2]];
  dim4=inflddim[pos[3]];
  return(0);
}
