/* cdfpix is a program to convert between candis pixel and  float formats*
 *  The -f option converts from pixel to float while  the -p option      *
 * converts from float to pixel.                                         *
 * the affected dimension (assumed to be the second ( so far i have never*
 * seen, or wanted to see a three or four dimensional pixel field)) has  *
 * pix added to the end of its name and it is added to the end of the    *
 * static slice, this is done so that other fields dependent on that     *
 * dimension will not be messed up.                                      *
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define PIXDEF 8
#define FLDEND "pix"
#define NAME argv[0]

static struct field *fb;                /* field info pointer */
static char fname[LINE];                /* name of the target field */
static float bad;                       /* value of bad data */
static char opt;                        /* option (-f|-p) */
static char hbuff[HBMAX][LINE];         /* buffer for the old header */
static char newbuff[HBMAX][LINE];       /* buffer for the new header */
static int bitset, setbit;              /* flag and bitsize for -p option */
static int bit,pak,nssze;               /* pixel size,number per long */
static long selmnt, velmnt;             /* slice element counts */
static long MASK[] = {0,1,3,0,15,0,0,0,255,0,0,0,0,0,0,0,64535,0,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,(64536ul*64536ul-1ul)};
static long putslice2();

main(argc,argv)
int argc;                       /* there should be 2 possibly 3 arguments */
char *argv[];
{

char cline[LINE],value[LINE] ;       /* comment line , parameter check*/
float *sptr,*vptr;              /* pointers to buffers */
long newfsize,oldsize;          /* old and new element counts */
char prec;                      /* precision 's','l','p' */
long nnum,tgtofst;              /* element sizes for putslice2 */
char  thedim[LINE],samedim[LINE];/* name of 2nd dim of target field */
char fmt ;                      /* format check char should be 'f' */
int loop,par;                    /* obvious*/
long bnum,cnvsze,cnt;           /* counters for slice sub-buffers */
float *vnewptr,*snewptr;        /* pointers to slice sub-buffers */
float *slastptr,*lastptr;       /* pointers to ends of buffers */
float *temptr;                  /* temporary pointer for sampling data */
long nfsze,bgnsze,sfirst;       /* sub-buffer counters */
long newfsze,sendsze,lastsze;   /* more of the same */
float smul,sadd;                /* multiplier and additive constant */
int sdone = 0,slice1;                  /* flag for reused dimensions */
char otherdim[LINE];            /* the first dimension */ 
bitset = 0;

  if (argc<3){
    fprintf(stderr,"Useage:\n");
    fprintf(stderr,"        %s -f field \n",NAME);
    fprintf(stderr,"        %s -p field [pixsize] \n",NAME);
    exit(1);
  }
  
  /* get the option and the field name */ 
  if (strcmp(argv[1],"-f") == 0)  opt = 'f';
    else if (strcmp(argv[1],"-p") ==0) opt = 'p'; 
      else {
        fprintf(stderr,"%s: unknown option %s \n",NAME,argv[1]);
        exit(1); 
      } 
  strcpy(fname,argv[2]);
  /* check for possible third argument and set bitset and bit as needed */
  if(argc == 4)
    if (opt == 'p'){
      bitset = 1;      /* raise the flag */
      bit = atof(argv[3]);
      if (bit >32 ) bit = 32;
    }
    else {
     fprintf(stderr,"%s: too many arguments \n",NAME);
     exit(1);
    }
  if(argc >4){ 
    fprintf(stderr,"%s: too many arguments \n",NAME);
    exit(1);
  }
  
  /* get the header and check that the field exists */
  gethdr(stdin,hbuff,HBMAX);
  if(( fmt = getfmt(hbuff,HBMAX)) != 'f'){
    fprintf(stderr,"%s: format not float \n",NAME);
    exit(1);
  }
  if(( fb = seekfld(hbuff,HBMAX,'v',fname)) == NULL){
    fprintf(stderr,"%s: field %s nonexistent \n",NAME,fname);
    exit(1);
  }
  /* get the second dimension name */
  strcpy(thedim,fb->dname2);
  /* and the first */
  strcpy(otherdim,fb->dname1);
  /* check that the field is in the right format for conversion */
  if(opt == 'p' && fb->bytes == 'p') {
    fprintf(stderr,"%s: field %s already pixel\n",NAME,fname);
    exit(1);
  }
  if(opt == 'f' && fb->bytes != 'p'){
    fprintf(stderr,"%s: field %s not pixel \n",NAME,fname);
    exit(1);
  }
  /* get the element counts */
 selmnt = elemcnt(hbuff,HBMAX,'s');
 velmnt = elemcnt(hbuff,HBMAX,'v');
 /* and now the buffers */
 sptr =  getbuff(selmnt);
 vptr =  getbuff(velmnt);
 
 /* copy the header into the new buffer */
 nullhdr(newbuff,HBMAX,"float");
 copycmt(newbuff,HBMAX,hbuff,HBMAX);
 copypar(newbuff,HBMAX,hbuff,HBMAX);
 
 /* copy the static fields over, the affected field will be appended later*/
 copyfld(newbuff,HBMAX,hbuff,HBMAX,'s');

 /*  say we were here */
 sprintf(cline,"%s: %s ",NAME,fname);
 if( opt == 'p') strcat(cline,"-p\n");
   else strcat(cline,"-f\n");
 addcline(newbuff,HBMAX,cline);
 /* figure out the new number of elements in variable and static fields */
 count(fname,sptr,vptr,&prec,&nfsze,&sfirst,&sendsze,
       &snewptr,&slastptr,&newfsze,&bgnsze,&lastsze,&vnewptr,&lastptr);
 /* don't forget the name of the dimension before alterations */
 strcpy(samedim,thedim);
 /* rename the 2nd dimension */
 strcat(thedim,FLDEND);
 
 /* add the dimension */ 
 fb = seekfld(hbuff,HBMAX,'s',samedim);
 addfld2(newbuff,HBMAX,'s',thedim,fb->smul,fb->sadd,fb->bytes,
        fb->dim,&sdone,thedim,nfsze);
 /* move variable fields over */
 loop = 0;
 while(( fb = getfld(hbuff,HBMAX,'v',loop)) != NULL){
   loop++;
   if(strcmp(fname,fb->fname) != 0)
     addfld(newbuff,HBMAX,'v',fb->fname,fb->smul,fb->sadd,fb->bytes,
            fb->dim,fb->dname1,fb->dsize1,fb->dname2,fb->dsize2,
            fb->dname3,fb->dsize3,fb->dname4,fb->dsize4);
     else {
       if(opt == 'f' ) {
         smul = 1.0 ;
         sadd = 0.0 ;
        }
        else {
          smul = (float)bit;
          sadd = (float)pak;
        }
       addfld(newbuff,HBMAX,'v',fname,smul,sadd,prec,2,fb->dname1,
              fb->dsize1,thedim,nfsze);
       oldsize = fb->tsize;
     } 
 }
    /*   time to work with the slices */
 slice1 = 1; 
 /* loop on slices remembering that we already have the first two */
 while((cnt = getslice(stdin,selmnt,sptr)) != EOF){
   if((cnt = getslice(stdin,velmnt,vptr)) != EOF){
     if(slice1 == 1){
       fb = seekfld(hbuff,HBMAX,'s',samedim) ;
       temptr = sptr+fb->ptr;
       sprintf(cline,"%s0",thedim);
       sprintf(value,"%f",temptr[0]);
       addpar(newbuff,HBMAX,cline,value);
       sprintf(value,"%f",temptr[1]-temptr[0]); 
       sprintf(cline,"d%s",thedim);
       addpar(newbuff,HBMAX,cline,value);
       sprintf(cline,"%s0",otherdim);
       if((par = seekpar(hbuff,HBMAX,cline,value)) == FAIL){
         fb = seekfld(hbuff,HBMAX,'s',otherdim);
         temptr = fb->ptr+sptr; 
         sprintf(value,"%f",*temptr);
         addpar(newbuff,HBMAX,cline,value);
         sprintf(cline,"d%s",otherdim);
         sprintf(value,"%f",temptr[1]-temptr[0]);
         addpar(newbuff,HBMAX,cline,value);
       } 
       /* get rid of the header */ 
       puthdr(stdout,newbuff,HBMAX);
      }
     if(sdone == 0){
       sconv(samedim,sptr,nfsze,snewptr); 
       /* get rid of static slice */
       putslice2(stdout,sfirst,sptr,nfsze,snewptr,0,NULL); 
     }
     /* no static slice changes were needed */
     else putslice(stdout,selmnt,sptr);
     /* now for the variable slices */
     /* find pointers to the begining  and end of the */
     /* target field */
     vconv(fname,vptr,newfsze,vnewptr); 
     /* get rid of the  slice */
     putslice2(stdout,bgnsze,vptr, newfsze,vnewptr,lastsze,lastptr); 
     slice1 = 0;
   }
 }                 /* end of for(;;) */
  /* keep sun happy */
  exit(0);
}                 /* of program */ 

/* count(...) is supposed to find out the field size after the conversion
 * and if converting to pixel it is supposed to decide on the size of the 
 * pixel                                                              */

count(fname,sptr,vptr,prec,snew,sfirst,send,snewptr,slastptr,
      vnew,vfirst,vend,vnewptr,vlastptr)

char *prec;                     /* precision 's','l','p' */
long *snew,*vnew,*sfirst,*vfirst,*send,*vend;
float **snewptr,**vnewptr,**slastptr,**vlastptr;
float *vptr,*sptr;
{
 int loop,loop2;
 int chk;    

 fb = seekfld(hbuff,HBMAX,'v',fname);
  if(opt == 'f'){
    bit = fb->smul;
    pak = fb->sadd;
    if(bit<=16) *prec = 's';
      else *prec = 'l'; 
    *vnew = pak*fb->tsize;
    fb = seekfld(hbuff,HBMAX,'s',fb->dname2);
    *snew = pak*fb->tsize;
  }
  else{
  /* if optional argument bitsize is set then don't bother */
  if(bitset != 1) 
      /* set bit size to default */
      bit = PIXDEF; 
    pak = sizeof(long)*8/bit;
    *prec = 'p'; 
    fb = seekfld(hbuff,HBMAX,'v',fname);
    *vnew = (fb->dsize1);
    fb = seekfld(hbuff,HBMAX,'s',fb->dname2);
    *snew = (fb->tsize +pak-1)/pak;
    *vnew *= *snew;
  }
  /* now get the new buffers and the size of each sub buffer */
  *snewptr = getbuff(*snew);
  *vnewptr = getbuff(*vnew);
  *sfirst   = selmnt ; 
  *send    = 0;
  *slastptr = sptr + fb->ptr + fb->tsize;
  fb = seekfld(hbuff,HBMAX,'v',fname);
  *vfirst = fb->ptr;
  *vend = velmnt - fb->ptr - fb->tsize;
  *vlastptr = vptr +fb->ptr +fb->tsize;
 }
/* sconv -- the easy part -- the new 2nd dimension will be found by 
 * either eliminating all but the 'pak'th points or interpolating between 
 * the existing points ( options -p and -f respectively). the modified 
 *  dimension will the be placed in the buffer which will be left for
 * putslice2 to  throw out */
  
sconv(dim,sptr,size,snewptr)

float *sptr;       /* pointer to the beginning part of the buffer */
long size;
float *snewptr;
char dim[]; 
{ 
  float incr;         
  int loop, loop2;      
  /* get info on the field */
  fb = seekfld(hbuff,HBMAX,'s',dim);
  sptr += fb->ptr;
  if(opt == 'p'){
    /* conversion to float */
    /* eliminate all but the pak'th elements */
    loop2 = 0;
    for(loop = 0 ; loop < fb->tsize; loop += pak){
       snewptr[loop2] = sptr[loop];
       loop2++;
    }
   }
  else {          /* now add elements to fields */
    incr = (sptr[fb->tsize -1] - sptr[0])/(size);   
    for(loop = 0; loop<(fb->tsize*pak);loop++)
      snewptr[loop] = *sptr + loop*incr;
   
   }
}


/* vconv - the hard part -- this should convert the variable slice from either 
 * float format to pixel format or from pixel to float format. the conversion
 * from float format is accomplished by taking each of the long elements which
 * the pixels are packed in. the longs are right shifted by bit bits on each 
 * pass and mask is set to remove all but the rightmost bit bits. the conversion * from float into pixel is done by unpacking the floats depending on thier
 * value of prec and then inserting them one at a time into the new longs
 * the proscess is then repeated until the long is  full ('pak' repititions)
 * the altered buffers are left out for putslice2  to pick up                                                                                            */ 
vconv(fname,vptr,size,newptr)
float *vptr,*newptr;
long size;
{
 int loop,loop2;                      /*generic loop counters */
 long oldcount;      /* count of packed data */
 long *lngptr;

 fb = seekfld(hbuff,HBMAX,'v',fname);
 vptr += fb->ptr;
 oldcount = fb->tsize;
 if(opt == 'p') lngptr = (long*)(newptr);
    else lngptr = (long*)vptr;
 /* now convert to float */               
 if(opt == 'f'){
   for(loop = 0; loop <oldcount; loop++)
      for(loop2 = 0 ;loop2<pak;loop2++)
         newptr[loop*pak+loop2] = (lngptr[loop]>>(loop2*bit))&MASK[bit];
  }
  /* now convert to pixel */
  else{
    for(loop = 0; loop <oldcount/pak;loop++){
      for(loop2 = 0; loop2 <pak; loop2++){
          lngptr[loop] += ((long)vptr[loop*pak+loop2]&MASK[bit])<<bit*loop2;
      } 
    } 
  }
}     /* finished */ 

/* addfld2 -- This routine adds a field to a header buffer.  The type is
 * either 's' for static or 'v' for variable.  The elements of the field 
 * structure are arguments 4, 5, 6, ... of the function call.  The total number
 * of arguments is variable depending on the dimension of the field.
 * this version of addfld does not complain and die if the field to be added
 * is a duplicate, instead it checks the field that it has found wiht the 
 * same name: if all the charichteristics of the field are the same then
 * the field is not added and a flag is sent back, if the field descriptions
 * do not match then a 1 is added to the end of the field name and the
 * checking is done again ... . */

addfld2(buffer,max,type,fname,smul,sadd,precis,dim,sdone,d1,n1,d2,n2,d3,n3,d4,n4)
char buffer[][LINE],type,precis,*fname,*d1,*d2,*d3,*d4;
float smul,sadd;
int max,dim,n1,n2,n3,n4,*sdone;
{
  int loop,length,stat;
  char line[2*LINE],temp[LINE],comm[LINE];
  struct field *fb;
/* check for proper type */
  if ((type != 's') && (type != 'v')) {
    fprintf(stderr,"addfld: Unrecognized type = %c\n",type);
    exit(1);
  }
  stat = (type == 's');
/* check for existence of field of this name */
  loop = 1;
  while(( fb = seekfld(buffer,max,type,fname)) != NULL) {
    if(smul == fb->smul && fb->sadd == sadd && fb->dsize1 == n1){
      *sdone = 1;
      return;
    }
    else{
      loop++;
      sprintf(comm,"%d",loop);
      strcat(fname,comm);
    }
  }
/* check for array dimension out of range */
  if ((dim < 0) || (dim > 4)) {
    fprintf(stderr,"addfld: dim = %d out of range on field = %s\n",dim,fname);
    exit(1);
  }
/* check for pixel field consistency */
  if (precis == 'p') {
    if (smul != 1. && smul != 2. && smul != 4. && smul != 8. &&
         smul != 16. && smul != 32.) {
      fprintf(stderr,"addfld: smul = %g not a proper pixel value\n",smul);
      exit(1);
    }
    if (smul*sadd != 8.*sizeof(float)) {
      fprintf(stderr,"addfld: smul = %g times sadd = %g != float size\n",
           smul,sadd);
      exit(1);
    }
  }
/* make up line */
  sprintf(line,"%s %g %g %c %d",fname,smul,sadd,precis,dim);
  if (dim > 0) {
    sprintf(temp," %s %d",d1,n1);
    strcat(line,temp);
  }
  if (dim > 1) {
    sprintf(temp," %s %d",d2,n2);
    strcat(line,temp);
  }
  if (dim > 2) {
    sprintf(temp," %s %d",d3,n3);
    strcat(line,temp);
  }
  if (dim > 3) {
    sprintf(temp," %s %d",d4,n4);
    strcat(line,temp);
  }
  strcat(line,"\n");
/* check for excessive length */
  if (strlen(line) > LINE - 1) {
    fprintf(stderr,"addfld: field too long: %s\n",line);
    exit(1);
  }
/* find end of buffer */
  length = 0;
  for (loop = 6; loop < max - 1; loop++) {
    if (ETEST(loop)) {
      length = loop;
      break;
    }
  }
/* test for bad format */
  if (length == 0) {
    fprintf(stderr,
      "addfld: Bad header buffer format or no more buffer space\n");
    exit(1);
  }
/* ripple down to make room for new insert */
  for (loop = length; loop > 0; loop--) {
    strcpy(buffer[loop + 1],buffer[loop]);
    sscanf(buffer[loop],"%s",temp);
    if ((stat && (strcmp(temp,VSH) == 0)) ||
      (!stat && (strcmp(temp,FSH) == 0))) {
      strcpy(buffer[loop],line);
      return;
    }
  }
/* error if we couldn't find field list sub-header */
  fprintf(stderr,"addfld: Bad header buffer format\n");
  exit(1);
}


/* putslice2 --  change to putslice that allows mulitple buffers to be
 * treated as one  slice buffer when they are put on stream
 * Putslice2 accepts two buffers of floats, an open file pointer,and the
 * element count, writes the number in 8 byte decimal ascii, then writes the
 * floats using fwrite.  Returns the number of floats written. Abuff is 
 * attached to the end of sbuff so that additions and modifications can be
 * made to a file wihtout endless duplication of massive slice buffers
 */

static long putslice2(stream,num,sbuff,anum,abuff,bnum,bbuff)
FILE *stream; /* the data file pointer */
long num,anum,bnum; /* the number of elements to write */
float sbuff[],abuff[],bbuff[];  /* the buffer of floats */
{
  long nwritten;  /* number of elements written */
  char elct[9];
  if(num+anum+bnum < 0 || num +anum+bnum> MAXSLICE) {
    fprintf(stderr,"putslice2: weird element count = %ld\n",num+anum+bnum);
    exit(1);
  } 
  sprintf(elct,"%8ld",(num+anum+bnum));
  fputs(elct,stream);
  if ((num+anum+bnum) == 0) return(0);
  nwritten = fwrite(sbuff,sizeof(float),num,stream);
  nwritten += fwrite(abuff,sizeof(float),anum,stream);
  nwritten += fwrite(bbuff,sizeof(float),bnum,stream);
  if(nwritten != (num+anum+bnum)) {
    fprintf(stderr,"putslice2: botched fwrite\n");
    exit(1);
  }
  return(nwritten);
}
