/* cdfpsel -- This program masks certain parcels based on conditions
 * expressed in command line arguments on output of cdflagr.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"
#define MAXFIELDS 50
#define TIME "time"
#define PARCEL "parcel"
#define GT '>'
#define LT '<'
#define NOT '/'

static char hbuff[HBMAX][LINE];               /* header buffer */
static char cline[LINE];                      /* character line buffer */
static long nsbuff,nvbuff;                    /* lengths of slice buffers */
static float *sbuff,*vbuff;                   /* slice buffer pointers */
static float *maskvar;                        /* field used to mask */
static float *othervar[MAXFIELDS];            /* other fields to mask */
static struct field *fp;                      /* temp field pointer */
static int nfields;                           /* number of fields */
static long loop,poop;                        /* looping variables */
static float bad,badlim;                      /* bad data values */
static int nparcels,ntimes;                   /* array size parameters */
static int iparcels,itimes;                 /* looping parameters for arrays */
static float testval;                         /* temporary for mask variable */
static int code;                              /* return code from decode */
static int mask;                              /* flag containing mask info */
static float value;                           /* masking value from decode */
static int decode();

main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc < 2) {
    fprintf(stderr,"Usage: cdfpsel 'field < or > or /< or /> value' ...\n");
    exit(1);
  }
/* get header from standard input */
  gethdr(stdin,hbuff,HBMAX);
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdfpsel: float format expected\n");
    exit(1);
  }
/* get bad data values */
  if (seekpar(hbuff,HBMAX,"bad",cline) == OK) bad = atof(cline);
  else bad = BAD;
  if (seekpar(hbuff,HBMAX,"badlim",cline) == OK) badlim = atof(cline);
  else badlim = BADLIM;
/* add comment */
  strcpy(cline,"cdfpsel:");
  for (loop = 1; loop < argc; loop++) {
    if (strlen(cline) + strlen(argv[loop]) + 3 > LINE) {
      fprintf(stderr,"cdfpsel: too many command line arguments\n");
      exit(1);
    }
    strcat(cline," ");
    strcat(cline,argv[loop]);
  }
  strcat(cline,"\n");
  addcline(hbuff,HBMAX,cline);
/* write header to standard output */
  puthdr(stdout,hbuff,HBMAX);
/* allocate buffer space for slices */
  nsbuff = elemcnt(hbuff,HBMAX,'s');
  sbuff = getbuff(nsbuff);
  nvbuff = elemcnt(hbuff,HBMAX,'v');
  vbuff = getbuff(nvbuff);
/* assign pointers to variable fields subject to masking */
  nfields = loop = 0;
  while ((fp = getfld(hbuff,HBMAX,'v',loop++)) != NULL) {
    if ((fp->dim == 2) && (strcmp(fp->dname1,PARCEL) == 0)
         && (strcmp(fp->dname2,TIME) == 0)) {
      othervar[nfields++] = getptr(hbuff,HBMAX,vbuff,'v','d',fp->fname);
      nparcels = fp->dsize1;
      ntimes = fp->dsize2;
    }
  }
/* pass static slice */
  getslice(stdin,nsbuff,sbuff);
  putslice(stdout,nsbuff,sbuff);
/* loop on variable slice reads */
  while (getslice(stdin,nvbuff,vbuff) != EOF) {
/* loop on command line arguments */
    for (loop = 1; loop < argc; loop++) {
/* decode argument */
      code = decode(argv[loop],cline,&value);
      maskvar = getptr(hbuff,HBMAX,vbuff,'v','d',cline);
/* do mask job */
      for (iparcels = 0; iparcels < nparcels; iparcels++) {
/* first, find mask condition on particular parcel */
        if (code <= 2) mask = 1;
        else mask = 0;
        for (itimes = 0; itimes < ntimes; itimes++) {
          testval = maskvar[itimes + ntimes*iparcels];
          if (fabs(testval) < badlim) {
            if ((code == 1) && (testval < value)) mask = 0;
            else if ((code == 2) && (testval > value)) mask = 0;
            else if ((code == 3) && (testval < value)) mask = 1;
            else if ((code == 4) && (testval > value)) mask = 1;
          }
        }
/* then, carry out masking */
        for (poop = 0; poop < nfields; poop++) {
          for (itimes = 0; itimes < ntimes; itimes++) {
            if (mask) {
              othervar[poop][itimes + ntimes*iparcels] = bad;
            }
          }
        }
      }
    }
/* write variable slice */
    putslice(stdout,nvbuff,vbuff);
  }
  exit(0);
}

/* decode a command line argument */
static int decode(arg,fname,valueptr)
char *arg;
char *fname;
float *valueptr;
{
  char buffer[LINE];                /* line buffer */
  char c;                           /* single character buffer */
  int pointer;                      /* scan pointer */
  int length;                       /* length of line buffer */
/* check for length of argument */
  if (strlen(arg) > LINE) {
    fprintf(stderr,"cdfpsel: command line argument %s too long\n",arg);
    exit(1);
  }
/* set pointer to beginning of line */
  length = pointer = 0;
/* scan for beginning of operator, saving stuff for name */
  while ((c = arg[pointer]) != '\0') {
    if ((c == ' ') || (c == '\t')) {
      pointer++;
      continue;
    }
    else if ((c == LT) || (c == GT) || (c == NOT)) break;
    else {
      buffer[length++] = c;
      pointer++;
    }
  }
  buffer[length] = '\0';
/* check the header on the name */
  if ((fp = seekfld(hbuff,HBMAX,'v',buffer)) == NULL) {
    fprintf(stderr,"cdfpsel: can't find variable field %s\n",buffer);
    exit(1);
  }
  if ((fp->dim != 2) || (strcmp(fp->dname1,PARCEL) != 0) ||
       (strcmp(fp->dname2,TIME) != 0)) {
    fprintf(stderr,"cdfpsel: field %s inappropriate for masking\n",buffer);
    exit(1);
  }
/* copy the name */
  strcpy(fname,buffer);
/* determine the operator */
  if (c == NOT) {
    c = arg[++pointer];
    if (c == LT) code = 3;
    else if (c == GT) code = 4;
    else code = 0;
  }
  else if (c == LT) code = 1;
  else if (c == GT) code = 2;
  else code = 0;
  if (code == 0) {
    fprintf(stderr,"cdfpsel: syntax error in argument %s\n",arg);
    exit(1);
  }
/* determine the mask value */
  pointer++;
  length = 0;
  while ((c = arg[pointer++]) != '\0') {
    if ((c == ' ') || (c == '\t')) {
      continue;
    }
    else {
      buffer[length++] = c;
    }
  }
  buffer[length] = '\0';
  *valueptr = atof(buffer);
/* return the code */
  return(code);
}
