/* cdfsmooth -- This program applies a low pass filter to the named
 * dimensions.  The smoothing length is also specified on the command
 * line for each dimension.
 */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

static char buffer[HBMAX][LINE],bufferw[HBMAX][LINE];
                       /* buffers for header data */
static float *sbuffs,*sbuffv;
                       /* buffers that contain slice & computed data */

main(argc,argv)
int argc;
char *argv[];
{
  int pos,loop,vpos,lpos;     /* used in looping */
  int dsiz[4];                /* pointers to area that contains data */
  int numset;                 /* number of test_var on command line */
  char varname[4][LINE];      /* contains names of dimensions */
  char cl[LINE],pvalue[LINE]; /* comment line data */
  long nel;                   /* size of data to be read and written */
  long inst,inc,siz,where;    /* used with the subspace routine */
  float dx,badlim;            /* used in computing the diff */
  float lambda[4],alpha;      /* the smoothing paramaters */
  float pta,ptb;              /* points taken for the weighted average */
  float *douts,*doutv;        /* used for updating data (var and static) */
  struct field *fp;           /* pointer to the field structure */

/* check command line arguments */
  if (((argc-1)%2 != 0)||(argc==1)) {
    fprintf(stderr,"Usage: cdfsmooth var1 lambda1 [var2 lambda 2 ...]\n");
    exit(1);
  }
/* move stuff out of argument line */
  numset=(argc-1)/2;
  if (numset>4) {
    fprintf(stderr,"cdfsmooth: too many variables\n");
    exit(1);
  }
/* check lambda to make sure that it is not zero */
  for (loop=0;loop<numset;loop++) {
    strcpy(varname[loop],argv[loop*2+1]);
    lambda[loop]=atof(argv[loop*2+2]);
    if (fabs(lambda[loop]) < 1.e-20) {
      fprintf(stderr,"cdfsmooth: lambda must not equal zero\n");
      exit(1);
    }
  }
/* get header from stdin */
  gethdr(stdin,buffer,HBMAX);
/* check format */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfsmooth: file must be float!\n");
    exit(1);
  }
/* check var fields */
  for (loop=0;loop<numset;loop++) {
    if((fp=seekfld(buffer,HBMAX,'s',varname[loop]))==NULL) {
      fprintf(stderr,"cdfsmooth: %s not an index field\n",varname[loop]);
      exit(1);
    }
    else if ((fp->dim != 1)||(strcmp(fp->dname1,varname[loop]) != 0)) {
      fprintf(stderr,"cdfsmooth: error with the test_var field");
      fprintf(stderr," %s\n",varname[loop]);
      exit(1);
    }
    else
      dsiz[loop]=fp->dsize1;
  }
/* make output header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer comments */
  copycmt(bufferw,HBMAX,buffer,HBMAX);
/* add comment line */
  sprintf(cl,"cdfsmooth: ");
  for (loop=1;loop<argc;loop++) {
    strcat(cl,argv[loop]);
    strcat(cl," ");
  }
  strcat(cl,"\n");
  addcline(bufferw,HBMAX,cl);
/* transfer parameters */
  copypar(bufferw,HBMAX,buffer,HBMAX);
  if (seekpar(buffer,HBMAX,"badlim",pvalue)==FAIL)
    badlim=BADLIM;
  else
    badlim=atof(pvalue);
/* transfer static fields */
  copyfld(bufferw,HBMAX,buffer,HBMAX,'s');
/* transfer variable fields */
  copyfld(bufferw,HBMAX,buffer,HBMAX,'v');
/* send header to stdout */
  puthdr(stdout,bufferw,HBMAX);
/* allocate memory for static slice */
  nel=elemcnt(buffer,HBMAX,'s');
  sbuffs=getbuff(nel);
  getslice(stdin,nel,sbuffs);
/* send static slice */
  putslice(stdout,nel,sbuffs);
/* allocate memory for variable slice */
  nel=elemcnt(buffer,HBMAX,'v');
  sbuffv=getbuff(nel);
/* loop through the variable slices if more than one */
  while (getslice(stdin,nel,sbuffv) != EOF) {
/* loop through the variable fields looking for ones that need to be smoothed */
    vpos=0;
    while ((fp=getfld(buffer,HBMAX,'v',vpos++))!=NULL) {
      doutv=getptr(buffer,HBMAX,sbuffv,'v','d',fp->fname);
/* loop through the dimensions to be smoothed */
      for (lpos=0;lpos<numset;lpos++) {
        douts=getptr(buffer,HBMAX,sbuffs,'s','d',varname[lpos]);
/* loop through all the instances of that dimension within field */
        for (inst=0;inst<(fp->tsize/dsiz[lpos]);inst++) {
/* is the dimension present, if not skip field */
          if (subspace1(fp,varname[lpos],inst,&where,&inc,&siz)!=FAIL) {
/* else loop forward through the subspace of data */
            for (loop=1;loop<siz-1;loop++) {
/* find dx and alpha */
              dx=douts[loop]-douts[loop-1];
              alpha=fabs(dx/lambda[lpos]);
/* is alpha within the limits (i.e  0 < alpha < 1.0 )? */
              if (alpha > 1.0) {
                fprintf(stderr,"cdfsmooth: smoothing length %f too small\n",
                     lambda[lpos]);
                fprintf(stderr,"   (forward: dx, alpha = %g, %g)\n",dx,alpha);
                exit(1);
              }
/* 1st part of smoothing process (also make sure data is good first) */
/* amid(i)=aold(i)*alpha+amid(i-1)*(1-alpha) */
              pos=where+inc*loop;
              pta=doutv[pos];
              ptb=doutv[pos-inc];
              if ((fabs(pta)<badlim)&&(fabs(ptb)<badlim))
                doutv[pos]=pta*alpha+ptb*(1-alpha);
            }
/* loop backward through the subspace of data */
            for (loop=siz-2;loop>0;loop--) {
/* find dx and alpha again */
              dx=douts[loop+1]-douts[loop];
              alpha=fabs(dx/lambda[lpos]);
/* is alpha within the limits? */
              if (alpha > 1.0) {
                fprintf(stderr,"cdfsmooth: smoothing length %f too small\n",
                     lambda[lpos]);
                fprintf(stderr,"   (backward: dx, alpha = %g, %g)\n",dx,alpha);
                exit(1);
              }
/* 2nd part of smoothing process */
/* anew(i)=amid(i)*alpha+anew(i+1)*(1-alpha) */
              pos=where+inc*loop;
              if ((fabs(doutv[pos])<badlim)&&(fabs(doutv[pos+inc])<badlim))
                doutv[pos]=doutv[pos]*alpha+doutv[pos+inc]*(1-alpha);
            }
          }
        }
      }
    }
/* output the variable slice to stdin and check for another */
    putslice(stdout,nel,sbuffv);
  }
  exit(0);
}
