/* cdftable -- create a Candis file from a table of numbers */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define MAXVARS 100
#define MAXBUFF 5000

static char buff[HBMAX][LINE];
static float *vbuff;
static long nvbuff;
static int nvars;
static float indexfld;

main(argc,argv)
int argc;
char *argv[];
{
  int loop,offset;
  char comment[81],cbuff[121],inbuff[MAXBUFF],token[MAXBUFF];
  int ptr;

/* check command line */
  if (argc == 1) {
    fprintf(stderr,"Usage: cdftable [-c comment] name1 name2 ...\n");
    exit(1);
  }
  nvars = argc - 1;
  if (nvars > MAXVARS) {
    fprintf(stderr,"cdftable: a maximum of %d variables allowed\n",MAXVARS);
    exit(1);
  }

  /* get possible comment */
  offset = 0;
  comment[0] = '\0';
  if (strcmp(argv[1],"-c") == 0) {
    offset = 2;
    strcpy(comment,argv[2]);
  }

/* make header and write */
  nullhdr(buff,HBMAX,"float");
  sprintf(cbuff,"cdftable: %s\n",comment);
  addcline(buff,HBMAX,cbuff);
  addfld(buff,HBMAX,'v',"index",1.,0.,'l',0);
  for (loop = 1 + offset; loop <= nvars; loop++) {
    addfld(buff,HBMAX,'v',argv[loop],1.,0.,'l',0);
  }
  puthdr(stdout,buff,HBMAX);

/* allocate buffer space */
  nvbuff = elemcnt(buff,HBMAX,'v');
  vbuff = getbuff(nvbuff);

/* write a null static slice */
  putslice(stdout,0,vbuff);

/* loop on input */
  indexfld = 0.;
  while (fgets(inbuff,MAXBUFF,stdin) != (char*)NULL) {

/* skip comment line */
    if (inbuff[0] == '#') continue;
    vbuff[0] = indexfld;
    indexfld += 1.;
    ptr = 0;
    for (loop = 1; loop < nvbuff; loop++) {
      while (inbuff[ptr] == ' ' || inbuff[ptr] == '\t') ptr++;
      sscanf(inbuff + ptr,"%s",token);
      *(vbuff + loop) = atof(token);
      while (inbuff[ptr] != ' ' && inbuff[ptr] != '\t' &&
	     inbuff[ptr] != '\n' && inbuff[ptr] != '\0') ptr++;
    }

/* write a variable slice */
    putslice(stdout,nvbuff,vbuff);
  }
  exit(0);
}
