/* cdfthin                                                            */
/* usage: cdfthin dname1 increment dname2 increment ......            */
/* cdfthin uses stdout & stdin for all i/o operations and can handel  */
/* 4d files.  Cdfthin accepts up to four dnames with a corresponing   */
/* increment value.  Cdfthin uses the increment value to              */
/* calculate the domains to remove from the field.                    */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define MA 4                   /* max number of agument sets */

static float *sbuff, *sbuffout, *vbuff, *vbuffout;
                          /* buffer for slice data */
static char hbuffer[HBMAX][LINE],hbufferw[HBMAX][LINE];
                          /* buffer for headers */

main(argc, argv)
int argc;
char *argv[];
{
  int loop,loop1,loop2;   /* counters for loops */
  int numsets;            /* number of pararameter sets */
  int ivalue[MA];         /* contains command line data (up & low) (int) */
  int x,y,z,t;            /* used in cycling thru data to be updated */
  int xi,yi,zi,ti;        /* increment size for (x,y,z&t) */
  int ds1,ds2,ds3,ds4;    /* dim size of  current field being sliced */
  int bpos,bposout,add;   /* position of element being read or written */
  int par[MA];            /* conditions for parameter updates */
  char pvalue[LINE],pname[LINE];    /* contains parameter data */
  char pnamec[MA][LINE];  /* contains data for comparing parameters */
  char dname[MA][LINE];   /* command line data (element to update) */
  char field;             /* which field is being updated (s or v) */
  char cl[LINE];          /* comment line */
  long numread;           /* number of lines read or status */
  long nel,nelw;          /* number of elements read and to write */
  float fvalue[MA];       /* float value for new increment value */
  struct field *fp;       /* pointer to field structure of data */
  struct field ftable[HBMAX];  /* table of variable field pointers */
  int nvfields;           /* number of variable fields */

/* check usage */
  if (((argc-1)%2 != 0) || (argc == 1)) {
    fprintf(stderr,"Usage:  cdfthin dname1 increment_value1 ...\n");
    exit(1);
  }
/* get header info and check format (must be float) */
  gethdr(stdin,hbuffer,HBMAX);
  if (getfmt(hbuffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfthin:  file must be float format\n");
    exit(1);
  }
/* determine # of command line argument sets */
  numsets = (argc-1)/2;
  if (numsets > MA) {
    fprintf(stderr,"cdfthin: too many command line arguments\n");
    exit(1);
  }
/* place arguments into respective variables */
  for (loop1=1,loop2=0;loop1<argc;loop2++,loop1+=2) {
    strcpy(dname[loop2],argv[loop1]);
    sprintf(pnamec[loop2],"d%s",dname[loop2]);
    ivalue[loop2] = atoi(argv[(loop1+1)]);
    if (ivalue[loop2] <= 0) {
      fprintf(stderr,"cdfthin: increment value negative or zero\n");
      exit(1);
    }
  }
/* check validity of names (dname must be in static field) */
  for (loop1=0;loop1<numsets;++loop1) {
    fp=seekfld(hbuffer,HBMAX,'s',dname[loop1]);
    if ((fp==NULL)||(fp->dim!=1)||(strcmp(dname[loop1],fp->dname1)!=0)) {
      fprintf(stderr,"cdfthin:  %s isn't an index field\n",dname[loop1]);
      exit(1);
    }
  }
/* find increment value for parameter update */
  for (loop=0;loop<numsets;++loop) {
    if((seekpar(hbuffer,HBMAX,pnamec[loop],pvalue)) != FAIL) {
      if (atof(pvalue)>fvalue[loop])
        fvalue[loop]=atof(pvalue);
    }
    else {
      strcpy(pvalue,"1.0");
      if (1.0>fvalue[loop])
        fvalue[loop]=1.0;
    }
    fvalue[loop]=ivalue[loop]*atof(pvalue);
  }
/* make new header */
  nullhdr(hbufferw,HBMAX,"float");
/* transfer comments */
  copycmt(hbufferw,HBMAX,hbuffer,HBMAX);
/* check parameters for updating */
  for (loop=0;loop<MA;loop++)
    par[loop]=0;
  loop1=0;
  while ((numread=getpar(hbuffer,HBMAX,loop1++,pname,pvalue)) != FAIL) {
    for (loop2=0;loop2<numsets;++loop2) {
      if (strcmp(pname,pnamec[loop2]) == 0) {
        sprintf(cl,"%f",fvalue[loop2]);
        strcpy(pvalue,cl);
        par[loop2]=1;
      }
    }
    addpar(hbufferw,HBMAX,pname,pvalue);
  }
/* if d_dname is not present then create it containing fvalue */
  for (loop=0;loop<numsets;loop++) {
    if (par[loop]==0) {
      sprintf(pvalue,"%f",fvalue[loop]);
      addpar(hbufferw,HBMAX,dname[loop],pvalue);
    }
  }
/* transfer static & variable fields making updates */
/* loop = 0 transfer static field, loop = 1 transfer variable field */
  for (loop=0;loop<2;loop++) {
    if (loop==0)
      field = 's';
    else
      field = 'v';
    loop1=0;
    while ((fp=getfld(hbuffer,HBMAX,field,loop1++)) != NULL) {
/* find which (if any) of the dimensions to update */
      for (loop2=0;loop2<numsets;++loop2) {
        ds1=ds2=ds3=ds4=0;
        if (strcmp(fp->dname1,dname[loop2])==0) {
          for (x=0;x<fp->dsize1;x+=ivalue[loop2])
            ds1++;
          fp->dsize1 = ds1;
        }
        else if (strcmp(fp->dname2,dname[loop2])==0) {
          for(y=0;y<fp->dsize2;y+=ivalue[loop2])
            ds2++;
          fp->dsize2 = ds2;
        }
        else if (strcmp(fp->dname3,dname[loop2])==0) {
          for (z=0;z<fp->dsize3;z+=ivalue[loop2])
            ds3++;
          fp->dsize3 = ds3;
        }
        else if (strcmp(fp->dname4,dname[loop2])==0) {
          for (t=0;t<fp->dsize4;t+=ivalue[loop2])
            ds4++;
          fp->dsize4 = ds4;
        }
      }
/* send updated field (static or variable) to buffer */
      addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,
          fp->bytes,fp->dim,fp->dname1,fp->dsize1,fp->dname2,
          fp->dsize2,fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
    }
  }
/* make comment line */
  strcpy(cl,"cdfthin: ");
  for (loop=1;loop<argc;loop++) {
    strcat(cl,argv[loop]);
    strcat(cl," ");
  }
  strcat(cl,"\n");
  addcline(hbufferw,HBMAX,cl);
/* new header is updated, send to stdout */
  puthdr(stdout,hbufferw,HBMAX);
/* set aside memory for out slice */
  nelw=elemcnt(hbufferw,HBMAX,'s');
  nel=elemcnt(hbuffer,HBMAX,'s');
  sbuffout=(float*)malloc(sizeof(float)*nelw);
  sbuff=(float*)malloc(sizeof(float)*nel);
  if ((sbuffout == NULL)||(sbuff == NULL)) {
    fprintf(stderr,"cdfthin:  not enough memory to hold slice data\n");
    exit(1);
  }
  getslice(stdin,nel,sbuff);
/* begin updating and sending to output buffer (sbuffout) */
  loop2=bpos=bposout=0;
  while ((fp = getfld(hbuffer,HBMAX,'s',loop2++)) != NULL) {
    if (fp->dim != 0) {
      ds1=ds2=ds3=ds4=1;
      if (fp->dim >= 1) ds1=fp->dsize1;
      if (fp->dim >= 2) ds2=fp->dsize2;
      if (fp->dim >= 3) ds3=fp->dsize3;
      if (fp->dim >= 4) ds4=fp->dsize4;
/* determine which of the dimensions to thin */
      xi=yi=zi=ti=1;
      for (loop1=0;loop1<numsets;loop1++) {
        if ((fp->dim>=1)&&(strcmp(fp->dname1,dname[loop1])==0))
          xi=ivalue[loop1];
        if ((fp->dim>=2)&&(strcmp(fp->dname2,dname[loop1])==0))
          yi=ivalue[loop1];
        if ((fp->dim>=3)&&(strcmp(fp->dname3,dname[loop1])==0))
          zi=ivalue[loop1];
        if ((fp->dim>=4)&&(strcmp(fp->dname4,dname[loop1])==0))
          ti=ivalue[loop1];
      }
/* cycle thru the slice data */
      for (x=0;x<ds1;x+=xi) {
        for (y=0;y<ds2;y+=yi) {
          for (z=0;z<ds3;z+=zi) {
            for (t=0;t<ds4;t+=ti) {
/* cycle thru data and send 'thined' data to sbuffout */
              add=(t+z*ds4+y*ds4*ds3+x*ds4*ds3*ds2);
              sbuffout[bposout++]=sbuff[bpos+add];
            }
          }
        }
      }
      bpos+=fp->tsize;
    }
    if (fp->dim == 0) {
      sbuffout[bposout++]=sbuff[bpos++];
    }
  }
/* send static slice to output */
  putslice(stdout,nelw,sbuffout);
/* free static buffers and set aside memory for variable buffers */
  free(sbuff);
  free(sbuffout);
  nel=elemcnt(hbuffer,HBMAX,'v');
  nelw=elemcnt(hbufferw,HBMAX,'v');
  vbuff=(float*)malloc(sizeof(float)*nel);
  vbuffout=(float*)malloc(sizeof(float)*nelw);
  if ((vbuff == NULL) || (vbuffout == NULL)) {
    fprintf(stderr,"cdfthin:  not enough memory to hold slice data\n");
    exit(1);
  }
/* begin above routine for variable slice */
/* first make a table of all the field pointers in the variable slice */
  nvfields = 0;
  while ((fp = getfld(hbuffer,HBMAX,'v',nvfields)) != NULL) {
    if (nvfields >= HBMAX) {
      fprintf(stderr,"cdfthin: too many variable fields\n");
      exit(1);
    }
    ftable[nvfields++] = *fp;
  }
/* loop thru all slices */
  while (getslice(stdin,nel,vbuff) != EOF) {
    bpos=bposout=0;
/* loop thru each field and send data within the parameters */
    for (loop2 = 0; loop2 < nvfields; loop2++) {
      fp = &ftable[loop2];
      if (fp->dim != 0) {
        ds1=ds2=ds3=ds4=1;
        if (fp->dim >= 1) ds1=fp->dsize1;
        if (fp->dim >= 2) ds2=fp->dsize2;
        if (fp->dim >= 3) ds3=fp->dsize3;
        if (fp->dim >= 4) ds4=fp->dsize4;
/* which element to take out? (x,y,z,t) */
        xi=yi=zi=ti=1;
        for (loop1=0;loop1<numsets;loop1++) {
          if ((fp->dim>=1)&&(strcmp(fp->dname1,dname[loop1])==0))
            xi=ivalue[loop1];
          if ((fp->dim>=2)&&(strcmp(fp->dname2,dname[loop1])==0))
            yi=ivalue[loop1];
          if ((fp->dim>=3)&&(strcmp(fp->dname3,dname[loop1])==0))
            zi=ivalue[loop1];
          if ((fp->dim>=4)&&(strcmp(fp->dname4,dname[loop1])==0))
            ti=ivalue[loop1];
        }
/* cycle thru data */
        for (x=0;x<ds1;x+=xi) {
          for (y=0;y<ds2;y+=yi) {
            for (z=0;z<ds3;z+=zi) {
              for (t=0;t<ds4;t+=ti) {
/* if data is within the given paramaters, send to vbuffout */
                add=(t+z*ds4+y*ds4*ds3+x*ds4*ds3*ds2);
                vbuffout[bposout++]=vbuff[bpos+add];
              }
            }
          }
        }
        bpos+=fp->tsize;
      }
      if (fp->dim == 0) {
        vbuffout[bposout++]=vbuff[bpos++];
      }
    }
/* output slice to stdout and cycle thru slice 'til all have been searched */
    putslice(stdout,nelw,vbuffout);
  }
  exit(0);
}
