/* cdftrans -- This program translates Candis files from one format
 * to another.  This new version uses the slice readers and writers
 * in auxio.c in the utils subdirectory.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbin[HBMAX][LINE],hbout[HBMAX][LINE]; /* header buffers */
static char informat,outformat;             /* file formats */
static char swapflag;                       /* swap bytes on integers if 1 */
static int size[HBMAX];                     /* integer element sizes */
static long length[HBMAX],offset[HBMAX];    /* field sizes and offsets */
static float add[HBMAX],mul[HBMAX];         /* integer scaling parameters */
static float *sbuff;                        /* slice buffer pointer */
static long nsbuff;                         /* buffer length */


main(argc,argv)
int argc;
char *argv[];
{
/* check arguments */
  if (argc < 2 || argc > 3) {
    fprintf(stderr,"Usage: cdftrans -[a|i|f] [-s]\n");
    exit(1);
  }
  if (strcmp(argv[1],"-a") == 0) outformat = 'a';
  else if (strcmp(argv[1],"-i") == 0) outformat = 'i';
  else if (strcmp(argv[1],"-f") == 0) outformat = 'f';
  else {
    fprintf(stderr,"cdftrans: unknown format %s\n",argv[1]);
    exit(1);
  }
  swapflag = 0;
  if (argc == 3) {
    if (strcmp(argv[2],"-s") == 0) swapflag = 1;
    else {
      fprintf(stderr,"Usage: cdftrans -[a|i|f] [-s]\n");
      exit(1);
    }
  }
/* get input header */
  gethdr(stdin,hbin,HBMAX);
/* get its format */
  informat = getfmt(hbin,HBMAX);
  if (swapflag && informat == 'a') {
    fprintf(stderr,
         "cdftrans: byte swapping doesn't work with ascii format on input\n");
    exit(1);
  }
/* make new header */
  if (outformat == 'a') nullhdr(hbout,HBMAX,"ascii");
  else if (outformat == 'i') nullhdr(hbout,HBMAX,"int");
  else nullhdr(hbout,HBMAX,"float");
  copycmt(hbout,HBMAX,hbin,HBMAX);
  if (swapflag) addcline(hbout,HBMAX,"cdftrans: -s\n");
  copypar(hbout,HBMAX,hbin,HBMAX);
  copyfld(hbout,HBMAX,hbin,HBMAX,'s');
  copyfld(hbout,HBMAX,hbin,HBMAX,'v');
/* write it out */
  puthdr(stdout,hbout,HBMAX);
/* static slice */
/* get needed structure information */
  getilist(hbin,HBMAX,'s',size,length,offset,add,mul);
/* allocate slice buffer */
  nsbuff = elemcnt(hbin,HBMAX,'s');
  sbuff = getbuff(nsbuff);
/* get slice */
  if (informat == 'a') gaslice(stdin,size,length,offset,nsbuff,sbuff);
  else if (informat == 'f') {
    getslice(stdin,nsbuff,sbuff);
    if (swapflag) floatswap(nsbuff,sbuff);
  }
  else gislice(stdin,size,length,offset,add,mul,nsbuff,sbuff,swapflag);
/* put slice */
  if (outformat == 'a') paslice(stdout,size,length,offset,nsbuff,sbuff);
  else if (outformat == 'f') putslice(stdout,nsbuff,sbuff);
  else pislice(stdout,size,length,offset,add,mul,nsbuff,sbuff);
/* free slice buffer */
  free(sbuff);
/* variable slices */
/* get needed structure information */
  getilist(hbin,HBMAX,'v',size,length,offset,add,mul);
/* allocate slice buffer */
  nsbuff = elemcnt(hbin,HBMAX,'v');
  sbuff = getbuff(nsbuff);
/* loop on variable slices */
  for (;;) {
    if ((informat == 'a') &&
         (gaslice(stdin,size,length,offset,nsbuff,sbuff) == EOF)) break;
    else if ((informat == 'f') && (getslice(stdin,nsbuff,sbuff) == EOF)) break;
    else if ((informat == 'i') && (gislice(stdin,size,length,offset,
         add,mul,nsbuff,sbuff,swapflag) == EOF)) break;
/* swap float bytes if demanded */
    if (swapflag) floatswap(nsbuff,sbuff);
/* put slice */
    if (outformat == 'a') paslice(stdout,size,length,offset,nsbuff,sbuff);
    else if (outformat == 'f') putslice(stdout,nsbuff,sbuff);
    else pislice(stdout,size,length,offset,add,mul,nsbuff,sbuff);
  }
  exit(0);
}
