/* cdftsel.c -- This filter looks for the variable field named on the
 * command line and selects all variable slices with the value of this 
 * variable between the starting and stopping "times" given by 
 * the user. If no such slices are encountered, an error halt is made.
 */
#include <stdio.h>
#include <math.h>
#include "cdfhdr.h"

static char hbuff[HBMAX][LINE],cline[LINE];
static int slices;
static float *heap;
static long ns,nv,poop;
static float begtime,endtime;
static struct field *time;

main(argc,argv)
int argc;
char *argv[];
{
/* check argument list */
  if (argc != 4) {
    fprintf(stderr,"Usage: cdftsel time_var_name beginning_time ending_time\n");
    exit(1);
  }
/* convert times to float format */
  begtime = atof(argv[2]);
  endtime = atof(argv[3]);
/* get header buffer from standard input */
  gethdr(stdin,hbuff,HBMAX);
/* check format and die if not float */
  if (getfmt(hbuff,HBMAX) != 'f') {
    fprintf(stderr,"cdftsel: format not float\n");
    exit(1);
  }
/* check to see if there is a time variable */
  time = seekfld(hbuff,HBMAX,'v',argv[1]);
  if ((time == NULL) || (time->dim != 0)) {
    fprintf(stderr,"cdftsel: no scalar %s field\n",argv[1]);
    exit(1);
  }
/* get static and variable slice lengths */
  ns = elemcnt(hbuff,HBMAX,'s');
  nv = elemcnt(hbuff,HBMAX,'v');
/* add comment line */
  sprintf(cline,"cdftsel: %s %s %s\n",argv[1],argv[2],argv[3]);
  addcline(hbuff,HBMAX,cline);
/* write header to standard output */
  puthdr(stdout,hbuff,HBMAX);
/* transfer static slice */
  if ((heap = (float *)malloc(ns*sizeof(float))) == NULL) {
    fprintf(stderr,"cdftsel: can't allocate enough memory\n");
    exit(1);
  }
  getslice(stdin,ns,heap);
  putslice(stdout,ns,heap);
  free(heap);
/* allocate memory for variable slice */
  if ((heap = (float *)malloc(nv*sizeof(float))) == NULL) {
    fprintf(stderr,"cdftsel: can't allocate enough memory\n");
    exit(1);
  }
  /*if ((heap2 = (float *)malloc(nv*sizeof(float))) == NULL) {
    fprintf(stderr,"cdftsel: can't allocate enough memory\n");
    exit(1);
  }*/
/* repeatedly get variable slice */
  slices = 0;
  for (;;) {
    if (getslice(stdin,nv,heap) == EOF) break;
/* check for time condition */
    if (heap[time->ptr] > endtime) continue;
    if (heap[time->ptr] >= begtime) {
/* found a slice we want */
      slices++;
      /* put the slice */
      putslice(stdout,nv,heap);
    }
  }
  if (slices == 0) {
    fprintf(stderr,"cdftsel: no times in requested interval\n");
    exit(1);
  }
  exit(0);
}
