/* cdfuniq:                                                              */
/* usage: cdfuniq < in_file > out_file                                   */
/* cdfuniq looks at every variable field that is not a scalar and checks */
/* to see if all the data is the same.  If they are the same then change */
/* the field to a scalar containing that value                           */
#include <stdio.h>
#include <string.h>
#include "cdfhdr.h"

static char buffer[HBMAX][LINE];              /* buffer for header info */
static char bufferw[HBMAX][LINE];             /* buffer for revised header */
static char cbuff[HBMAX/2][LINE];             /* buffer for comments */
static float *sbuff,*vbuff,*vbuffout;         /* buffers for slice data */

main(argc, argv)
int argc;
char *argv[];
{
  int flag;             /* keeps track of succesive numbers */
  int vpos,loop;        /* for looping though fields and data */
  long nel,nr,buffstat; /* length and positions in slice data */
  float fvalue;         /* used in comparison */
  struct field *fp;     /* pointer to the field structure of data */

/* check argument list */
  if (argc != 1) {
    fprintf(stderr,"usage: cdfuniq\n");
    exit(1);
  }
/* get the header */
  gethdr(stdin,buffer,HBMAX);
/* check format */
  if (getfmt(buffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfuniq: expecting float format\n");
    exit(1);
  }
/* create new header */
  nullhdr(bufferw,HBMAX,"float");
/* transfer comments, paramaters, and static fields */
  copypar(bufferw,HBMAX,buffer,HBMAX);
  copycmt(bufferw,HBMAX,buffer,HBMAX);
  copyfld(bufferw,HBMAX,buffer,HBMAX,'s');
  addcline(bufferw,HBMAX,strcat(argv[0],"\n"));
/* set aside memory for slices */
  nr=elemcnt(buffer,HBMAX,'s');
  nel=elemcnt(buffer,HBMAX,'v');
  vbuff=getbuff(nel);
/* output variable slice buffer can be the same as the input buffer */
  vbuffout = vbuff;
  sbuff=getbuff(nr);
/* read in slices */
  getslice(stdin,nr,sbuff);
  getslice(stdin,nel,vbuff);
/* cycle through the variable fields */
  vpos=buffstat=0;
  while ((fp=getfld(buffer,HBMAX,'v',vpos++)) != NULL) {
/* if a scalar then send without change or manipulation */
    if (fp->dim == 0) {
      addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,0);
      vbuffout[buffstat++]=vbuff[fp->ptr];
    }
/* if non-scalar */
    else {
      flag = 0;
/* cycle through the data checking to see if each point is the same as the */
/* first point */
      fvalue = vbuff[fp->ptr];
      for (loop=1;loop<fp->tsize;loop++) {
        if (fvalue != vbuff[loop+fp->ptr]) {
          flag=1;
          break;
        }
      }
/* if the data is not the same throughout then send */
      if (flag==1) {
        addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,
                 fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,
                 fp->dname3,fp->dsize3,fp->dname4,fp->dsize4);
        for (loop=0;loop<fp->tsize;loop++)
           vbuffout[buffstat++]=vbuff[loop+fp->ptr];
      }
/* else send only the one point */
      else {
        addfld(bufferw,HBMAX,'v',fp->fname,fp->smul,fp->sadd,fp->bytes,0);
        vbuffout[buffstat++]=fvalue;
      }
    }
  }
/* output the data */
  puthdr(stdout,bufferw,HBMAX);
  putslice(stdout,nr,sbuff);
  putslice(stdout,buffstat,vbuffout);
  exit(0);
}
