/* cdfwindow -- cuts out one or more dimension segments */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cdfhdr.h"

#define MA 4			  /* max number of agument sets */

static char hbuffer[HBMAX][LINE],hbufferw[HBMAX][LINE];
                                  /* buffer for headers */
static char name[MA][LINE];	  /* command line data (element to update) */
static float *sbuff,*sbuffout;	  /* buffer for slice data */
static float bad;
static int inbox();

main(argc, argv)
int argc;
char *argv[];
{
  int loop,loop1,loop2;    /* counters for loops */
  int numsets,flag1,flag2; /* number of pararameter sets and flags */
  int lo[MA],hi[MA];       /* index field cuts in no fill case */
  float lotemp,hitemp;     /* temporaries */
  int par[MA],ninc[MA];    /* flags for parameters and variable updates */
  int start[MA],stop[MA];  /* looping size for in and out data */
  int dsold[MA];           /* dimension size of the "old" index variables */
  int index[MA];           /* num points above or below (index < 0) old0 */
  int s1,s2,status,numrd;  /* status and temp variables */
  int x,y,z,t,xp,yp,zp,tp; /* used in cycling thru data to be updated */
  int ds1,ds2,ds3,ds4;     /* dim size of  current field being sliced */
  int low[MA],high[MA];    /* extraction limits */
  int way[MA],size[MA];    /* flag and which way to window (pad or not) */
  int locnt[MA],hicnt[MA]; /* num index points away from old0, old9, resp. */
  int extflag[MA];         /* flag: 1 - fld extends down, 2 - up, 3 - both */
  char pnamec[MA][LINE];   /* contains data for comparing parameters */
  char dname[MA][LINE];    /* name of the increment parameters */
  char pvalue[LINE];       /* contains parameter data */
  char pname[LINE],field;  /* which field is being updated (s or v) */
  char cl[LINE];           /* comment line */
  long nel,nelw;           /* number of elements to read and write */
  float flo[MA],fhi[MA];   /* contains command line data (float) */
  float dvalue[MA],dif;	   /* value of name */
  float old0[MA],old9[MA]; /* initial value (old data) and final value (old) */
  float chek1,chek2,temp;  /* temp storage */
  float *din,*dout;        /* pointers to in and out data */
  float newflo, newfhi;    /* temporary flo, fhi values */
  float *tempfld[MA];      /* pointer to new extended index fields */
  float newdelta;          /* temporary delta value for irregular fields */
  struct field *fp;        /* pointer to field structure of data */

/* check usage */
  if (((argc-1)%3 != 0) || (argc == 1)) {
    fprintf(stderr,"Usage:  cdfwindow index_var1 start1 stop1 ...\n");
    exit(1);
  }

/* get header info and check format (must be float) */
  gethdr(stdin,hbuffer,HBMAX);
  if (getfmt(hbuffer,HBMAX) != 'f') {
    fprintf(stderr,"cdfwindow:  file must be float format\n");
    exit(1);
  }

/* determine # of command line argument sets */
  numsets = (argc-1)/3;
  if (numsets > MA) {
    fprintf(stderr,"cdfwindow: too many command line arguments\n");
    exit(1);
  }

/* place arguments into respective variables */
  for (loop1=1,loop2=0;loop1<argc;loop2++,loop1+=3) {
    strcpy(name[loop2],argv[loop1]);
    sprintf(pnamec[loop2],"%s0",name[loop2]);
    sprintf(dname[loop2],"d%s",name[loop2]);
    flo[loop2] = atof(argv[(loop1+1)]);
    fhi[loop2] = atof(argv[(loop1+2)]);
  }

/* read static slice */
  nel = elemcnt(hbuffer,HBMAX,'s');
  sbuff=getbuff(nel);
  status=getslice(stdin,nel,sbuff);

/* check validity of names (name must be in static field) */
  for (loop1=0;loop1<numsets;loop1++) {
    fp=seekfld(hbuffer,HBMAX,'s',name[loop1]);
    if ((fp==NULL)||(fp->dim!=1)||(strcmp(name[loop1],fp->dname1)!=0)) {
      fprintf(stderr,"cdfwindow:  %s isn't an index variable\n",name[loop1]);
      exit(1);
    }
    dsold[loop1]=fp->dsize1;
    old9[loop1]=sbuff[fp->ptr+dsold[loop1]-1];
    old0[loop1]=sbuff[fp->ptr];

/* get indices nearest cut values */
    lotemp = 1.e30;
    hitemp = 1.e30;
    lo[loop1] = hi[loop1] = 0;
    for (loop2 = 0; loop2 < fp->dsize1; loop2++) {
      if (fabs(flo[loop1] - sbuff[fp->ptr + loop2]) < lotemp) {
        lotemp = fabs(flo[loop1] - sbuff[fp->ptr + loop2]);
        lo[loop1] = loop2;
      }
      if (fabs(fhi[loop1] - sbuff[fp->ptr + loop2]) < hitemp) {
        hitemp = fabs(fhi[loop1] - sbuff[fp->ptr + loop2]);
        hi[loop1] = loop2;
      }
    }

/* find values of dname, values of ninc and the size of the "old" index field */
    if ((seekpar(hbuffer,HBMAX,dname[loop1],pvalue))==FAIL)
      dvalue[loop1]=(old9[loop1]-old0[loop1])/((float)(dsold[loop1]-1));
    else
      dvalue[loop1]=atof(pvalue);

/* check to make sure that flo and fhi agree with direction of dvalue */
    flag1=0;
    if ((dvalue[loop1] < 0.0)&&(fhi[loop1] > flo[loop1])) flag1=1;
    else if ((dvalue[loop1] > 0.0)&&(fhi[loop1] < flo[loop1])) flag1=1;
    if (flag1==1) {
      temp=flo[loop1];
      flo[loop1]=fhi[loop1];
      fhi[loop1]=temp;
    }

/* determine which way to window (with(1) or without(-1) padding) */
    flag2=0;
    if (dvalue[loop1] < 0.0) {
      if ((flo[loop1] <= old0[loop1])&&(fhi[loop1] <= old0[loop1])) flag2++;
      if ((flo[loop1] >= old9[loop1])&&(fhi[loop1] >= old9[loop1])) flag2++;
      if ((flo[loop1] <= old9[loop1])||(fhi[loop1] >= old0[loop1])) flag2 = 9;
    }
    else if (dvalue[loop1] > 0.0) {
      if ((flo[loop1] >= old0[loop1])&&(fhi[loop1] >= old0[loop1])) flag2++;
      if ((flo[loop1] <= old9[loop1])&&(fhi[loop1] <= old9[loop1])) flag2++;
      if ((flo[loop1] >= old9[loop1])||(fhi[loop1] <= old0[loop1])) flag2 = 9;
    }
/* catch nonsensical window limits that miss the field completely */
    if (flag2 == 9) {
      fprintf(stderr,"cdfwindow: limits don't include %s.\n",name[loop1]);
      exit(1);
    } else {
      if (flag2==2) { 
	way[loop1] = -1;
      } else { 
	way[loop1] = 1;
      }
    }

/* setup extrapolated index fields, preserving orginal data where possible */
    if (way[loop1] == 1) {
/* find value and index limits of lower part of field */
      dif = 1.e30;
      loop2 = 0;
      locnt[loop1] = 0;
      index[loop1] = 0;
      extflag[loop1] = 0;
      newdelta = dvalue[loop1];
      while (dif > fabs(newdelta)/2.) {
/* flo is below old0 -- extrapolate downwards */
	if (((dvalue[loop1] > 0.0) && (flo[loop1] < old0[loop1])) ||
	    ((dvalue[loop1] < 0.0) && (flo[loop1] > old0[loop1]))) {
	  if (loop2 > 0) {
	    locnt[loop1]++;
	    index[loop1]--;
	    extflag[loop1] = 1;
	  }
	  newflo = old0[loop1] - dvalue[loop1]*loop2++;
	} else { 
/* flo is above old0 -- find nearest existing point */
	  if (loop2 > 0) {
	    locnt[loop1]++;
	    index[loop1]++;
/* we want the nearest delta value, particularly in unregular fields */
	    if (loop2 < dsold[loop1]) {
	      newdelta = sbuff[fp->ptr + loop2] - sbuff[fp->ptr + loop2 - 1];
	    }
	  }
	  newflo = sbuff[fp->ptr + loop2++];
	}
	if (fabs(flo[loop1] - newflo) < dif) {
	  dif = fabs(flo[loop1] - newflo);
	}
      }
      
/* find value and index limits of upper part of field */
      dif = 1.e30;
      loop2 = 0;
      hicnt[loop1] = 0;
      newdelta = dvalue[loop1];
      while (dif > fabs(newdelta)/2.) {
/* fhi is below old9 -- find nearest existing point */
	if (((dvalue[loop1] > 0.0) && (fhi[loop1] <= old9[loop1])) ||
	    ((dvalue[loop1] < 0.0) && (fhi[loop1] >= old9[loop1]))) {
	  if (loop2 < 0) {
	    hicnt[loop1]++;
	    if (loop2 > -dsold[loop1]) {
	      newdelta = sbuff[fp->ptr + (dsold[loop1] - 1) + loop2] -
		sbuff[fp->ptr + (dsold[loop1] - 1) +  loop2 - 1];
	    } 
	  }
	  newfhi = sbuff[fp->ptr + (dsold[loop1] - 1) + loop2--];
	} else {
/* fhi is above old9 -- extrapolate upwards */
	  if (loop2 < 0) {
	    hicnt[loop1]++;
	    if ((locnt[loop1] > 0) && (index[loop1] < 0)) {
	      extflag[loop1] = 3;
	    } else { 
	      extflag[loop1] = 2;
	    }
	  }
	  newfhi = old9[loop1] - dvalue[loop1]*loop2--;
	}
	if (fabs(fhi[loop1] - newfhi) < dif) {
	  dif = fabs(fhi[loop1] - newfhi);
	}
      }
      
/* allocate memory for temporary new field arrays */
      if (extflag[loop1] == 1) {
	ninc[loop1] = locnt[loop1] + (dsold[loop1] - hicnt[loop1]);
      } else if (extflag[loop1] == 2) {
	ninc[loop1] = hicnt[loop1] + (dsold[loop1] - locnt[loop1]);
      } else if (extflag[loop1] == 3) {
	ninc[loop1] = locnt[loop1] + dsold[loop1] + hicnt[loop1];
      } else {
/* no extrap needed - skip (flo or fhi are within dvalue/2 of old0, old9) */
	way[loop1] = -1;
	ninc[loop1] = abs(hi[loop1] - lo[loop1]) + 1;
	continue;
      }
      tempfld[loop1] = getbuff(ninc[loop1]);
      
/* construct new index field -- piece-by-piece */
      loop = 0;
/* assign from low extrapolated point to just below old0 */
      if (((extflag[loop1] == 1) || (extflag[loop1] == 3)) &&
	  (locnt[loop1] > 0)) {
	loop2 = locnt[loop1];
	while (loop2 > 0) {
	  newflo = old0[loop1] - dvalue[loop1]*loop2--;
	  tempfld[loop1][loop++] = newflo;
	}
/* assign from point nearest to flo in old field (>= old0) up to old9 */
      } else {
	loop2 = locnt[loop1];
	while (loop2 < dsold[loop1]) {
	  newflo = sbuff[fp->ptr + loop2++];
	  tempfld[loop1][loop++] = newflo;
	}
      }
/* copy entire original field to new field, if needed */
      if (extflag[loop1] == 3) {
	loop2 = 0;
	while (loop2 < dsold[loop1]) {
	  tempfld[loop1][loop++] = sbuff[fp->ptr + loop2++];
	}
      }
/* assign from old0 up to point nearest to fhi in old field */
      if (((extflag[loop1] != 2) && (extflag[loop1] != 3)) ||
	  (hicnt[loop1] == dsold[loop1])) {
	loop2 = 0;
	while (loop2 < (dsold[loop1] - hicnt[loop1])) {
	  newfhi = sbuff[fp->ptr + loop2++];
	  tempfld[loop1][loop++] = newfhi;
	}
      }
/* assign from old9 up to extrapolated high point */
      if (((extflag[loop1] == 2) || (extflag[loop1] == 3)) &&
	  (hicnt[loop1] > 0)) {
	loop2 = 1;
	while (loop2 <= hicnt[loop1]) {
	  newfhi = old9[loop1] + dvalue[loop1]*loop2++;
	  tempfld[loop1][loop++] = newfhi;
	}
      }
    }

/* set number of elements in index field to be extracted (w/o padding) */
    if (way[loop1] != 1) {
      ninc[loop1] = abs(hi[loop1] - lo[loop1]) + 1;
    }
  }

/* get value for bad */
  if ((seekpar(hbuffer,HBMAX,"bad",pvalue))==FAIL)
    bad=BAD;
  else
    bad=atof(pvalue);

/* make sure the fields are always increasing (or decreasing) */
  for (loop=0;loop<numsets;loop++) {
    fp=seekfld(hbuffer,HBMAX,'s',name[loop]);
    for (loop1=1;loop1<(fp->dsize1);loop1++) {
      if (loop1==1) {
        chek1=sbuff[fp->ptr+loop1]-sbuff[fp->ptr+loop1-1];
        chek1=chek1/fabs(chek1);
      }
      else if (loop1 > 1) {
        chek2=sbuff[fp->ptr+loop1]-sbuff[fp->ptr+loop1-1];
        if ((chek2*chek1)<=0) {
          fprintf(stderr,"cdfwindow: index field is not monotonic\n");
          exit(1);
        }
      }
    }
  }

/* find the reduction values for windowing (if no padding required) for high: */
  for (loop=0;loop<numsets;loop++) {
    high[loop]=low[loop]=0;
    dif = 1.e30;
    fp=seekfld(hbuffer,HBMAX,'s',name[loop]);
    for (loop1=0;loop1<fp->dsize1;loop1++) {
      if (fabs(fhi[loop]-sbuff[fp->ptr+loop1]) < dif) {
        dif = fabs(fhi[loop]-sbuff[fp->ptr+loop1]);
        high[loop]=loop1;
      }
    }
    ++high[loop];

/* for low: */
    dif = 1.e30;
    for (loop1=0;loop1<fp->dsize1;loop1++) {
      if (fabs(flo[loop]-sbuff[fp->ptr+loop1]) < dif) {
        dif = fabs(flo[loop]-sbuff[fp->ptr+loop1]);
        low[loop]=loop1;
      }
    }
  }

/* make new header and transfer comments */
  nullhdr(hbufferw,HBMAX,"float");
  copycmt(hbufferw,HBMAX,hbuffer,HBMAX);

/* check parameters for updating */
  for (loop1=0;loop1<MA;loop1++)
    par[loop1] = -1;
  loop1=0;
  while ((numrd=getpar(hbuffer,HBMAX,loop1++,pname,pvalue)) != FAIL) {
    for (loop2=0;loop2<numsets;loop2++) {
      if (strcmp(pname,pnamec[loop2]) == 0) {
        par[loop2]=0;
        sprintf(pvalue,"%g",flo[loop2]);
      }
    }
    addpar(hbufferw,HBMAX,pname,pvalue);
  }

/* check if there is a parameter to create (par = -1) */
  for (loop1=0;loop1<numsets;loop1++) {
    if (par[loop1] == -1) {
      sprintf(pvalue,"%g",flo[loop1]);
      addpar(hbufferw,HBMAX,pnamec[loop1],pvalue);
    }
  }

/* transfer static & variable fields making updates */
/* loop = 0 transfer static field, loop = 1 transfer variable field */
  for (loop=0;loop<2;loop++) {
    if (loop==0)
      field = 's';
    else
      field = 'v';
    loop1=0;
    while ((fp=getfld(hbuffer,HBMAX,field,loop1++)) != NULL) {

/* find which (if any) of the dimensions to update */
      for (loop2=0;loop2<numsets;loop2++) {
        if (strcmp(fp->dname1,name[loop2])==0)
          fp->dsize1 = ninc[loop2];
        else if (strcmp(fp->dname2,name[loop2])==0)
          fp->dsize2 = ninc[loop2];
        else if (strcmp(fp->dname3,name[loop2])==0)
          fp->dsize3 = ninc[loop2];
        else if (strcmp(fp->dname4,name[loop2])==0)
          fp->dsize4 = ninc[loop2];
      }

/* send updated field (static or variable) to buffer */
      addfld(hbufferw,HBMAX,field,fp->fname,fp->smul,fp->sadd,fp->bytes,
          fp->dim,fp->dname1,fp->dsize1,fp->dname2,fp->dsize2,fp->dname3,
          fp->dsize3,fp->dname4,fp->dsize4);
    }
  }

/* make comment lines */
  strcpy(cl,"cdfwindow:\n");
  addcline(hbufferw,HBMAX,cl);
  for (loop=0;loop<numsets;loop++) {
    sprintf(cl,"  %s %g %g\n",name[loop],flo[loop],fhi[loop]);
    addcline(hbufferw,HBMAX,cl);
  }

/* new header is updated, send to stdout */
  puthdr(stdout,hbufferw,HBMAX);

/* set aside memory for out slice */
  nelw=elemcnt(hbufferw,HBMAX,'s');
  sbuffout=getbuff(nelw);

/* transfer the new index fields if padding is required */
  for (loop=0;loop<numsets;loop++) {
    if (way[loop] == 1) {
      dout=getptr(hbufferw,HBMAX,sbuffout,'s','d',name[loop]);
      for (loop1=0;loop1<ninc[loop];loop1++)
        dout[loop1] = (float) (tempfld[loop][loop1]);
/* free temporary buffer for new index field */  
      free(tempfld[loop]);
    }
  }

/* begin updating and sending to output buffer (sbuffout) */
  field='s';
  flag2=0;
  while (status != EOF) {
    loop2 = 0;
    while ((fp = getfld(hbuffer,HBMAX,field,loop2++)) != NULL) {
      din=getptr(hbuffer,HBMAX,sbuff,field,'d',fp->fname);
      dout=getptr(hbufferw,HBMAX,sbuffout,field,'d',fp->fname);

/* if a scalar then transfer and continue */
      if (fp->dim == 0) {
        *dout = *din;
        continue;
      }

/* see if its one of the index fields thats already been updated */
      flag1=0;
      if (strcmp(fp->dname1,fp->fname) == 0) {
        for (loop=0;loop<numsets;loop++)
          if ((strcmp(fp->fname,name[loop]) == 0)&&(way[loop] == 1)) flag1=1;
      }
      if (flag1 == 1) continue;

/* get the ranges to loop over */
      start[0]=start[1]=start[2]=start[3]=0;
      size[0]=size[1]=size[2]=size[3]=stop[0]=stop[1]=stop[2]=stop[3]=1;
      if (fp->dim >=1) size[0]=stop[0]=fp->dsize1;
      if (fp->dim >=2) size[1]=stop[1]=fp->dsize2;
      if (fp->dim >=3) size[2]=stop[2]=fp->dsize3;
      if (fp->dim >=4) size[3]=stop[3]=fp->dsize4;
      ds1=ds2=ds3=ds4=1;
/* 1st dim ... */
      for (loop=0;loop<numsets;loop++) {
        if (fp->dim >= 1) {
          ds1=fp->dsize1;
          if ((strcmp(fp->dname1,name[loop])==0)&&(way[loop]==1)) {
            start[0]=index[loop];
            stop[0]=index[loop]+ninc[loop];
            size[0]=ninc[loop];
          }
          else if ((strcmp(fp->dname1,name[loop])==0)&&(way[loop]== -1)) {
            stop[0]=high[loop];
            start[0]=low[loop];
            size[0]=ninc[loop];
          }
        }

/* 2nd dim */
        if (fp->dim >= 2) {
          ds2=fp->dsize2;
          if ((strcmp(fp->dname2,name[loop])==0)&&(way[loop]==1)) {
            start[1]=index[loop];
            stop[1]=index[loop]+ninc[loop];
            size[1]=ninc[loop];
          }
          else if ((strcmp(fp->dname2,name[loop])==0)&&(way[loop]== -1)) {
            stop[1]=high[loop];
            start[1]=low[loop];
            size[1]=ninc[loop];
          }
        } 

  /* 3rd dim */
        if (fp->dim >= 3) {
          ds3=fp->dsize3;
          if ((strcmp(fp->dname3,name[loop])==0)&&(way[loop]==1)) {
            start[2]=index[loop];
            stop[2]=index[loop]+ninc[loop];
            size[2]=ninc[loop];
          }
          else if ((strcmp(fp->dname3,name[loop])==0)&&(way[loop]== -1)) {
            stop[2]=high[loop];
            start[2]=low[loop];
            size[2]=ninc[loop];
          }
        }

/* 4th dim */
        if (fp->dim >= 4) {
          ds4=fp->dsize4;
          if ((strcmp(fp->dname4,name[loop])==0)&&(way[loop]==1)) {
            start[3]=index[loop];
            stop[3]=index[loop]+ninc[loop];
            size[3]=ninc[loop];
          }
          else if ((strcmp(fp->dname4,name[loop])==0)&&(way[loop]== -1)) {
            stop[3]=high[loop];
            start[3]=low[loop];
            size[3]=ninc[loop];
          }
        }
      }
 
/* do the loops and move the data */
      for (x=start[0],xp=0;x<stop[0];x++,xp++) {
        for (y=start[1],yp=0;y<stop[1];y++,yp++) {
          for (z=start[2],zp=0;z<stop[2];z++,zp++) {
            for (t=start[3],tp=0;t<stop[3];t++,tp++) {
              s2=tp+zp*size[3]+yp*size[3]*size[2]+xp*size[3]*size[2]*size[1];
              dout[s2]=bad;
              if ((numrd=inbox(0,ds1,0,ds2,0,ds3,0,ds4,x,y,z,t))==OK) {
                s1=t+z*ds4+y*ds4*ds3+x*ds4*ds3*ds2;
                dout[s2]=din[s1];
              }
            }
          }
        }
      }
    }

/* send static slice to output */
    putslice(stdout,nelw,sbuffout);
  
/* free static buffers and set aside memory for variable buffers */
    if (flag2 == 0) {
      ++flag2;
      free(sbuff);
      free(sbuffout);
      nel=elemcnt(hbuffer,HBMAX,'v');
      nelw=elemcnt(hbufferw,HBMAX,'v');
      sbuff=getbuff(nel);
      sbuffout=getbuff(nelw);
      field='v';
    }
    status=getslice(stdin,nel,sbuff);
  }
  exit(0);
}

/* is the point (x,y,z,t) in the box defined by xl,xh yl,yh zl,zh tl,th ? */
static int inbox(xl,xh,yl,yh,zl,zh,tl,th,x,y,z,t)
int xl,xh,yl,yh,zl,zh,tl,th,x,y,z,t;
{
  int flag;

  flag=0;
  if (x < 0) --flag;
  if (y < 0) --flag;
  if (z < 0) --flag;
  if (t < 0) --flag;
  if ((xl <= x)&&(xh > x)) ++flag;
  if ((yl <= y)&&(yh > y)) ++flag;
  if ((zl <= z)&&(zh > z)) ++flag;
  if ((tl <= t)&&(th > t)) ++flag;
  if (flag == 4) return(OK);
  else return(FAIL);
}
