/* thermo.h -- declarations for thermodynamics package */
/* $Id: thermo.h,v 1.2 1992/02/07 03:03:23 raymond Exp raymond $ */

/* constants (Iribarne and Godson) */
#define CTOK 273.15               /* Celsius to Kelvin conversion */
#define RD 287.05                 /* dry air gas constant */
#define EPS 0.622                 /* vapor to dry air mol weight ratio */
#define RV (RD/EPS)               /* water vapor gas constant */
#define CPD 1005.                 /* specific heat const pres dry air */
#define CPV 1850.                 /* specific heat const pres water vapor */
#define CL 4218.                  /* specific heat liquid water */
#define CIMEAN 1959.              /* specific heat of ice at -20 C */
#define LR 2.5008e6               /* heat of vaporization at 0 C */
#define LFR 3.337e5               /* heat of fusion at 0 C */
#define DLFDT 2.45e3              /* slope of heat of fusion curve */
#define TR 273.15                 /* reference temperature (K) */
#define PR 100000.                /* reference pressure (Pa) */
#define TMIN 190.                 /* minimum reasonable atmospheric temp */
#define GEE 9.807                 /* acceleration of gravity */
#define DELTA 0.606               /* virtual temperature factor */

/* function declarations */
float es();                       /* saturation vapor pressure */
float ent();                      /* entropy */
void invent();                    /* invert entropy equation */
float mratio();                   /* mixing ratio */
