/***************************************************************************/
/* The next section consists of configuration-dependent flags which        */
/* should be set appropriately when Candis is being installed on a         */
/* new system.                                                             */
/***************************************************************************/

/* Set ANSI to 1 if the C compiler is ANSI standard, and in particular,
 * if it needs the include file stdlib.h.
 */

#define ANSI 1

#if ANSI
#include <stdlib.h>
#endif

/* Set VAX = 1 for VAXes and other machines with similar byte ordering
 * for integers.  Motorola 68000 machines should have VAX set
 * to 0.  This is only important
 * for programs that convert non-cdf files containing integers
 * in binary format to cdf files.  Examples are radcedric
 * and rafread.  (Intel machines and DECstation 3100s are like VAXes.)
 */

#define VAX 1

/* Set FLOATSWAP = 1 if you wish to make getslice and putslice swap the
 * order of bytes in floats.  This is for Intel-X86 computers, which
 * have IEEE format, but have the bytes swapped relative to BIG ENDIAN
 * machines like Suns.  There is a slight performance penalty, but it
 * means translations to ascii format and back don't have to be done
 * between Suns and PCs.  The convection is to set FLOATSWAP = 1 for
 * PCs and to 0 for Suns and similar machines.
 */

#define FLOATSWAP 1

/* Set BSD = 1 for Berkeley-derived UNIX systems.  Set LINUX = 1 for
 * POSIX systems such as Linux.  Set both to zero for System V.
 * This flag is important only for serial input/output programs such
 * as zin.  SunOS-3.X should have BSD = 1.  SunOS-4.X can have either
 * 1 or 0, but 0 works more efficiently.  ULTRIX should probably have
 * 1, while HPUX should probably have 0.  (Don't turn on both BSD and
 * LINUX at the same time!)
 */

#define BSD 0
#define LINUX 1

/* When converting floats to ints, most computers truncate negative floats
 * toward zero.  For such machines set ROUNDTOZERO = 1.  Otherwise set
 * ROUNDTOZERO = 0.
 */

#define ROUNDTOZERO 1

/***************************************************************************/
/* Generally speaking, don't mess with stuff below this line.              */
/***************************************************************************/

/* define failure and success for getpar and seekpar */
#define OK 0
#define FAIL -1
 
/* line buffer length */
#define LINE 202

/* end of header character */
#define ENDCHAR '*'

/* bad data and bad data limit defaults */
#define BAD 1.e30
#define BADLIM 0.999e30

/* define maximum number of lines in a header buffer
 * This parameter is here so that standard utilities
 * can allocate reasonably sized header buffers with
 * the expectation that all common data format files
 * will conform.
 */
#define HBMAX 1000

/* define the maximum possible number of slice elements
 * Note: Don't define slice buffers this big -- dynamically
 * allocate them with malloc instead!  The only reason for
 * this parameter is to make possible a phase error check
 * when reading common data format files.
 *
 * This can be as big as 2147483647 for 32 bit signed longs.
 */
#define MAXSLICE 1000000000
 
/* logical condition for reaching end of header buffer */
#define ETEST(x) (buffer[x][0] == ENDCHAR)&&\
  ((buffer[x][1] == '\n')||(buffer[x][1] == ' '))
 
/* sub-header definitions */
#define CSH "***comments***"
#define PSH "***parameters***"
#define SSH "***static_fields***"
#define VSH "***variable_fields***"
#define FSH "***format***"
 
/* structure defining field specification */
struct field {
  char fname[LINE];  /* field name */
  float smul,sadd;   /* scaling parameters for integer packing */
  char bytes;        /* precision of integers in packed format --
                        c = 1 byte (char), s = 2 bytes (short),
                        l = 4 bytes (long), p = pixel (no packing) */
  int dim;           /* number of dimensions to field (0, 1, 2, 3, or 4) */
  char dname1[LINE],dname2[LINE],dname3[LINE],dname4[LINE];
                     /* names of dimensions */
  int dsize1,dsize2,dsize3,dsize4;  /* sizes of dimensions */
  long tsize;        /* total size of field */
  long ptr;          /* location pointer in data slice -- gives
                        offset (in elements) from beginning of slice */
};

/* function type declarations */
long elemcnt(),getelcnt(),getslice(),putslice();
long gislice(),pislice(),gaslice(),paslice();
struct field *getfld(),*seekfld();
char getfmt();
float *getbuff(),*getptr(),*getptr2();
