/* mcidas.h -- This file defines common things for decoding pc-mcidas
 * data streams.
 */

/* set VERBOSE if you want satellite header information printed to
   standard output for every image */
#define VERBOSE 1

/* maximum length of data (current mcidas upper limit is 768 - 5;
   let's be conservative) */
#define DATAMAX 770

/* we accept only one thread (0-3) as defined below */
#define THREAD 0

/* number of retries allowed on finding sync */
#define TRYMAX 2000

/* max number of images allowed for looping */
#define MAXIMAGES 60

/* structure containing packet information */
struct mcpacket {
  unsigned char fxub;            /* packet header */
  int thread;                    /* thread code (0-3) */
  unsigned char r_code;          /* routing code */
  unsigned char length1;         /* first length byte */
  unsigned char length2;         /* second length byte */
  int length;                    /* computed length in bytes of data */
  unsigned char data[DATAMAX + 1];   /* data from packet (leave room for
                                        null termination) */
  char marker;                   /* data marker */
};

/* structure containing GOES and GVAR satellite image header data */
struct goes {
  long satid;                    /* satellite id number */
  int year;                      /* year in form 19** */
  int day;                       /* day of year */
  int hour;                      /* hour (24 hr clock) GMT */
  int minute;                    /* minute */
  int second;                    /* second */
  long ycoor;                    /* upper left line in satellite coords */
  long xcoor;                    /* upper left elem in satellite coords */
  long zcoor;                    /* upper left hand z coordinate */
  long ysize;                    /* number of lines in area */
  long xsize;                    /* number of elements per line */
  long nbyte;                    /* number of bytes per data element */
  long yres;                     /* line resolution */
  long xres;                     /* element resolution */
  long zres;                     /* number of bands */
  long bandmap;                  /* bit vector -- bit = 1 if band present */
  long dprelen;                  /* length of data prefix */
  long dataoset;                 /* byte offset to data block */
  long navoset;                  /* byte offset to nav block */
  long validity;                 /* validity code for data prefix */
  char source[5];                /* source type from area directory */
  char calib[5];                 /* calibration type */
  char navtype[5];               /* must be "GOES" or "GVAR" */

  /* GOES (some GVAR) */
  long epochd;                   /* epoch date (yymmdd) */
  long epocht;                   /* epoch time (hhmmss)  */
  float semima;                  /* semimajor axis of orbit (km) */
  float eccen;                   /* orbital eccentricity */
  float orbinc;                  /* orbital inclination (deg) */
  float meana;                   /* mean anomaly (deg) */
  float perigee;                 /* argument of perigee (deg) */
  float asnode;                  /* right ascension of ascending node (deg) */
  float declin;                  /* declination of satellite axis (deg) */
  float rascen;                  /* right ascension of satellite axis (deg) */
  long piclin;                   /* picture center line number */
  float spinp;                   /* spin period */
  float deglin;                  /* total sweep angle, line direction (deg) */
  long lintot;                   /* number of scan lines (all scanners) */
  long scanners;                 /* number of scanners */
  float degele;                  /* total sweep angle, element direction
                                     (deg) */
  long eletot;                   /* number of elements in a scan line */
  float pitch;                   /* forward leaning angle (deg) */
  float yaw;                     /* sideways leaning angle (deg) */
  float roll;                    /* rotation (deg) */
  long iajust;                   /* e-w adjustment value */
  double iajtim;                 /* adjustment time (sec) */
  float iseang;                  /* angle betwee vissr and sun sensor (deg) */
  long scanline1;                /* scan line 1 (betas) */
  double timesl1;                /* time of scan line 1 after 0 Z (sec) */
  long betacount1;               /* huh? */
  long scanline2;                /* scan line 2 */
  double timesl2;                /* time of scan line 2 (sec) */
  long betacount2;               /* yup! */
  float gamma;                   /* element offset */
  float gammadot;                /* element drift per hour */

  /* additional things needed for GVAR */
  float lam;
  float dr;
  float phi;
  float psi;
  float rma;
  float pma;
  float elevbnds;
  float elevdln;
  float scanbnds;
  float scanpix;
  float lineoffset;
  float scanoffset;
};
