// 
// util -- Implements general purpose utility functions that
//         are used by both  
//         the CongugateGradient  Minimization Algorithm
//         and the initialization routines as well as
//         the evaluation of the target function.
//

#include "ng3dvar.h"
//#include <stdlib.h>
//#include <stdio.h>

double *Dgetbuff(long nelem);
float *Fgetbuff(long nelem);
int *Igetbuff(long nelem);

int gp(int Ny, int Nz, int i, int j, int k);
int gpe(int Nx, int Ny, int Nz, int c, int i, int j, int k);

FILE *OpenFileR(char *FileName);
FILE *OpenFileW(char *FileName);

void stop();
void fun_Stop(char *s) ;
double FieldNorm2(int N, double *fld);
int CheckVector(int N,double *V);

//
//--------------------------------------------------------------------
//
void fun_Stop(char *s) {
//
// Used by functions to stop proccesing and send a final message.
//
   fprintf(stderr,"%s\n",s);
   exit(1);
}
//
//--------------------------------------------------------------------
//
void stop(){
//
// Just a marker to use in conjuntion with gdb 
//
  fprintf(stderr,"Mark\n");
}
//
//--------------------------------------------------------------------
//
double FieldNorm2(int N, double *fld){
//
// Get the norm of fld, which is size N.
//
double sum = 0;
int i;
  for (i = 0; i< N; i++) { sum += (fld[i] * fld[i]);}

	return(sum);
}
//
//--------------------------------------------------------------------
//
FILE *OpenFileR(char *FileName){
FILE *fh;
  if ((fh = fopen(FileName,"r")) == NULL) {
    fprintf(stderr,"OpenFileR: can't open %s\n",FileName);
    exit(1);
  }
  return(fh);
}
//
//--------------------------------------------------------------------
//
FILE *OpenFileW(char *FileName){
FILE *fh;
  if ((fh = fopen(FileName,"w")) == NULL) {
    fprintf(stderr,"OpenFileW: can't open %s\n",FileName);
    exit(1);
  }
  return(fh);
}
//
//-------------------------------------------------------------------
//
double *Dgetbuff(long nelem){
// 
// Dgetbuff -- This routine uses malloc to allocate buffer
// space of doubles.  The argument nelem defines the number
// of floating point elements required by the user.  
// It returns a double pointer to a block slightly larger than
// requested. HopeFully this precaution takes care of the 
// off-by-one bugs -- see malloc documentation.
// If malloc fails, Dgetbuff dies with an error message.
//
int i;
double *point,sum;
  if ((point = (double*)malloc(sizeof(double)*(nelem+2))) == NULL) {
     fprintf(stderr,"Dgetbuff: cannot get enough memory\n");
     exit(1);
  } 
	//Check the chunk
	sum = 0.0;
	for (i = 0; i< nelem; i++) { point[i]= 0.0;sum += (point[i] * point[i]);}
	if (sum != 0) {
     fprintf(stderr,"Dgetbuff: Got a Bad Chunk of memory\n");
     exit(1);
  }
  return(point);
}
//
//-------------------------------------------------------------------
//
float *Fgetbuff(long nelem){
// 
// replaces candis getbuff, because we need to have the possibility
// to recompile using the garbache collector.
//
// Fgetbuff -- This routine uses malloc to allocate buffer
// space of floats.  The argument nelem defines the number
// of floating point elements required by the user.  
// It returns a float pointer to a block slightly larger than
// requested. HopeFully this precaution takes care of the 
// off-by-one bugs -- see malloc documentation.
// If malloc fails, Fgetbuff dies with an error message.
//
int i;
float *point,sum;
  if ((point = (float*)malloc(sizeof(float)*(nelem+2))) == NULL) {
     fprintf(stderr,"Fgetbuff: cannot get enough memory\n");
     exit(1);
  } 
	//Check the chunk
	sum = 0.0;
	for (i = 0; i< nelem; i++) { point[i]= 0.0;sum += (point[i] * point[i]);}
	if (sum != 0) {
     fprintf(stderr,"Dgetbuff: Got a Bad Chunk of memory\n");
     exit(1);
  }
  return(point);
}
//
//-------------------------------------------------------------------
//
int *Igetbuff(long nelem){
// 
// Igetbuff -- This routine uses malloc to allocate buffer
// space of floats. 
//
int *point,i,sum;
  if ((point = (int*)malloc(sizeof(int)*(nelem+2))) == NULL) {
     fprintf(stderr,"Igetbuff: cannot get enough memory\n");
     exit(1);
  } 
	//Check the chunk
	sum = 0.0;
	for (i = 0; i< nelem; i++) { point[i]= 0;sum += (point[i] * point[i]);}
	if (sum != 0) {
     fprintf(stderr,"Dgetbuff: Got a Bad Chunk of memory\n");
     exit(1);
  }
  return(point);
}
//
//-------------------------------------------------------------------
//
int CheckVector(int N,double *V) {
int i;
int flag = 1;
  for (i=0; i< N; i++){
    //     if ( __finite (V[i]) == 0) { 
     if (V[i] == 0) { 
        flag = -1; 
				fprintf(stderr,"V[%i] = %g |",i,V[i]);
	   }
  }
	if (flag > 0) fprintf(stderr,"Vector is fine\n");
	return(flag);
}
//-------------------------------------------------------------------
//
// Access to storage in RowMajor order.
// -- Macros are generally less safe than functions.
// 
// i,j,k are the indexes of the grid point(numbering starts from zero).
// c     -- component number [0,1,2] 
//
// --
//
int gp(int Ny, int Nz, int i, int j, int k){
// use this for scalars defined on a 3D grid
 return(k + Nz * ( j + Ny * i ));
}
//
// --
//
int gpe(int Nx, int Ny, int Nz, int c, int i, int j, int k){
// use this for Vectors defined on a 3D grid. where c is the component
// c = 0 reduces to an scalar field case, but you waste two extra ops.
  return( k + Nz * (j + Ny * (i + Nx * c)));
}
