// 
// cgm -- Conjugate Gradient Minimization 
//
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
//#include "/data.m2/clopez/Packages/GC/include/leak_detector.h"

#define MAXCHAR 200
#define MAXLINES 500

#define D2R 0.017453  /* degrees to radians conversion */
#define R2D 57.296    /* radians to degrees conversion */
#define EARTH_R 6370.0
#define EARTH_D 12740.0

//
// Access to storage in RowMajor order.
//
//#define gp(i,j,k) (k) + ( (j) + (i) * Nyg) * Nzg
//#define Gp(vc,i,j,k) gp((i),(j),(k)) + (Nxg * Nyg * Nzg) * (vc)
// the above macros have been repaced by functions gp and gpe defined
// in mgm_util.c

//
#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))

//
// Safe deallocation of memory previosly gather using malloc.
//
#define Dfreebuff(p) if(p) { free(p); (p)=NULL; } 

//
//-------------------------------------------------------------
//
struct clpInfo { // command line parameters Info

// iff --indicates if there is an initialization file for the field.
// tfe --total function evaluations.

  int argc;
  char **argv;
  int bff; 
  char  bgf_fname[MAXCHAR]; // name of file with backgraundfield (if any)
  double pmc,pxc,phs,pvs,pbg;  // penalty factors;
  int cg_nfe;             //max Num of function evaluations allow to CG.
  int    ol_num;          // Number of Outer Iterations
  double ol_p0;           //initial penalty for mass continuity term.
  double ol_gr;           // Penalty Growth Rate for mass continuity term
  int    ol_out; //[-1|1]=[No|Yes]to output AtEndOf each outer iteration
  int    ini_out;// [-1|1] = [No|Yes] to output after initial solution 
  int    sca;    //scaling of penalty term [-1(default) = off; 1 = on];

};
//
//-------------------------------------------------------------
//
struct RadarDataInfo { //Package of Radar Information
//
char *fn;              //file name
char  hbin[MAXCHAR][MAXLINES]; //Header Buffer
long  nsbin;                   //Static Buffer size
long  nvbin;                   //Variables Buffer size
float *sbin;                   //Static Buffer
float *vbin;                   //Static Buffer
float bad,badlim;              // actual values for file.
float lon0,lat0,alt0;          // Grid Origin
int Nx,Ny,Nz;                  // Grid Size
float Dx,Dy,Dz;                //Grid Cell Size
float x0,y0,z0;                //
float *xg,*yg,*zg;                //Grid Axis
float *u1, *u2,  *u3;          // eigen velocities
float *a1, *a2,  *a3;          // eigen values
float *e1x,*e1y, *e1z;         // eigen vector 1
float *e2x,*e2y, *e2z;         // eigen vector 2
float *e3x,*e3y, *e3z;         // eigen vector 3
float *wpTV;                   // Vertical particle terminal velocity
float *dbz;                    // reflectivity factor
float xxd,yyd,zzd;             // Distances between Analysis and Data grids.
int Nobs;                      // Number of good Observations in the grid.
int sc;                        // spherical coordinates flag indicator
char xname[MAXCHAR];
char yname[MAXCHAR];
char zname[MAXCHAR];
};
//
//-------------------------------------------------------------
//
//-------------------------------------------------------------
//
struct BGFdata { //Background Field data
//
//This structure is for holding the Background Field 
//Currently just the horizontal velocity field at each point of the 
//analysis grid. 
//
float *vx, *vy, *vz;    // velocities (m/s,m/s,m/s)
float bad,badlim;       // in model file
};
//
//-------------------------------------------------------------
//
//-------------------------------------------------------------
//
struct gridInfo {
//
// Nc -- NumberOfComponents for the target field(1D=scalar 3D-vec isthemax;
// Nd -- Number of Grid Dimensions
// Nx,Ny,Nz -- grid dimensions for grid level
// Dx,Dy,Dz -- grid steps for grid level
// Np       -- Total number of point in the grid;
// Ne       -- Total number of elements defined on the grid.
// sc       -- spherical coordinates indicatero flag.
//
  int sc,Nc,Nd;
  int Nx,Ny,Nz,Np,Ne;
  float Dx,Dy,Dz;
	float x0,y0,z0;
};
//
//-------------------------------------------------------------
//
struct TFT {
  double PBG; // flag to add background field term to target function
  double PBC; // flag to add Boundary conditions term to target function
  double PMC; // flag to add Mass continuity term to target function
  double PHS; // flag to horizontal smoothing term to target function
  double PVS; // flag to vertical smoothing term to target function
  double PRA; // flag to add radar data term to target function
  double PXC; // flag to add extra constrain
	double BGsigma; //weight for the misfit between analyzed and BG velocities
};
//
//-------------------------------------------------------------
//
struct ConCri { // ConvergenceCriteria
  double g; // value of convergence criteria base on gradient.
  double x; // value of convergence criteria base on solution.
  double f; // value of convergence criteria base on targetfunction@sol.
  double c; // 1 if convergence is satisfied zero otherwise.
};
//
//-------------------------------------------------------------
//
struct cgInfo {
  double eps; // eps and acc Set the global accuracy parameters for conmin
  double acc; //  -- Nash & Soffer pg.375
  int MXFUN;  // MaxNumOfFunctionsEvaluations for CONMIN 
  double *WA; // working array for CONMIN
};
//
//-------------------------------------------------------------
//
struct funInfo {
//
//tfe --total function evaluations.
//
  struct RadarDataInfo rd;
	struct BGFdata       bgf;   //bgf data on analysis grid
  struct gridInfo grid;
  struct cgInfo cgi; // CongugateGradientInfo
  struct ConCri CC;
  struct TFT TFT0; // Holds initial selection of terms to be added to the
                   // target function.

  float lon0,lat0,alt0; // origin of analysis grid
  int NumVC;  // num of velocity components
  int mxn;    // Max Size of target field.
  int iff;    // flag --initial solution file
  int bff;    // flag --Background file.
	int bgf_ft;  // file type bringing the background field. 
  char *inif; // pointer initial solution data --file name.
  double PBG; // flag to add background field term to target function
  double PBC; // flag to add Boundary conditions term to target function
  double PMC; // flag to add Mass continuity term to target function
  double PVS; // flag to horizontal smoothing term to target function
  double PHS; // flag to vertical smoothing term to target function
  double PRA; // flag to add radar data term to target function
  double PXC; // flag to add extra constrain
  double PP;  // Penalty Parameter.
  
  int tfe;    // Total number of function evaluations.

  float *XG,*YG,*ZG; // auxiliar space to hold axis of analysis grid.

  double *rho;  //Holds Density profile on Analysis grid.
  double *rhoF; //HoldsDensityFactorOnAnalysisGrid(required for termVel).
  double *dbzi; //Holds dbz on the analysis grid (Interpolated).
  double *wtZi; //Holds dbz on the analysis grid (Interpolated).

  double *wtZ;  // Holds particle Terminal Velocity(from dbZ)

  double *AG;   // AuxiliarArrayFor evaluation of the function Gradient.
							 

  double mb;   // holds the last value of the mass balance
                // this value is the missfit calculated by 
                // function calcfg_mc

  int sc;               // flag set to 1 to indicate 
                        // Spherical Coordinates and to -1 for Cartesian.
   double dx,dy,dz; // grid steps cast as double and rads if required

//->   double y0; // origing cast to double and rads if required.

  
// double *hmfc;  // Holds HorizontalMassFluxConvergence
// double *vmf;  // Holds VerticalMassFlux
// struct mgmInfo *mgmi;

};
//-------------------------------------------------------------
struct DCS {
//detailed Convergence sequence
  double GSQ[100],XSQ[100],DXQ[100],TF[100],MC[100];
};
//-------------------------------------------------------------
