// 
// driver_clp -- Implements function to get Command Line Parameters
// 

#include "ng3dvar.h"
#include <unistd.h>
#define _GNU_SOURCE
#include <getopt.h>
#include <string.h>

#define USAGE "usage:ng3dvarX [-h] [--nfe=val] [--phs=val] [--pvs=val] [--pbg val:fname] [--pmc=p0:gr:num] [-s] [-o]"


struct clpInfo *fun_cgeta(int argc,char **argv);
void CheckCLP(struct clpInfo *clp);
void fun_usage();

//
// SetUp functions for this driver.
//
//-------------------------------------------------------------------
struct clpInfo *fun_cgeta(int argc,char **argv) {
/* getopt stuff */
extern char *optarg;
extern int optind;

int nonOptArgs = 0;
int l,k,j,c;

struct clpInfo *clp; // packs all required info

//Mandatory Arguments
float Lon0,Lat0,Alt0; //Origin og the analysis Grid
float Dx0,Dy0,Dz0;    // Steps on the analysis Grid
float Nx0,Ny0,Nz0;    // Sises of the analysis Grid
char rdx[MAXCHAR];    // Name of the Radar Data File.

// Optional Arguments
char initFile[MAXCHAR]; //Initial Solution File Name
char bgFile[MAXCHAR];   //Background Field File Name
double pmc,pxc,phs,pvs,pbg;
//int cg; //Conjugate Gradient Flag 
int cg_nfe; // Max Number of Function Evaluations used in Conjugate Gradient
int ol_num; // Number of Outer Iterations
double ol_p0; //initial weight on outer iteration
double ol_gr; // weight Growth Rate on outer iteration
int    ol_out; // [-1|1] = [No|Yes] to output AtEndOf each outer iteration
int    ini_out; // [-1|1] = [No|Yes] to output after initial solution

//Flags indicate when positive indicate availability of the associated file.
int radFlag  =  0; // radar file
int initFlag = -1; // Initial solution
int bgFlag   = -1; // Background Field

int scaling; // [-1(default) no scaling done; 1 scaling on]


  if (argc == 1) fun_Stop(USAGE);

  // 
  // Get space to hold the structures.
  //
  if((clp =(struct clpInfo*)malloc(sizeof(struct clpInfo)*(2)))==NULL){
     fprintf(stderr,"fun_cgeta: cannot get enough memory\n"); exit(1);}

  
  /*  set default value for optional arguments */
  scaling = -1;
	//cg = -1; 
  cg_nfe = -1;
  pmc = pxc = phs = pvs = pbg = -1.0;
  ol_p0 = ol_gr = -1;
  ol_num = ol_out = ini_out = -1;
  //initilaize strings.
	for (l=0;l<MAXCHAR;l++){rdx[l]=bgFile[l]=initFile[l]='\0';}

  /* Process Arguments */
  optind = 1;
  int option_index = 0;
  static struct option long_options[] = {
         {"pxc", 1, 0, 1},
         {"phs", 1, 0, 2},
         {"pvs", 1, 0, 3},
         {"nfe", 1, 0, 4},
         {"pmc", 1, 0, 5},
         {"pbg", 1, 0, 6},
         {0, 0, 0, 0}
  };
  while ((c = getopt_long(argc,argv,"hsoi",
                          long_options, &option_index)) != -1) {
    switch (c) {
    case 1: if (sscanf(optarg,"%lg",&pxc) != 1)
				       fun_Stop("incorrect format for PXC ");
    break;
    case 2: if (sscanf(optarg,"%lg",&phs) != 1)
				       fun_Stop("incorrect format for PHS ");
    break;
    case 3: if (sscanf(optarg,"%lg",&pvs) != 1)
				       fun_Stop("incorrect format for PVS ");
    break;
    case 4: if (sscanf(optarg,"%i",&cg_nfe) != 1) 
				       fun_Stop("incorrect format for nfe");
    break;
    case 5: 
		if (sscanf(optarg,"%lg:%lg:%i",&ol_p0,&ol_gr,&ol_num) != 3)
				fun_Stop("incorrect format --pmc=p0:gr:num");
    break;
		case 6: if (sscanf(optarg,"%lg:%s",&pbg,bgFile) != 2) 
	      fun_Stop("incorrect format for --pbg:PenaltyVal:fileName");
    break;
    case 's' : scaling=1;
    break;
    case 'o' : ol_out=1;
    break;
    case 'i' : ini_out=1;
    break;
    case 'h' : fun_usage();
    break;
    default  : fprintf(stderr,"Default\n"); fun_Stop(USAGE);
      break;
    } 
  }

  //
  // FillUp the Command Line Structure
  //
  // Notes: The defaults assigned in this file are just flags for the 
  // main program to interpret that the user did not want to change
  // the default values, whiach are actually keep in the main program.
  // 
  //
  (*clp).cg_nfe = cg_nfe;

  (*clp).pxc = pxc;
  (*clp).phs = phs;
  (*clp).pvs = pvs;
  (*clp).pbg = pbg;
  strcpy((*clp).bgf_fname, bgFile);

  (*clp).sca = scaling;
  (*clp).ol_out = ol_out;

  (*clp).ol_p0 = ol_p0;
  (*clp).ol_gr = ol_gr;
  (*clp).ol_num = ol_num;
  
  //(*clp).rff = 1; // It si going to be always gridded data. radFlag;
  //(*clp).bff = bgFlag;
  //sprintf((*clp).rf,"%s",rdx);
  //sprintf((*clp).bgf,"%s",bgFile);

  (*clp).argc = argc;
  (*clp).argv = argv;
  
  return(clp);

} /* EOfun_cgeta */

//
//---------------------------------------------------------------------
//
void CheckCLP(struct clpInfo *clp) {
fprintf(stderr,"cg_nfe=%i\n",(*clp).cg_nfe);

fprintf(stderr,"bff=%i\n",(*clp).bff);
fprintf(stderr,"pxc=%g\n",(*clp).pxc);
fprintf(stderr,"phs=%g\n",(*clp).phs);
fprintf(stderr,"pvs=%g\n",(*clp).pvs);

fprintf(stderr,"ol_p0=%g\n",(*clp).ol_p0);
fprintf(stderr,"ol_gr=%g\n",(*clp).ol_gr);
fprintf(stderr,"ol_num=%i\n",(*clp).ol_num);


fprintf(stderr,"ol_out=%i\n",(*clp).ol_out);
fprintf(stderr,"sca=%i\n",(*clp).sca);
}
//
//---------------------------------------------------------------------
void fun_usage() {
//
//Note: actual Default values are kept in the main function at driver.c
//the following define line should have those values in it. 
//Default values assigned below, in this file, are only markes to signal 
//the main function that the user does not wish to change the actual defaults
//
fprintf(stderr,"\nOptions meanings and their defaults.\n for more info type man ng3dvar \n");
fprintf(stderr," ---------------------------------------------------\n");
fprintf(stderr,"nfe --Maximun number of function evaluations (default 5000)\n");
fprintf(stderr,"phs --Weight of horizontal smoothing         (default none)\n");
fprintf(stderr,"pvs --Weight of vertical smoothing           (default none)\n");
fprintf(stderr,"pmc -- iteration parameters: \n");
fprintf(stderr,"  p0 --Initial penalty for residual of MC    (default 1)\n");
fprintf(stderr,"  gr -- growth rate for the penalty          (default 1.5)\n");
fprintf(stderr,"  num -- Maximun Number of iterations        (default 10)\n");
fprintf(stderr,"-o request to  write Output every iteration  \n");
fprintf(stderr,"-s request scaling of target before starting the iteration\n");

fun_Stop(USAGE);
}
