/* driver_is -- Initial Solution 
                contains fun_InPut, which is called when an initialization
                file for the analysis is provided by the user. 
*/

#include "ng3dvar.h"
#include "cdfhdr.h"

//#define RHO "rho"
//
// Externally visible
//
void SetInitalSol(struct funInfo *finfo,double *xx);

// 
// External Functions
//
float *Fgetbuff(long nelem);
FILE *OpenFileR(char *FileName);
void fun_Stop(char *s);

//
// Internal Functions
//
static void fun_InPut(struct funInfo *finfo,double *vel,int *rhoFlag);
static void fun_GetCandisHeader(FILE *fh);
static void fun_alloc();
static void fun_FillGridPoints(double *vel);
//static void fun_GetRhoProfile(double *rho);

//Module Globals for convinience.
static float lon0,lat0,alt0;    /* grid origin */
static float Dxg,Dyg,Dzg;       /* step size */
static int Nxg,Nyg,Nzg,Ng;	/* grid dimensions */

/* candis stuff */
static char hbin[HBMAX][LINE];
static char cline[LINE],pline[LINE];
static float bad,badlim;
static float *sbin,*vbin;
static long nsbin,nvbin;
static struct field *fp;

/* fields having to do with the grid */
static float *vxi,*vyi,*vzi; // grid velocities
static float *rhoi;          // density profile

//-------------------------------------------------------------
void SetInitalSol(struct funInfo *finfo,double *xx){
//
// The Initialization is always  done on the finest grid!!
//
// If there is an initialization file, it is read in by fun_InPut
// this function tries also to read the density profile if one is
// available -- null is returned if the density is not in the read file.
// 
int i,Nzg,Ne,initFlag;
int rhoFlag = -1;
float Dzg;
double *rhoProf;

  rhoProf = (*finfo).rho; 
  initFlag = (*finfo).iff;
  Dzg = (*finfo).grid.Dz;
  Nzg = (*finfo).grid.Nz;
  Ne  = (*finfo).grid.Ne;

	if (initFlag > 0) { 
    fun_InPut(finfo,xx,&rhoFlag);
  } else {
    for (i = 0; i< Ne; i++){ xx[i] = 0.0;}
  }

}
//-------------------------------------------------------------------
void fun_InPut(struct funInfo *finfo,double *vel,int *rhoFlag){

FILE *fhandle;
double *rho;
  
//
// The input always bring in the field on the finest grid.
//
//
	fhandle = OpenFileR((*finfo).inif);

	//rho = (*finfo).rho;

  lon0 = (*finfo).lon0;
  lat0 = (*finfo).lat0;
  alt0 = (*finfo).alt0;

  Dxg = (*finfo).grid.Dx;
  Dyg = (*finfo).grid.Dy;
  Dzg = (*finfo).grid.Dz;

  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;
  Ng =  (*finfo).grid.Np;

  /* read inheader and make outheader */
  fun_GetCandisHeader(fhandle);

  /* allocate memory for static & variable fields */
	fun_alloc(); 

	getslice(fhandle,nsbin,sbin); //we need to get pass this to access vars
	getslice(fhandle,nvbin,vbin);
  
  fun_FillGridPoints(vel);

  // Ignore  density and also reflectivity fields, they should
  // be the same as those in the input data. the refl is actually
  // the result of the interpolation.

  Dfreebuff(sbin);
  Dfreebuff(vbin);
      
} //
/*--------------------------------------------------------------------*/
 void fun_GetCandisHeader(FILE *fh) {
float val;
float TOLe = 1e-6;
int vali;

   /* get header of input file */
   gethdr(fh,hbin,HBMAX);
   if (getfmt(hbin,HBMAX) != 'f') {
      fun_Stop("inPut@fun_GetCandisHeader:format f expected");
   }
   // Check lon0
	 if (seekpar(hbin,HBMAX,"lon0",pline) == OK) val = atof(pline);
	 else fun_Stop("inPut@fun_GetCandisHeader:Can't find lon0");
	 if ( fabs(val - lon0) > TOLe )
     fun_Stop("inPut@fun_GetCandisHeader:Wrong value of lon0");

   // Check lat0
	 if (seekpar(hbin,HBMAX,"lat0",pline) == OK) val = atof(pline);
	 else fun_Stop("inPut@fun_GetCandisHeader:Can't find lat0");
	 if ( fabs(val - lat0) > TOLe )
     fun_Stop("inPut@fun_GetCandisHeader:Wrong value of lat0");

   // Check alt0
	 if (seekpar(hbin,HBMAX,"alt0",pline) == OK) val = atof(pline);
	 else fun_Stop("inPut@fun_GetCandisHeader:Can't find alt0");
	 if ( fabs(val - alt0) > TOLe )
     fun_Stop("inPut@fun_GetCandisHeader:Wrong value of alt0");

  // 
  // Check Grid Sizes-- should be equal to the finest Grid sizes.
  //
  if ( (fp = seekfld(hbin,HBMAX,'s',"x")) != NULL) vali = (*fp).dsize1;
	else fun_Stop("inPut@fun_GetCandisHeader:Can't find x");
  if ( vali != Nxg) {
	  fprintf(stderr,"X-dimSizeOnInputFile = %i\t expectd %i\n",vali,Nxg);
	  fun_Stop("inPut@fun_GetCandisHeader:Dimension Mismatch");
  }

  if ( (fp = seekfld(hbin,HBMAX,'s',"y")) != NULL) vali = (*fp).dsize1;
	else fun_Stop("inPut@fun_GetCandisHeader:Can't find y");
  if ( vali != Nyg) {
	  fprintf(stderr,"Y-dimSizeOnInputFile = %i\t expectd %i\n",vali,Nyg);
	  fun_Stop("inPut@fun_GetCandisHeader:Dimension Mismatch");
	}

  if ( (fp = seekfld(hbin,HBMAX,'s',"z")) != NULL) vali = (*fp).dsize1;
	else fun_Stop("inPut@fun_GetCandisHeader:Can't find z");
  if ( vali != Nzg) {
	fprintf(stderr,"Z-dimSizeOnInputFile = %i\t expectd %i\n",vali,Nzg);
	fun_Stop("inPut@fun_GetCandisHeader:Dimension Mismatch");
	}


   // Should also check x0,y0,z0,dx,dy,dz,

} /* EOfun_make_header */
/*--------------------------------------------------------------------*/
void fun_alloc() {
  
  /* allocate buffers */
  nsbin  = elemcnt(hbin,HBMAX,'s');
  nvbin  = elemcnt(hbin,HBMAX,'v');
  sbin   = Fgetbuff(nsbin);
  vbin   = Fgetbuff(nvbin);

  // Get Pointers
  vxi = getptr(hbin,HBMAX,vbin,'v','d',"vx");
  vyi = getptr(hbin,HBMAX,vbin,'v','d',"vy");
  vzi = getptr(hbin,HBMAX,vbin,'v','d',"vz");

  //rhoi = getptr(hbin,HBMAX,vbin,'v','r',RHO); // NULL if thereIsNoRHO

} 
/*--------------------------------------------------------------------*/
//void fun_GetRhoProfile(double *rho) {
//int loop;
//  if (rhoi != NULL ) {
//    for(loop = 0; loop < Nzg; loop++) { rho[loop]  = (double) rhoi[loop];}
//  }
//  else { 
//	   fprintf(stderr,"#\n#inPut@fun_GetRhoProfile\n");
//	   fprintf(stderr,"#could not find rho in the input variable slice\n#\n");
//	   fprintf(stderr,"#will use preloaded profile\n#\n");
//     //rho  = NULL;
//  }
//}
/*--------------------------------------------------------------------*/
void fun_FillGridPoints(double *vel) {
//
// Initialize velocities
//
int gloop;
  
  for(gloop = 0; gloop < Ng; gloop++) { /*loop over grid points */
    vel[gloop]        = (double) vxi[gloop];
		vel[gloop + Ng]   = (double) vyi[gloop];
    vel[gloop + 2*Ng] = (double) vzi[gloop];
  }
} 
//-------------------------------------------------------------------
