/* driver_os -- Output Solution
 */

#include "ng3dvar.h"
#include "cdfhdr.h"

//OutPutFieldNames
#define RHO "rho"
#define DBZ "dbz"
#define Ptv "wpt"
#define VX  "u"
#define VY  "v"
#define VZ  "w"

//
// Externally Visible
//
void fun_ws2f(struct clpInfo *clp,struct funInfo *finfo,
              double *vel, char *Ofile);

//
//  External Functions
//
char *strcpy(char *dest, const char *src);
char *strcat(char *dest, const char *src);
float *Fgetbuff(long nelem);
FILE *OpenFileW(char *FileName);

// 
// Internal Functions 
//
void fun_make_header(struct clpInfo *clp, struct funInfo *finfo,
                     char hbout[][LINE]);
void fun_fill_index(struct funInfo *finfo,
                    float xg0, float yg0, float zg0,
                    float *xg, float *yg, float *zg);

//-------------------------------------------------------------------
//
void fun_ws2f(struct clpInfo *clp,struct funInfo *finfo,
              double *vel, char *Ofile){
//Candis Stuff
char hbout[HBMAX][LINE];
float *sbout,*vbout;
long nsbout,nvbout;

FILE *ofh;

float *rho,*dbz,*ptv; //grid vars
float *vx,*vy,*vz;    // grid velocities
float *xg,*yg,*zg;	  // grid coordinates 
float xg0,yg0,zg0;    // Origin Of coordinates.
double *rhoProf;
double *in_dbz;
double *in_ptv;
int Nx,Ny,Nz,Ng,gloop;


//
// Get OutputFile handle
//
  if ( strcmp(Ofile,"stdout") != 0 ) ofh = OpenFileW(Ofile);
  else                               ofh = stdout;

// Get Info
  Nx = (*finfo).rd.Nx;
  Ny = (*finfo).rd.Ny;
  Nz = (*finfo).rd.Nz;
  Ng  = (*finfo).grid.Np;
  rhoProf = (*finfo).rho;
  in_dbz = (*finfo).dbzi;
  in_ptv = (*finfo).wtZi;



  /* read inheader and make outheader */
  fun_make_header(clp,finfo,hbout);

  /* write out header */
  puthdr(ofh,hbout,HBMAX);
  
  /* allocate memory for static & variable fields */
  nsbout = elemcnt(hbout,HBMAX,'s');
  nvbout = elemcnt(hbout,HBMAX,'v');
  sbout  = Fgetbuff(nsbout);
  vbout  = Fgetbuff(nvbout);
  
  // Get Pointer to index fields and fill them up
  xg = getptr(hbout,HBMAX,sbout,'s','d',(*finfo).rd.xname);
  yg = getptr(hbout,HBMAX,sbout,'s','d',(*finfo).rd.yname);
  zg = getptr(hbout,HBMAX,sbout,'s','d',(*finfo).rd.zname);
  //fun_fill_index(finfo,xg0,yg0,zg0,xg,yg,zg);

  for (gloop = 0; gloop < Nx; gloop++) xg[gloop] = (*finfo).rd.xg[gloop];
  for (gloop = 0; gloop < Ny; gloop++) yg[gloop] = (*finfo).rd.yg[gloop];
  for (gloop = 0; gloop < Nz; gloop++) zg[gloop] = (*finfo).rd.zg[gloop];

  /* write static slice */
  putslice(ofh,nsbout,sbout);
  
  /* get pointers for new fields  and fill them up */
  rho = getptr(hbout,HBMAX,vbout,'v','d',RHO);
  dbz = getptr(hbout,HBMAX,vbout,'v','d',DBZ);
  ptv = getptr(hbout,HBMAX,vbout,'v','d',Ptv);
  vx = getptr(hbout,HBMAX,vbout,'v','d',VX);
  vy = getptr(hbout,HBMAX,vbout,'v','d',VY);
  vz = getptr(hbout,HBMAX,vbout,'v','d',VZ);
  
  // xfer rho 
  for (gloop = 0; gloop < Nz; gloop++){
	   rho[gloop] = (float) rhoProf[gloop];
	}
  // Xfer 3D-fileds
  for(gloop = 0; gloop < Ng; gloop++) {
    
    dbz[gloop] = (float) in_dbz[gloop];
    ptv[gloop] = (float) in_ptv[gloop];
    vx[gloop] = (float) vel[gloop];
		vy[gloop] = (float) vel[gloop + Ng];
    vz[gloop] = (float) vel[gloop + 2*Ng];
  }
      
  /* write variable slice */
  putslice(ofh,nvbout,vbout);

  // CleanUp
  if (Ofile) fclose(ofh);
  Dfreebuff(sbout);
  Dfreebuff(vbout);
}
/*--------------------------------------------------------------------*/
void fun_make_header(struct clpInfo *clp,struct funInfo *finfo, 
                     char hbout[][LINE]) {
  
char pline[LINE];
char xn[LINE],yn[LINE],zn[LINE];
char x0n[LINE],y0n[LINE],z0n[LINE];
char Dxn[LINE],Dyn[LINE],Dzn[LINE];
char Nxn[LINE],Nyn[LINE],Nzn[LINE];
float bad,badlim;
int Nx,Ny,Nz,Ng;	/* grid dimensions */
float Dx,Dy,Dz;
float x0,y0,z0;
float lon0,lat0,alt0;

	lon0 = (*finfo).rd.lon0;
	lat0 = (*finfo).rd.lat0;
	alt0 = (*finfo).rd.alt0;
	Nx = (*finfo).rd.Nx;
	Ny = (*finfo).rd.Ny;
	Nz = (*finfo).rd.Nz;
	Dx = (*finfo).rd.Dx;
	Dy = (*finfo).rd.Dy;
	Dz = (*finfo).rd.Dz;
  x0 = (*finfo).rd.xg[0];
  y0 = (*finfo).rd.yg[0];
  z0 = (*finfo).rd.zg[0];

  strcpy(pline,(*finfo).rd.xname);
	strcat(pline,"0");
	strcpy(x0n,pline);
  strcpy(pline,(*finfo).rd.yname);
	strcat(pline,"0");
	strcpy(y0n,pline);
  strcpy(pline,(*finfo).rd.zname);
	strcat(pline,"0");
	strcpy(z0n,pline);

	strcpy(pline,"d");
  strcat(pline,(*finfo).rd.xname);
	strcpy(Dxn,pline);
	strcpy(pline,"d");
  strcat(pline,(*finfo).rd.yname);
	strcpy(Dyn,pline);
	strcpy(pline,"d");
  strcat(pline,(*finfo).rd.zname);
	strcpy(Dzn,pline);

	strcpy(pline,"N");
  strcat(pline,(*finfo).rd.xname);
	strcpy(Nxn,pline);
	strcpy(pline,"N");
  strcat(pline,(*finfo).rd.yname);
	strcpy(Nyn,pline);
	strcpy(pline,"N");
  strcat(pline,(*finfo).rd.zname);
	strcpy(Nzn,pline);

	strcpy(xn,(*finfo).rd.xname);
	strcpy(yn,(*finfo).rd.yname);
	strcpy(zn,(*finfo).rd.zname);

  /* make header of output file */
  /* comments */

  nullhdr(hbout,HBMAX,"float");

	copycmt(hbout,HBMAX,(*finfo).rd.hbin[0],HBMAX);

  addcommand((*clp).argc,(*clp).argv,hbout);

  sprintf(pline,"PRA=%g PBG=%g PMC=%g PBC=%g PHS=%g PVS=%g PXC=%g\n",
  (*finfo).TFT0.PRA,(*finfo).TFT0.PBG,(*finfo).TFT0.PMC,
  (*finfo).TFT0.PBC,(*finfo).TFT0.PHS,(*finfo).TFT0.PVS,(*finfo).TFT0.PXC);
  addcline(hbout,HBMAX,pline);

  /* parameters */
  bad = BAD;
  sprintf(pline,"%g",bad);
  addpar(hbout,HBMAX,"bad",pline);
  badlim = BADLIM;
  sprintf(pline,"%g",badlim);
  addpar(hbout,HBMAX,"badlim",pline);
  
  /* index parameters */

  sprintf(pline,"%g",lon0);
  addpar(hbout,HBMAX,"lon0",pline);
  sprintf(pline,"%g",lat0);
  addpar(hbout,HBMAX,"lat0",pline);
  sprintf(pline,"%g",alt0);
  addpar(hbout,HBMAX,"alt0",pline);

  sprintf(pline,"%g",x0);
  addpar(hbout,HBMAX,x0n,pline);
  sprintf(pline,"%g",y0);
  addpar(hbout,HBMAX,y0n,pline);
  sprintf(pline,"%g",z0);
  addpar(hbout,HBMAX,z0n,pline);

  sprintf(pline,"%g",Dx);
  addpar(hbout,HBMAX,Dxn,pline);
  sprintf(pline,"%g",Dy);
  addpar(hbout,HBMAX,Dyn,pline);
  sprintf(pline,"%g",Dz);
  addpar(hbout,HBMAX,Dzn,pline);

  sprintf(pline,"%i",Nx);
  addpar(hbout,HBMAX,Nxn,pline);
  sprintf(pline,"%i",Ny);
  addpar(hbout,HBMAX,Nyn,pline);
  sprintf(pline,"%i",Nz);
  addpar(hbout,HBMAX,Nzn,pline);
  
  /* add index fields */
  addfld(hbout,HBMAX,'s',xn,1000.,0.,'l',1,xn,Nx);
  addfld(hbout,HBMAX,'s',yn,1000.,0.,'l',1,yn,Ny);
  addfld(hbout,HBMAX,'s',zn,1000.,0.,'l',1,zn,Nz);
  
  /* add new fields  */
  addfld(hbout,HBMAX,'v',RHO,10.,0.,'s',1,zn,Nz);
  addfld(hbout,HBMAX,'v',DBZ,10.,0.,'s',3,xn,Nx,yn,Ny,zn,Nz);
  addfld(hbout,HBMAX,'v',Ptv,10.,0.,'s',3,xn,Nx,yn,Ny,zn,Nz);
  addfld(hbout,HBMAX,'v',VX,10.,0.,'s',3,xn,Nx,yn,Ny,zn,Nz);
  addfld(hbout,HBMAX,'v',VY,10.,0.,'s',3,xn,Nx,yn,Ny,zn,Nz);
  addfld(hbout,HBMAX,'v',VZ,10.,0.,'s',3,xn,Nx,yn,Ny,zn,Nz);
  
  /* write output header */
  //puthdr(ofh,hbout,HBMAX);
} 
/*--------------------------------------------------------------------*/
void fun_fill_index(struct funInfo *finfo,
                    float xg0, float yg0, float zg0,
                    float *xg, float *yg, float *zg) {
int Nxg,Nyg,Nzg;
float Dxg,Dyg,Dzg;
float auxvf;
int iloop;

	Nxg = (*finfo).grid.Nx;
	Nyg = (*finfo).grid.Ny;
	Nzg = (*finfo).grid.Nz;

	Dxg = (*finfo).grid.Dx;
	Dyg = (*finfo).grid.Dy;
	Dzg = (*finfo).grid.Dz;

  
  auxvf = xg[0] = xg0; 
  for (iloop = 1; iloop < Nxg; iloop++) {
    auxvf += Dxg;
    xg[iloop] = auxvf;
  }
  auxvf = yg[0] = yg0;
  for (iloop = 1; iloop < Nyg; iloop++) {
    auxvf += Dyg;
    yg[iloop] = auxvf;
  }
  auxvf = zg[0] = zg0;
  for (iloop = 1; iloop < Nzg; iloop++){
    auxvf += Dzg;
    zg[iloop] = auxvf;
  }
} /* EOfun_fill_index */
/*--------------------------------------------------------------------*/
