//
// calcfg -- this implements the function to be minimized by conmin.
//
#include "ng3dvar.h"

//
// Externally Visble
//
int calcfg(struct funInfo *finfo,int N, double *x, double *f, double *g);
void ScaleTF(struct funInfo *finfo,int N,double *x,double *f,double *g);
double MC_L2norm(struct funInfo *finfo,int N,double *x,double *g);
double RadMissFit_L2norm(struct funInfo *finfo,int N,double *x,double *g);
double Lap_L2norm(struct funInfo *finfo,int N,double *x,double *g);
double BGFit_L2norm(struct funInfo *finfo,int N,double *x,double *g);
void stop();

//extern int eva;

//
// External Functions
//
double FieldNorm2(int N, double *fld);

double if_ngrad(struct funInfo *finfo,int N,
                double *ndat,double *x,double *g);
double Fmc(struct funInfo *finfo,int N, double *x,double *g);
double FmcSC(struct funInfo *finfo,int N, double *x,double *g);
double Flap(struct funInfo *finfo,int N, double *x,double *g);
double Fbgf(struct funInfo *finfo,int N, double *x,double *g);
//


//-------------------------------------------------------------
int calcfg(struct funInfo *finfo,int N, double *x, double *f, double *g){

double PBG,PBC,PMC,PHS,PVS,PRA,PXC;
double sum,cf;
double frad,fmc,fbc,flap,fxc,fbg;
double grad,gdiv,gbc,glap,gbg,gnorm;
int i,kloop;
double STRO = 1.0; // Normalize to the number of radar Observations.
double Nrobs = 0;
double ScaRad;
double *auxG,faux;
double UnWe_fmc;
double UnWe_fbc;
double UnWe_flap;
//
   int ig,jg,Nxg,Nyg,Nzg;
   Nxg = (*finfo).grid.Nx;
   Nyg = (*finfo).grid.Ny;
   Nzg = (*finfo).grid.Nz;


  PRA = (*finfo).PRA; 
  PMC = (*finfo).PMC; 
  PBC = (*finfo).PBC; 
  PHS = (*finfo).PHS; 
  PVS = (*finfo).PVS; 
  PXC = (*finfo).PXC; 
  PBG = (*finfo).PBG; 

  //fprintf(stderr,"calcfg_if.c@calcfg:PRA = %g\n",PRA);
  //fprintf(stderr,"                  :PMC = %g\n",PMC);
  //fprintf(stderr,"                  :PHS = %g\n",PHS);
  //fprintf(stderr,"                  :PVS = %g\n",PVS);

  //fprintf(stderr,"                  :PBC = %g\n",PBC);
  //fprintf(stderr,"                  :PXC = %g\n",PXC);
  //fprintf(stderr,"                  :PBG = %g\n",PBG);


	//
	// Initialize function and its gradient, 
	// as well as the auxiliar holder. 
	//
	auxG = (*finfo).AG;
	*f = 0.0;
	for (i = 0; i< N; i++){ auxG[i] = 0.0; g[i] = 0.0;}

  //
	// Get the contribution from the radar
	// Then update the function, gradient and reset the holder
	//
  if ( PRA > DBL_EPSILON ) { //This always has a weight of one.
		 frad = if_ngrad(finfo,N,&Nrobs,x,auxG); 

     if ( Nrobs > 0 ) ScaRad = PRA/Nrobs; else ScaRad = PRA;
     //ScaRad = PRA;

     // Scale rad contribution -- This is the only functions 
     // whose contributions is done afterwards.
     //
     frad *= (ScaRad);
     *f += frad;
     for (i = 0; i< N; i++) { g[i] += (auxG[i]*ScaRad); auxG[i] = 0.0;}
	   //
		 //grad = FieldNorm2(N,auxG); //Testing
		 //
  } else { frad = 0.0; } 

  //
  //
  // Get the contribution from the Smoothing
  //
  if ( PHS > DBL_EPSILON || PVS > DBL_EPSILON) {
    flap = Flap(finfo,N,x,auxG);
    *f += flap; 
    for (i = 0; i< N; i++) { g[i] +=  auxG[i]; auxG[i] = 0.0;}
	  //
		//UnWe_fmc = fmc/PMC; //Get un-weighted value of contribution 
    //gdiv = FieldNorm2(N,auxG); //Testing
	  //
  } else { flap = 0.0;} //gdiv = 0.0;

  //
  // Get the contribution from the MassContinuity
  //
  if ( PMC > DBL_EPSILON ) {

    if ( (*finfo).sc > 0 ) fmc = FmcSC(finfo,N,x,auxG);
    else fmc = Fmc(finfo,N,x,auxG);

    *f += fmc; 
    for (i = 0; i< N; i++) { g[i] +=  auxG[i]; auxG[i] = 0.0;}
	  //
		//UnWe_fmc = fmc/PMC; //Get un-weighted value of contribution 
    //gdiv = FieldNorm2(N,auxG); //Testing
	  //
  } else { fmc = 0.0;} //gdiv = 0.0;

  //
  // Get the contribution from the Background Field
  //
  if ( PBG > DBL_EPSILON ) {
    fbg = Fbgf(finfo,N,x,auxG);
    *f += fbg; 
    for (i = 0; i< N; i++) { g[i] +=  auxG[i]; auxG[i] = 0.0;}
  } else { fbg = 0.0;} //gdiv = 0.0;

   if (*f < 0. ) {
     fprintf(stderr,"WARNING!! f=%g < 0 \n", *f);
   }
 // fprintf(stderr,"%i frad=%.4e  fmc=%.4e  fbc=%.4e flap=%.4e\n", 
 //      (*finfo).tfe,frad,fmc,fbc,flap); 
	

  (*finfo).tfe += 1; // update the number of evaluations so far.

  return(1);
}
//------------------------------------------------------------------------
double MC_L2norm(struct funInfo *finfo,int N,double *x,double *g){
//
//  This is just an interface to evaluate the norm of mass continuity at x
//
double PRA,PBC,PHS,PVS,PMC,PXC,PBG,f;

  //
  // Save Input Params
  //
  PMC = (*finfo).PMC;
  PRA = (*finfo).PRA;
  PBC = (*finfo).PBC;
  PHS = (*finfo).PHS;
  PVS = (*finfo).PVS;
  PXC = (*finfo).PXC;
  PBG = (*finfo).PBG;

  //
  // Set params, so we get the Mass Continuity without constraints
  //
  (*finfo).PMC =  1.0; 
  (*finfo).PRA =  0.0; 
  (*finfo).PBC =  0.0; 
  (*finfo).PHS =  0.0; 
  (*finfo).PVS =  0.0; 
  (*finfo).PXC =  0.0; 
  (*finfo).PBG =  0.0;

  calcfg(finfo,N,x,&f,g); 

  //
  // Recover  Input Params
  //
  (*finfo).PMC = PMC;
  (*finfo).PRA = PRA;
  (*finfo).PBC = PBC;
  (*finfo).PHS = PHS;
  (*finfo).PVS = PVS;
  (*finfo).PXC = PXC;
  (*finfo).PBG = PBG;

  return(f);
}
//------------------------------------------------------------------------
double RadMissFit_L2norm(struct funInfo *finfo,int N,double *x,double *g){
//
//  This is just an interface to evaluate only the radar missfit through
// the target function. The value of the missfit as defined in the target
// function, is a measure of how well the synthetic field fits the
// "observed" field taking into account the tolerances in the observatios.
// We still have to decided when we have fit the data and hence when we 
// have satisfied the constraint the observations impose on the continuity
// equation.  If this constraint is not satisfied, then the algorithm
// has to increase the penalty parameter. This is done not by the
// unconstraind minimization, but by constraint minimization in the
// outer loop of the algorithm.
//
double PRA,PBC,PHS,PVS,PMC,PXC,PBG,f;

  //Save Input Params
  PRA = (*finfo).PRA;
  PMC = (*finfo).PMC;
  PBC = (*finfo).PBC;
  PHS = (*finfo).PHS;
  PVS = (*finfo).PVS;
  PXC = (*finfo).PXC;
  PBG = (*finfo).PBG;

  //Set Params
  (*finfo).PRA = 1.0;
  (*finfo).PMC = 0.0;
  (*finfo).PBC = 0.0;
  (*finfo).PHS = 0.0;
  (*finfo).PVS = 0.0;
  (*finfo).PXC = 0.0;
  (*finfo).PBG = 0.0;

  // evaluate function for current params
  //fprintf(stderr,"RadMissFit_L2norm:\n");
  calcfg(finfo,N,x,&f,g);

  // Recover Input Params
  (*finfo).PRA =  PRA;
  (*finfo).PMC =  PMC;
  (*finfo).PBC =  PBC;
  (*finfo).PHS =  PHS;
  (*finfo).PVS =  PVS;
  (*finfo).PXC =  PXC;
  (*finfo).PBG =  PBG;

  return(f);
}
//------------------------------------------------------------------------
double Lap_L2norm(struct funInfo *finfo,int N,double *x,double *g){
//
//  This is just an interface to evaluate the norm of the Laplacian at x
//
double PRA,PBC,PHS,PVS,PMC,PXC,PBG,f;

  //Save Input Params
  PRA = (*finfo).PRA;
  PMC = (*finfo).PMC;
  PBC = (*finfo).PBC;
  PHS = (*finfo).PHS;
  PVS = (*finfo).PVS;
  PXC = (*finfo).PXC;
  PBG = (*finfo).PBG;

  //Set Params
  (*finfo).PHS = 1.0;
  (*finfo).PVS = 1.0;
  (*finfo).PRA = 0.0;
  (*finfo).PMC = 0.0;
  (*finfo).PBC = 0.0;
  (*finfo).PXC = 0.0;
  (*finfo).PBG = 0.0;

  // evaluate function for current params
  //fprintf(stderr,"Lap_L2norm:\n");
  calcfg(finfo,N,x,&f,g);

  // Recover Input Params
  (*finfo).PRA =  PRA;
  (*finfo).PMC =  PMC;
  (*finfo).PBC =  PBC;
  (*finfo).PHS =  PHS;
  (*finfo).PVS =  PVS;
  (*finfo).PXC =  PXC;
  (*finfo).PBG =  PBG;

  return(f);
}
//------------------------------------------------------------------------
double BGFit_L2norm(struct funInfo *finfo,int N,double *x,double *g){
//
//  This is just an interface to evaluate the norm of the misfit with the
//  background field at x
//
double PRA,PBC,PHS,PVS,PMC,PXC,PBG,f;

  if ( (*finfo).TFT0.PBG < DBL_EPSILON ) return(0.0); //HERE;

  //
  // Save Input Params
  //
  PMC = (*finfo).PMC;
  PRA = (*finfo).PRA;
  PBC = (*finfo).PBC;
  PHS = (*finfo).PHS;
  PVS = (*finfo).PVS;
  PXC = (*finfo).PXC;
  PBG = (*finfo).PBG;

  //
  // Set params, so we get the Mass Continuity without constraints
  //
  (*finfo).PMC =  0.0; 
  (*finfo).PRA =  0.0; 
  (*finfo).PBC =  0.0; 
  (*finfo).PHS =  0.0; 
  (*finfo).PVS =  0.0; 
  (*finfo).PXC =  0.0; 
  (*finfo).PBG =  1.0;

  calcfg(finfo,N,x,&f,g); 

  //
  // Recover  Input Params
  //
  (*finfo).PMC = PMC;
  (*finfo).PRA = PRA;
  (*finfo).PBC = PBC;
  (*finfo).PHS = PHS;
  (*finfo).PVS = PVS;
  (*finfo).PXC = PXC;
  (*finfo).PBG = PBG;

  return(f);
}
//---------------------------------------------------------------
void ScaleTF(struct funInfo *finfo,int N,double *x,double *f,double *g){
//
// if originaly requested,
// check if constraint is active or not.
// if it is active(i.e violated), then scale it.
//
double Fmc,Fxc,Fbc,Fbg; //Value of penalty terms.
double PMC, PBG, PBC, PXC;
double MainTarget,PP;

  PP = 1.0; //(*finfo).PP; Go ahead and do scaling
  if ( PP <= DBL_EPSILON ) {
		return; // Nothing to do.
  }

	//Collect user input. What constraints were requested?
	PBG = (*finfo).TFT0.PBG; 
	PBC = (*finfo).TFT0.PBC;
	PXC = (*finfo).TFT0.PXC;
	PMC = (*finfo).PMC; //This changes during execution 


  Fmc = Fxc = Fbc = Fbg = 0; 

  //Do some trivial checks
  if ( (*finfo).TFT0.PRA != 1) {
     fprintf(stderr,"PRA must be selected\n"); 
     exit(1);
  }

  // By Default Set to zero the penalty terms
  // So, we can calculate the MainTarget value.
	// The main target includes data + smoothing terms
  (*finfo).PBG =  0.0;
  (*finfo).PBC =  0.0; 
  (*finfo).PXC =  0.0; 
  (*finfo).PMC =  0.0; 
  calcfg(finfo,N,x,f,g); 
  MainTarget = fabs(*f);

  //
  // Evaluate penalty terms; if active then scale and penalize them.
  // otherwise keep them inactive.
  //

  if ( -1 > DBL_EPSILON ) {   // Do Not scale the Mass contuity
	//                          // this will take care of itself, but
	//                          // perhaps more slowly.
  //if ( (*finfo).TFT0.PMC > DBL_EPSILON )  //user requested constraint?
     //fprintf(stderr,"We got a Problem \n"); exit(1);
     (*finfo).PMC =  1.0;
     (*finfo).PRA =  0.0;
     (*finfo).PBC =  0.0;
     (*finfo).PHS =  0.0;
     (*finfo).PVS =  0.0;
     (*finfo).PXC =  0.0;
     (*finfo).PBG =  0.0;
     calcfg(finfo,N,x,f,g); 
     Fmc = fabs(*f);
     if (Fmc > DBL_EPSILON)//if active constraint, Scale penalty
           PMC = ((*finfo).PMC * MainTarget)/(Fmc + 1.0); 
     else  PMC = 0.0;
  } 

  //Evaluate XC term; if active then scale it and penalize it.
  if ( (*finfo).TFT0.PXC > DBL_EPSILON ) { //user requested constraint?
     (*finfo).PXC =  1.0;
     (*finfo).PBG =  0.0;
     (*finfo).PRA =  0.0;
     (*finfo).PMC =  0.0;
     (*finfo).PBC =  0.0;
     (*finfo).PHS =  0.0;
     (*finfo).PVS =  0.0;
     calcfg(finfo,N,x,f,g); 
     Fxc = fabs(*f);
     if (Fxc > DBL_EPSILON)//if active constraint, Scale penalty
          PXC = ((*finfo).TFT0.PXC * MainTarget)/(Fxc + 1.0);
     else PXC = 0.0;
  }

  //Evaluate BC term; if active then scale it and penalize it.
  if ( (*finfo).TFT0.PBC > DBL_EPSILON ) { //user requested constraint?
     (*finfo).PBC =  1.0; 
     (*finfo).PMC =  0.0; 
     (*finfo).PRA =  0.0; 
     (*finfo).PHS =  0.0; 
     (*finfo).PVS =  0.0; 
     (*finfo).PXC =  0.0; 
     (*finfo).PBG =  0.0;
     calcfg(finfo,N,x,f,g); 
     Fbc = fabs(*f);
     if (Fbc > DBL_EPSILON)//if active constraint, Scale penalty
          PBC = ((*finfo).TFT0.PBC * MainTarget)/(Fbc + 1.0); 
     else PBC = 0.0;

  }

  //Evaluate BG term; if active then scale it and penalize it.
  if ( (*finfo).TFT0.PBG > DBL_EPSILON ) { //user requested constraint?
     (*finfo).PBG =  1.0;
     (*finfo).PRA =  0.0;
     (*finfo).PMC =  0.0; 
     (*finfo).PXC =  0.0; 
     (*finfo).PBC =  0.0; 
     (*finfo).PHS =  0.0; 
     (*finfo).PVS =  0.0; 
     calcfg(finfo,N,x,f,g); 
     Fbg = fabs(*f);
     if (Fbg > DBL_EPSILON)//if active constraint, Scale term 
          PBG = ((*finfo).TFT0.PBG * MainTarget)/(Fbg + 1.0);
     else PBG = 0.0;
  }

	//recover weights for terms in main target
  (*finfo).PRA =  (*finfo).TFT0.PRA;
  (*finfo).PHS =  (*finfo).TFT0.PHS;
  (*finfo).PVS =  (*finfo).TFT0.PVS;

	// Update "Penalty factors" Scale factors rather; 
  (*finfo).PXC = PXC; 
  (*finfo).PBC = PBC; 
  (*finfo).PBG = PBG; 
  (*finfo).PMC = PMC; 

fprintf(stderr,"Scale MT=%g MC=%g Rest=%g\n",MainTarget,Fmc,Fxc*Fbc*Fbg);
fprintf(stderr,"PRA=%g PHS=%g PVS=%g PMC=%g PXC=%g PBC=%g PBG=%g \n",
           (*finfo).PRA,(*finfo).PHS,(*finfo).PVS,(*finfo).PMC,
           (*finfo).PXC,(*finfo).PBC,(*finfo).PBG);

}
