#include "ng3dvar.h"

// 
// Externally Visble
//
struct funInfo *SetfunInfo(
			   struct clpInfo *clp, struct RadarDataInfo *rdi,struct BGFdata *bgf,
			   struct gridInfo *gridi, struct cgInfo *cgi,
			   double PRA,double PMC,double PBC,double PHS,
			   double PVS,double PBG,double PXC);

void CheckFINFO(struct funInfo *finfo);

//
// External Functions
//
double *Dgetbuff(long nelem);
float *Fgetbuff(long nelem);
void fun_rho(struct funInfo *finfo);
void fun_wpTVZ(struct funInfo *finfo);

//-------------------------------------------------------------------
struct funInfo *SetfunInfo(
			   struct clpInfo *clp, struct RadarDataInfo *rdi,
			   struct BGFdata *bgf,
			   struct gridInfo *gridi, struct cgInfo *cgi,
			   double PRA,double PMC,double PBC,double PHS,
			   double PVS,double PBG,double PXC){

  struct funInfo *finfo;
  int Ntot;
  // 
  // Get space to hold the structures.
  //
  if((finfo =(struct funInfo*)malloc(sizeof(struct funInfo)*(2)))==NULL){
    fprintf(stderr,"SetUpfunInfo: cannot get enough memory\n"); exit(1);}

  // initialize the counter of function evaluations.
  (*finfo).tfe = 0; 

  //
  // Choose what terms are included in the taget function computation
  //
  (*finfo).PRA =  PRA; // RadarMisffit --always one.
  (*finfo).PMC =  PMC; // MassContinuity Term
  (*finfo).PBC =  PBC; //1e2; // BoundaryConditions Term
  (*finfo).PHS =  PHS; // Horizontal Smoothing Term
  (*finfo).PVS =  PVS; // Vertical Smoothing Term
  (*finfo).PXC =  PXC; // ExtraConstrain

  if (PBG > 0 ) {//Until this point PBG has the value of sigma from the user
    (*finfo).PBG =  1; // Background Term is on!
    (*finfo).TFT0.BGsigma = PBG;  //save this value here. From now on
    // PBG may change to accommodate scaling
  }

  // Honor User request, if any.
  // This is now don at the main routine.
  //if ( (*clp).pmc >= 0 )  (*finfo).PMC =  (*clp).pmc;
  //if ( (*clp).pxc >= 0 )  (*finfo).PXC =  (*clp).pxc;
  //if ( (*clp).phs >= 0 )  (*finfo).PHS =  (*clp).phs;
  //if ( (*clp).pvs >= 0 )  (*finfo).PVS =  (*clp).pvs;

  //
  // Save Initial Selection: This is important because as the algorithm
  // progresses we have to deactivate constraints that are satisfied on the
  // current solution, but we have to reactivate them if they become
  // violated further on. Since we only have to reactivate constraints
  // originally requested to be active, we need to keep handy what they are.
  //
  (*finfo).TFT0.PRA = (*finfo).PRA; 
  (*finfo).TFT0.PMC = (*finfo).PMC; 
  (*finfo).TFT0.PBC = (*finfo).PBC; 
  (*finfo).TFT0.PHS = (*finfo).PHS;
  (*finfo).TFT0.PVS = (*finfo).PVS;
  (*finfo).TFT0.PXC = (*finfo).PXC;
  (*finfo).TFT0.PBG = (*finfo).PBG;

  //
  // Get a pointer to radar data information.
  //
  (*finfo).rd = *rdi;
  (*finfo).sc = (*rdi).sc;
  // fprintf(stderr,"rdi.sc = %i finfo.sc = %i\n",(*rdi).sc,(*finfo).sc);
	
  //
  // Get a pointr to Background field (if any);
  //
  if (PBG > 0.) (*finfo).bgf = *bgf;

  // Get Grid steps and y0 cast as doubles and to rads if necessary.
  (*finfo).dx = (double) (*rdi).Dx;
  (*finfo).dy = (double) (*rdi).Dy;
  (*finfo).dz = (double) (*rdi).Dz;
  //(*finfo).y0 = (double) (*rdi).lon0;
  if ( (*rdi).sc > 0 ) { // Conver@radians.
    (*finfo).dx *= D2R;
    (*finfo).dy *= D2R;
    //(*finfo).y0 *= D2R; //<<--------------------------------
  } 

  //
  // Get a pointer to grid information.
  //
  (*finfo).grid = *gridi; 

  //
  // Get a pointer to cg information.
  //
  (*finfo).cgi = *cgi; 

  //
  // extra info regarding the grids.
  //
  (*finfo).NumVC = (*gridi).Nc;   // No. of velocity components
  (*finfo).mxn   = (*gridi).Ne;   //No. of elements on finer grid.

  //
  // Origin of coordinates for the analysis grid.
  //
  (*finfo).lon0 = (*rdi).lon0;
  (*finfo).lat0 = (*rdi).lat0;
  (*finfo).alt0 = (*rdi).alt0;

  //
  // Input files information
  // flags
  (*finfo).bff  = (*clp).bff;
  //(*finfo).bgf  = (*clp).bgf;

  //  (*finfo).rd.fn   = (*clp).rf;
  //(*finfo).iff  = (*clp).iff;
  //(*finfo).rf   = (*clp).rf;
  //(*finfo).inif = (*clp).inif;

  //
  // Set Up space to hold Radar Data and Get the data.
  //
  // //fun_GRDget(finfo);

  //
  // Request memory to hold auxiliar arrays
  // required in evaluating the function
  //
  // auxiliar holder for evaluation of the function Gradient.
  (*finfo).AG  = Dgetbuff((*gridi).Ne+2);

  (*finfo).XG   = Fgetbuff((*gridi).Nx+2);
  (*finfo).YG   = Fgetbuff((*gridi).Ny+2);
  (*finfo).ZG   = Fgetbuff((*gridi).Nz+2);

  (*finfo).rho  = Dgetbuff((*gridi).Nz+2);
  (*finfo).rhoF = Dgetbuff((*gridi).Nz+2);

  //
  // For display purposes, the reflectivity factor and
  // the particle terminal velocities are interpolated to
  // the analysis grid, so they must be defined in that space.
  //
  (*finfo).dbzi  = Dgetbuff((*gridi).Np+2);
  (*finfo).wtZi  = Dgetbuff((*gridi).Np+2);
  // 
  // The particle vertical velocity for the numerical work 
  // must be defined on the same space as the observation data. 
  //
  (*finfo).wtZ = Dgetbuff((*finfo).rd.Nx * (*finfo).rd.Ny * (*finfo).rd.Nz);
  
  //
  // SetUp "Continuous" PARAMETERS -- required during function evaluation.
  // reflectivity factor(dbz) and density.
  //
  fun_rho(finfo);

  // 
  // Get Particle Terminal Velocity on the Data Grid.
  //
  //fun_wtZ(finfo); //<<GetThis Back in for radar data.
  fun_wpTVZ(finfo); // This replaces fun_wtZ

  //fun_getDBZ(finfo);
  //fun_wtZcalc(finfo);
  //fun_CoarseWTZ(finfo);

  // Auxiliar Fields -- they are here for simplicity in passing info to
  // the writing routine. This fields are only for displaying purposes.

  // (*finfo).hmfc = Dgetbuff((*gridi).Np+2);
  // (*finfo).vmf = Dgetbuff((*gridi).Np+2);

  return(finfo);
}
//------------------------------------------------------------
//
//---------------------------------------------------------------------
//
void CheckFINFO(struct funInfo *finfo){
  int mxn,l;

  fprintf(stderr,"(lo,la,a)(%f,%f,%f)\n",
	  (*finfo).lon0,(*finfo).lat0,(*finfo).alt0);

  //fprintf(stderr,"(rff,rf)=(%i,%s)\n",(*finfo).rff,(*finfo).rf);
  fprintf(stderr,"(iff,inif)=(%i,%s)\n",(*finfo).iff,(*finfo).inif);
  //fprintf(stderr,"(bff,bgf)=(%i,%s)\n",(*finfo).bff,(*finfo).bgf);

  fprintf(stderr,"PRA = %g --Radar\n",(*finfo).PRA);
  fprintf(stderr,"PBC = %g --Boundaries\n",(*finfo).PBC);
  fprintf(stderr,"PMC = %g --MassContinuity\n",(*finfo).PMC);
  fprintf(stderr,"PBG = %g --BackGround\n",(*finfo).PBG);
  fprintf(stderr,"PHS = %g --Smoothin\n",(*finfo).PHS);
  fprintf(stderr,"PVS = %g --Smoothin\n",(*finfo).PVS);
  fprintf(stderr,"PXC = %g --ExtraConstrain\n",(*finfo).PXC);

  fprintf(stderr,"NumVC   = %i \n",(*finfo).NumVC);
  fprintf(stderr,"FunEval = %i \n",(*finfo).tfe);

  mxn=(*finfo).grid.Nx * (*finfo).grid.Ny * (*finfo).grid.Nz * (*finfo).NumVC;

  fprintf(stderr,"mxn   = %i  ==  %i = Nx*Ny*Nz*NumVC\n",(*finfo).mxn,mxn);

  fprintf(stderr,"(Nx,Ny,Nz)=(%i,%i,%i)\n",
	  (*finfo).grid.Nx,(*finfo).grid.Ny,(*finfo).grid.Nz);
  fprintf(stderr,"(Dx,Dy,Dz)=(%f,%f,%f)\n",
	  (*finfo).grid.Dx,(*finfo).grid.Dy,(*finfo).grid.Dz);
}
//
//-------------------------------------------------------------------
//
