/* mcr_cc -- MassContinuity Residual in Cartesian Coordinates*/

#include "ng3dvar.h"

//
// Externally Visble
//
double Fmc(struct funInfo *finfo,int N, double *f,double *g);
double mc_L1norm(struct funInfo *finfo,double *x);
void fun_mf(struct funInfo *finfo, double *f,double *hmfc,double *vmf);

//
// External Functions
//
int gpe(int Nx, int Ny, int Nz, int c, int i, int j, int k);
int gp(int Ny, int Nz, int i, int j, int k);


// 
// Static Functions
//
static void fda(int l,int m,int n,
        int Nxg,int Nyg, int Nzg,float Dxg,float Dyg, float Dzg,
        int *ax, int *bx, double *cx, 
        int *ay, int *by, double *cy, 
        int *az, int *bz, double *cz,
        double *Az, double *Bz);

//-------------------------------------------------------------------
double Fmc(struct funInfo *finfo,int N, double *f,double *g){
// 
// Penalty function assocciated with the MassContinuity and its gradient
// respect to the velocity components.
//
double *rho;
int i,j,k;
double aux,SF,Norma,sum,Nobs;
int ax,bx,ay,by,az,bz;
double Az,Bz;
double cx,cy,cz;
double D,Vxa,Vxb,Vya,Vyb,Vza,Vzb;
int Nxg,Nyg,Nzg;
float Dxg,Dyg,Dzg;
int ind;
int HCB = 1; //HardCode Boundary =1 for yes, 0 for no
double cf,raz,rbz;

  //fprintf(stderr,"I'm on the Mass Contimnuity Term\n");//exit(1);

	// 
  // Find out in what grid level we are working.
  //

  // Get Grid dimensions  for current grid level
  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;
  Dxg = (*finfo).grid.Dx;
  Dyg = (*finfo).grid.Dy;
  Dzg = (*finfo).grid.Dz;
  Nobs = (double) (*finfo).rd.Nobs;

  // get density 
  rho = (*finfo).rho;

  //
  // Scaling
  //
  SF = Dxg*Dxg/Nobs;

  // Weight associatied with the penalty --scaled
  Norma = (*finfo).PMC * SF;

  sum = 0.0;
  for (i = 0; i < Nxg; i++) {
    for (j = 0; j < Nyg; j++) {
     for (k = 0; k < Nzg; k++) {

        fda(i,j,k,Nxg,Nyg,Nzg,Dxg,Dyg,Dzg,
            &ax,&bx,&cx, &ay,&by,&cy, &az,&bz,&cz, &Az,&Bz);

        Vxa = f[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)]; 
        Vxb = f[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)]; 

        Vya = f[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)]; 
        Vyb = f[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)]; 

        Vza = f[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)]; 
        Vzb = f[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)]; 

        // Mass continuity equation value at GridPoint(i,j,k);
        // cy = 0.0; // set to zero for the updraft problem
        D = rho[k] * (cx*(Vxa-Vxb) + cy*(Vya-Vyb)) + 
            cz * (Az * rho[k+az]*Vza  - Bz * rho[k+bz]*Vzb) ;

        //Modification of Penalty term 
        cf = 1.0 / rho[k]; 
        //cf = 1.0; //TEST
        D *= cf;

        sum += (D*D); 

        // 
        // apply scaling to gradient componets
        //
        //aux = D * rho[k] * Norma; // use this if cf = 1 --revert to orig.
        aux = D * Norma;
					 
        g[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)] += (aux * cx);
        g[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)] -= (aux * cx);
        //
        // The components respect to vy are commente out
        // for the updraft problem.
        //
        g[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)] += (aux * cy);
        g[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)] -= (aux * cy);

        if ( ( (k+az) == 0 || (k+az) == (Nzg -1) ) && HCB == 1 )
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)] += 0.0;
        else {
           raz = rho[k+az]/rho[k]; //CORRECTION
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)] += (aux* raz *cz * Az);
        }

        if ( ( (k+bz) == 0 || (k+bz) == (Nzg -1) ) && HCB == 1 )
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)] -= 0.0;
        else {
           rbz = rho[k+bz]/rho[k]; //CORRECTION;
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)] -= (aux * rbz * cz * Bz);
        }

     }//k-loop
   }//j-loop
  }//i-loop
  
  if (sum > DBL_MAX/2.0){
     // if this condition ever comes up, we just have to 
     // apply the normlization term by term.
     fprintf(stderr,"sum=%g ...DBLMAX exceeded!!\n",sum);
     exit(1);
  }
  //
  // Update Mas Balance
  //

  return(0.5*sum*Norma); // apply scaling to function
}
//--------------------------------------------------------------------
void fda(int l,int m,int n,
int Nxg,int Nyg, int Nzg,float Dxg,float Dyg, float Dzg,
int *ax, int *bx, double *cx, 
int *ay, int *by, double *cy, 
int *az, int *bz, double *cz,
double *Az, double *Bz){
int HCBC = 1; //=1 for hard coded Top and Bottom Boundaries. =0 otherwise.
//
// fda -- finite difference approximation.
// This function checks that the given point belongs to the grid and
// then returns the write coeficients to calculate the appropiate
// finete difference approximation to the derivatives.
// 
// D(i,j,k) = DxFx + DyFy + DzFz
// DxFx =      Fx(i+ax,j,k) -      Fx(i+bx,j,k) / (ax -bx)*dx
// DyFy =      Fy(i,j+ay,k) -      Fy(i,j+by,k) / (ay -by)*dy
// DzFz = Az * Fz(i,j,k+az) - Bz * Fz(i,j,k+bz) / (az -bz)*dy
double dx,dy,dz;

  dx = (double) Dxg;
  dy = (double) Dyg;
  dz = (double) Dzg;

  if (l < 0 || l > (Nxg-1)) {fprintf(stderr,"i OutOfBounds");exit(1);}
  if (m < 0 || m > (Nyg-1)) {fprintf(stderr,"j OutOfBounds");exit(1);}
  if (n < 0 || n > (Nzg-1)) {fprintf(stderr,"k OutOfBounds");exit(1);}
  //
  // assume center differences and modify to acomodate special cases
	//
  *ax = 1.0; *bx = -1.0; 
  if (l == 0 )     {*ax = 1.0; *bx =  0.0;} // Forward
  if (l == Nxg -1) {*ax = 0.0; *bx = -1.0;} // Backward
  *cx = ( 1.0 / ( ((float) (*ax - *bx)) * dx) );

  *ay = 1.0; *by = -1.0; 
  if (m == 0 )     {*ay = 1.0; *by =  0.0;} // Forward
  if (m == Nyg -1) {*ay = 0.0; *by = -1.0;} // Backward
  *cy = ( 1.0 / ( ((float) (*ay - *by)) * dy) );

  *az = 1.0; *bz = -1.0;
  if (n == 0 )     {*az = 1.0; *bz =  0.0;} // Forward
  if (n == Nzg -1) {*az = 0.0; *bz = -1.0;} // Backward
  *cz = ( 1.0 / ( ((float) (*az - *bz)) * dz) );

  // Assume Top and bottom boundaries are not Hard Coded
  *Az = 1.0; *Bz = 1.0;
  if (HCBC == 1) {
    if (n <   2      ) { *Az = 1.0; *Bz = 0.0; }
    if (n > (Nzg -3) ) { *Az = 0.0; *Bz = 1.0; }
  }

}
//-------------------------------------------------------------------
void fun_mf(struct funInfo *finfo, double *f,double *hmfc,double *vmf){
// 
// This function is for diagnostic purposes only. It must be called 
// after the target field has been found.
//
double *rho;
int i,j,k,p;
int pt;
double aux,Norma,sum;
int ax,bx,ay,by,az,bz;
double Az,Bz;
double cx,cy,cz;
int Nxg,Nyg,Nzg;
float Dxg,Dyg,Dzg;
double Vxa,Vxb,Vya,Vyb;


  // Get Grid Sizes
  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;
  Dxg = (*finfo).grid.Dx;
  Dyg = (*finfo).grid.Dy;
  Dzg = (*finfo).grid.Dz;

  // Set density 
  rho = (*finfo).rho;

  for (i = 0; i < Nxg; i++) {
    for (j = 0; j < Nyg; j++) {
     for (k = 0; k < Nzg; k++) {

        // Get Grid Point
        p =  gp(Nyg,Nzg, i,j,k);
        pt =  gpe(Nxg,Nyg,Nzg,0,i,j,k);
        if (p != pt) {fprintf(stderr,"calcdf_mc:p differ!!\n"); exit(1);}

        fda(i,j,k,Nxg,Nyg,Nzg,Dxg,Dyg,Dzg,
            &ax,&bx,&cx, &ay,&by,&cy, &az,&bz,&cz, &Az, &Bz);

        Vxa = f[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)]; 
        Vxb = f[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)]; 

        Vya = f[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)]; 
        Vyb = f[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)]; 

        //Horizontal Mass Flux Convergence.
        hmfc[p] = -1.0*rho[k]*(cx*(Vxa-Vxb) + cy*(Vya-Vyb)); 

       //Vertical Mass Flux
       vmf[p] = rho[k]*f[gpe(Nxg,Nyg,Nzg,2,i,j,k)];

     }//k-loop
   }//j-loop
  }//i-loop
}
//--------------------------------------------------------------------
double mc_L1norm(struct funInfo *finfo,double *x){
//
// Criteria for satisfaction of the Mass continuity equation
// using L1norm. This criteria is not used during the uncostrained
// minimization. It is used in the loop that deal with the constrained
// minimization. i.e. the outer loop.
//
int i,j,k;
int ax,bx,ay,by,az,bz;
double Az,Bz;
double cx,cy,cz;
double D,Vxa,Vxb,Vya,Vyb,Vza,Vzb;
int Nxg,Nyg,Nzg;
float Dxg,Dyg,Dzg;
double *rho;

double Max;

  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;
  Dxg = (*finfo).grid.Dx;
  Dyg = (*finfo).grid.Dy;
  Dzg = (*finfo).grid.Dz;
  rho = (*finfo).rho;


  Max = 0.0;
  for (i = 0; i < Nxg; i++) {
     for (j = 0; j < Nyg; j++) {
        for (k = 0; k < Nzg; k++) {

        fda(i,j,k,Nxg,Nyg,Nzg,Dxg,Dyg,Dzg,
            &ax,&bx,&cx, &ay,&by,&cy, &az,&bz,&cz,&Az,&Bz);

        Vxa = x[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)]; 
        Vxb = x[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)]; 

        Vya = x[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)]; 
        Vyb = x[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)]; 

        Vza = x[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)]; 
        Vzb = x[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)]; 

        // Mass continuity equation value at GridPoint(i,j,k);
        D = rho[k] * (cx*(Vxa-Vxb) + cy*(Vya-Vyb)) + 
            cz * (Az * rho[k+az]*Vza  - Bz * rho[k+bz]*Vzb) ;

        D = fabs(D);
        if (D > Max) Max = D;
     }
   }
  }
 //fprintf(stderr,"MaxMC = %g\n",Max);
 return(Max);
}
