/* mcr_sc -- MassContinuity Residual in Spherical Coordinates. */

#include "ng3dvar.h"

//
// Externally Visble
//
double FmcSC(struct funInfo *finfo,int N, double *f,double *g);
double scmc_L1norm(struct funInfo *finfo,double *x);

//
// External Functions
//
int gpe(int Nx, int Ny, int Nz, int c, int i, int j, int k);


// 
// Static Functions
//
static void fdaO(int l,int m,int n,
        int Nxg,int Nyg, int Nzg,float Dxg,float Dyg, float Dzg,
        int *ax, int *bx, double *cx, 
        int *ay, int *by, double *cy, 
        int *az, int *bz, double *cz,
        double *Az, double *Bz);

static void fda(int l,int m,int n, int Nxg,int Nyg, int Nzg,
                int *ax, int *bx, int *ay, int *by, int *az, int *bz,
                double *Az, double *Bz);

//-------------------------------------------------------------------
double FmcSC(struct funInfo *finfo,int N, double *f,double *g){
// 
// Penalty function assocciated with the MassContinuity in 
// Spherical coordinates and its gradient respect to the 
// velocity components.
//
int i,j,k;
int ax,bx,ay,by,az,bz;
int Nxg,Nyg,Nzg;
int HCB = 1;       //HardCode Boundary =1 for yes, 0 for no

double *rho;
double aux,SF,Norma,sum,Nobs;
double Az,Bz;
double cx,cy,cz;
double D,Vxa,Vxb,Vya,Vyb,Vyc,Vza,Vzb;
double Dxg,Dyg,Dzg;
double cf,raz,rbz;
double y0,radi,radi2,alpha,rhok,lamda,lamdaP,lamdaM,radiP,radiM;
double XA,XB,YA,YB,YC,ZA,ZB;


  // Get Grid dimensions  for current grid level
  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;

  Dxg = (double) (*finfo).grid.Dx;
  Dyg = (double) (*finfo).grid.Dy;
  Dzg = (double) (*finfo).grid.Dz;
	y0 =  (double) (*finfo).lat0;
	Nobs = (double) (*finfo).rd.Nobs;

	//fprintf(stderr,"Dxg=%g\tDyg=%g\tDzg=%g\n",Dxg,Dyg,Dzg);

  // angles must be in radians;
  Dxg *= D2R;
  Dyg *= D2R;
  y0  *= D2R;

  // Get density 
  rho = (*finfo).rho;

  //
  // Scaling
  //
  //SF = 1.0;
  SF = Dxg*Dxg/Nobs;

  // Weight associatied with the penalty --scaled
  Norma = (*finfo).PMC * SF;

  sum = 0.0;
  for (k = 0; k < Nzg; k++) {
     radi = EARTH_R + (double) k * Dzg;
     radi2 = radi*radi;
     rhok = rho[k];

     for (j = 0; j < Nyg; j++) {
        lamda  = y0 + (double) j * Dyg;
        alpha  = rhok/(radi * cos(lamda));

        for (i = 0; i < Nxg; i++) {

           //fda(i,j,k,Nxg,Nyg,Nzg,Dxg,Dyg,Dzg,
           //   &ax,&bx,&cx, &ay,&by,&cy, &az,&bz,&cz, &Az,&Bz);

           fda(i,j,k,Nxg,Nyg,Nzg,&ax,&bx,&ay,&by,&az,&bz,&Az,&Bz);
           cx = ((double) (ax - bx)) * Dxg; 
           cy = ((double) (ay - by)) * Dyg; 
           cz = ((double) (az - bz)) * Dzg;
           
           lamdaP = lamda + (double) ay * Dyg;
           lamdaM = lamda + (double) by * Dyg;

           radiP  = radi  + (double) az * Dzg;
           radiP  *= radiP;
           radiP  /= radi2;

           radiM  = radi  + (double) bz * Dzg;
           radiM  *= radiM;
           radiM  /= radi2;

           XA = alpha/cx;                  XB = -XA;
           YA = alpha*cos(lamdaP)/cy;      YB = -alpha*cos(lamdaM)/cy; 
           ZA = radiP*rho[k+az]/cz;        ZB = -radiM*rho[k+bz]/cz;

           Vxa = f[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)]; 
           Vxb = f[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)]; 

           Vya = f[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)]; 
           Vyb = f[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)]; 

           Vza = f[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)]; 
           Vzb = f[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)]; 

           // Mass continuity equation value at GridPoint(i,j,k);
           D = XA*Vxa + XB*Vxb + YA*Vya + YB*Vyb + ZA*Vza + ZB*Vzb;

           //Modification of Penalty term 
           cf = 1.0 / rhok; 
           D *= cf;

           sum += (D*D); 

           // 
           // apply scaling to gradient componets
           //
           aux = D * Norma / rhok;
					 
        g[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)] += (aux * XA);
        g[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)] += (aux * XB);
        //
        // The components respect to vy are commente out
        // for the updraft problem.
        //
        g[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)] += (aux * YA);
        g[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)] += (aux * YB);

        if ( ( (k+az) == 0 || (k+az) == (Nzg -1) ) && HCB == 1 )
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)] += 0.0;
        else {
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)] += (aux * Az * ZA);
        }

        if ( ( (k+bz) == 0 || (k+bz) == (Nzg -1) ) && HCB == 1 )
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)] += 0.0;
        else {
           g[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)] += (aux * Bz * ZB);
        }

     }//i-loop
   }//j-loop
  }//k-loop
  
  if (sum > DBL_MAX/2.0){
     // if this condition ever comes up, we just have to 
     // apply the normlization term by term.
     fprintf(stderr,"sum=%g ...DBLMAX exceeded!!\n",sum);
     exit(1);
  }

  return(0.5*sum*Norma); // apply scaling to function
}
//-------------------------------------------------------------------
double scmc_L1norm(struct funInfo *finfo,double *x){
//
// Criteria for satisfaction of the Mass continuity equation
// using L1norm. This criteria is not used during the uncostrained
// minimization. It is used in the loop that deal with the constrained
// minimization. i.e. the outer loop.
//
int i,j,k;
int ax,bx,ay,by,az,bz;
int Nxg,Nyg,Nzg;

double *rho;
double Az,Bz;
double cx,cy,cz;
double D,Vxa,Vxb,Vya,Vyc,Vyb,Vza,Vzb;
double Dxg,Dyg,Dzg;
double y0,radi,radi2,alpha,rhok,lamda,lamdaP,lamdaM,radiP,radiM;
double XA,YA,ZA;
double XB,YB,ZB;
double Max;

  Nxg = (*finfo).grid.Nx;
  Nyg = (*finfo).grid.Ny;
  Nzg = (*finfo).grid.Nz;

  Dxg = (double) (*finfo).grid.Dx;
  Dyg = (double) (*finfo).grid.Dy;
  Dzg = (double) (*finfo).grid.Dz;
	y0  = (double) (*finfo).lat0;

  // angles must be in radians;
  Dxg *= D2R;
  Dyg *= D2R;
  y0  *= D2R;

  rho = (*finfo).rho;

  Max = 0.0;
  for (k = 0; k < Nzg; k++) {
     radi = EARTH_R + (double) k * Dzg;
     radi2 = radi*radi;
     rhok = rho[k];

     for (j = 0; j < Nyg; j++) {
        lamda  = y0 + (double) j * Dyg;
        alpha  = rhok/(radi * cos(lamda));

        for (i = 0; i < Nxg; i++) {

           //fda(i,j,k,Nxg,Nyg,Nzg,Dxg,Dyg,Dzg,
           //    &ax,&bx,&cx, &ay,&by,&cy, &az,&bz,&cz, &Az,&Bz);

           fda(i,j,k,Nxg,Nyg,Nzg,&ax,&bx,&ay,&by,&az,&bz,&Az,&Bz);
           cx = (double) (ax - bx) * Dxg; 
           cy = (double) (ay - by) * Dyg; 
           cz = (double) (az - bz) * Dzg;

           lamdaP = lamda + (double) ay * Dyg;
           lamdaM = lamda + (double) by * Dyg;

           radiP  = radi  + (double) az * Dzg;
           radiP  *= radiP;
           radiP  /= radi2;

           radiM  = radi  + (double) bz * Dzg;
           radiM  *= radiM;
           radiM  /= radi2;

           XA = alpha/cx;                  XB = -XA;
           YA = alpha*cos(lamdaP)/cy;      YB = -alpha*cos(lamdaM)/cy; 
           ZA = radiP*rho[k+az]/cz;        ZB = -radiM*rho[k+bz]/cz;

           Vxa = x[gpe(Nxg,Nyg,Nzg,0,i+ax,j,k)]; 
           Vxb = x[gpe(Nxg,Nyg,Nzg,0,i+bx,j,k)]; 

           Vya = x[gpe(Nxg,Nyg,Nzg,1,i,j+ay,k)]; 
           Vyb = x[gpe(Nxg,Nyg,Nzg,1,i,j+by,k)]; 

           Vza = x[gpe(Nxg,Nyg,Nzg,2,i,j,k+az)]; 
           Vzb = x[gpe(Nxg,Nyg,Nzg,2,i,j,k+bz)]; 

          // Mass continuity equation value at GridPoint(i,j,k);
          D = XA*Vxa + XB*Vxb + YA*Vya + YB*Vyb + ZA*Vza + ZB*Vzb;

          D = fabs(D);
          if (D > Max) Max = D;
     }
   }
  }
 //fprintf(stderr,"MaxMC = %g\n",Max);
 return(Max);
}
//--------------------------------------------------------------------
void fdaO(int l,int m,int n,
int Nxg,int Nyg, int Nzg,float Dxg,float Dyg, float Dzg,
int *ax, int *bx, double *cx, 
int *ay, int *by, double *cy, 
int *az, int *bz, double *cz,
double *Az, double *Bz){
int HCBC = 1; //=1 for hard coded Top and Bottom Boundaries. =0 otherwise.
//
// fda -- finite difference approximation.
// This function checks that the given point belongs to the grid and
// then returns the write coeficients to calculate the appropiate
// finete difference approximation to the derivatives.
// 
// D(i,j,k) = DxFx + DyFy + DzFz
// DxFx =      Fx(i+ax,j,k) -      Fx(i+bx,j,k) / (ax -bx)*dx
// DyFy =      Fy(i,j+ay,k) -      Fy(i,j+by,k) / (ay -by)*dy
// DzFz = Az * Fz(i,j,k+az) - Bz * Fz(i,j,k+bz) / (az -bz)*dz
double dx,dy,dz;

  dx = (double) Dxg;
  dy = (double) Dyg;
  dz = (double) Dzg;

  if (l < 0 || l > (Nxg-1)) {fprintf(stderr,"i OutOfBounds");exit(1);}
  if (m < 0 || m > (Nyg-1)) {fprintf(stderr,"j OutOfBounds");exit(1);}
  if (n < 0 || n > (Nzg-1)) {fprintf(stderr,"k OutOfBounds");exit(1);}
  //
  // assume center differences and modify to acomodate special cases
	//
  *ax = 1.0; *bx = -1.0; 
  if (l == 0 )     {*ax = 1.0; *bx =  0.0;} // Forward
  if (l == Nxg -1) {*ax = 0.0; *bx = -1.0;} // Backward
  *cx = ( 1.0 / ( ((float) (*ax - *bx)) * dx) );

  *ay = 1.0; *by = -1.0; 
  if (m == 0 )     {*ay = 1.0; *by =  0.0;} // Forward
  if (m == Nyg -1) {*ay = 0.0; *by = -1.0;} // Backward
  *cy = ( 1.0 / ( ((float) (*ay - *by)) * dy) );

  *az = 1.0; *bz = -1.0;
  if (n == 0 )     {*az = 1.0; *bz =  0.0;} // Forward
  if (n == Nzg -1) {*az = 0.0; *bz = -1.0;} // Backward
  *cz = ( 1.0 / ( ((float) (*az - *bz)) * dz) );

  // Assume Top and bottom boundaries are not Hard Coded
  *Az = 1.0; *Bz = 1.0;
  if (HCBC == 1) {
    if (n <   2      ) { *Az = 1.0; *Bz = 0.0; }
    if (n > (Nzg -3) ) { *Az = 0.0; *Bz = 1.0; }
  }

}
//--------------------------------------------------------------------
void fda(int l,int m,int n, int Nxg,int Nyg, int Nzg,
int *ax, int *bx, int *ay, int *by, int *az, int *bz,
double *Az, double *Bz){
int HCBC = 1; //=1 for hard coded Top and Bottom Boundaries. =0 otherwise.
//
// fda -- finite difference approximation.
// This function checks that the given point belongs to the grid and
// then returns the appropiate coeficients to calculate the appropiate
// finete difference approximation to the derivatives.
// 
// D(i,j,k) = DxFx + DyFy + DzFz
// DxFx =      Fx(i+ax,j,k) -      Fx(i+bx,j,k) / (ax -bx)*dx
// DyFy =      Fy(i,j+ay,k) -      Fy(i,j+by,k) / (ay -by)*dy
// DzFz = Az * Fz(i,j,k+az) - Bz * Fz(i,j,k+bz) / (az -bz)*dz
double dx,dy,dz;

  if (l < 0 || l > (Nxg-1)) {fprintf(stderr,"i OutOfBounds");exit(1);}
  if (m < 0 || m > (Nyg-1)) {fprintf(stderr,"j OutOfBounds");exit(1);}
  if (n < 0 || n > (Nzg-1)) {fprintf(stderr,"k OutOfBounds");exit(1);}
  //
  // assume center differences and modify to acomodate special cases
	//
  *ax = 1; *bx = -1; 
  if (l == 0 )     {*ax = 1; *bx =  0;} // Forward
  if (l == Nxg -1) {*ax = 0; *bx = -1;} // Backward

  *ay = 1; *by = -1; 
  if (m == 0 )     {*ay = 1; *by =  0;} // Forward
  if (m == Nyg -1) {*ay = 0; *by = -1;} // Backward

  *az = 1; *bz = -1;
  if (n == 0 )     {*az = 1; *bz =  0;} // Forward
  if (n == Nzg -1) {*az = 0; *bz = -1;} // Backward

  // Assume Top and bottom boundaries are not Hard Coded
  *Az = 1.0; *Bz = 1.0;
  if (HCBC == 1) {
    if (n <   2      ) { *Az = 1.0; *Bz = 0.0; }
    if (n > (Nzg -3) ) { *Az = 0.0; *Bz = 1.0; }
  }

}
