// wtcalc -- calculate particle terminal velocity from height and
// reflectivity
#include "stdlib.h"
#include "math.h"
#include "stdio.h"

#define BOTT_SNOW 4.8  // height of bottom of snow layer
#define TOP_RAIN 4.3   // height of top of rain layer

//
// This function is used by fun_wtZ to estimate the particle vertical
// velocity from reflectivity data.
//
double fun_rhoFactor(double z)
{
  double TMSL=300.0;
  double PMSL=100.0;
  double GAMMA=7.0;
  double RGAS=287.6;
  double GRAV=9.8;
  double rhoF,rho,z1,mu,rhomsl,mum1,dzg,temp,term1;
  
  z1     = TMSL/GAMMA;
  mu     = GRAV/(RGAS*GAMMA*0.001);
  rhomsl = 1000.0 * PMSL/(RGAS*TMSL);
  mum1   = mu -1.0;
  
  rho = rhomsl*pow(1.0 - (z/z1),mum1);

  temp = TMSL - GAMMA * z;
  if (rho != 0.0) {
    term1 = pow(10.0,(0.43*log10(rhomsl/rho) -
		      0.4*pow(log10(rhomsl/rho),2.5)));
    rhoF = (1.+0.0023*(1.1-(rho/rhomsl))*(TMSL-temp))*term1;
  } else rhoF = 0.0; // a consistent choice with radvert4.
  
  return(rhoF);
}

double wtcalc(double z, double dbz)
{
  double R, S, rhos, refl, Zw, Zs, vts, vtw;
       
  // Prepare for linear interpolation
  if (z >= BOTT_SNOW)   { R = 0.0; S = 1.0;}
  else {
    if (z <= TOP_RAIN) { R = 1.0; S = 0.0;}
    else {
      R = (BOTT_SNOW - z)/(BOTT_SNOW - TOP_RAIN);
      S = (z  - TOP_RAIN)/(BOTT_SNOW - TOP_RAIN);
    }
  }
  
  //
  // Get the density factor at the data altitude z.
  //
  rhos = fun_rhoFactor(z); 
  
  // get reflectivity from reflectivity factor
  refl = pow(10.,(dbz/10.));
  Zw = refl;
  
  // SHOULD THERE BE A SEPARATE Zs?
  Zs = 4.72*refl;

  //
  // calculate terminal velocity for snow
  vts = rhos*0.817*pow(Zs,0.063);
  
  //
  // Joss and Waldvolgel (1970) formula
  //
  vtw = rhos*2.6*pow(Zw,0.107);
  
  // linearly interp snow and rain terminal velocities between
  // TOP_RAIN and BOTT_SNOW heights -- directed downwards.
  //
  return( -1.0*(R * vtw +  S * vts));
}
