// ng3dvar_tf_bgf -- generates contribution to target function
//                   from the missfit with the background field.
//
// Carlos Lopez Carrillo
// Geophysical Research Center, NMT.
//

#include "ngsonde_tf_misfit.h"
#include "ngutil.h"
#include "cdfhdr.h"
#include "nggfit.h"
#include <math.h>
#include <stdio.h>

#define USAGE "should be called by function calcfg inside calcfg_if.c"

/* assumed names for Data Info */

//
// Internal Functions.
//
double fun_bgf(struct funInfo *finfo,
				  double *vel,double *cgf);

//
// Return value -- contribution to the function from radar gridded data
// funInfo -- structure  defined in mgm.h and populated in mgm_user_if.c
// f -- 1D vector that holds the velocity field on the grid.
// g -- 1D vector that holds the contribution from radar data to 
//      the gradient with respect to the velocity field components
//      of the function being minimized. 
// N -- is the size of the vectors f and g
//
double if_bgf(struct funInfo *finfo,int N,double *vel,double *g)
{
	double cf;

	cf = fun_bgf(finfo,vel,g);

	return(cf); 
}
//-------------------------------------------------------------------
double fun_bgf(struct funInfo *finfo, double *vel,double *cgf){

	int ig,jg,kg,pg; //,pgpNp,pgp2Np;
	int Nxg,Nyg,Nzg; //,Np;

	double bad,badlim;
	double value, svalue;
	double missfit,sum;
	double ZIP;
	double bgfW;

  //
  // HERE
  //
	ngFitInfo *ginfo = (ngFitInfo*)finfo->userData;


	Nxg = ginfo->nx; //(*finfo).grid.Nx; 
	Nyg = ginfo->ny; //(*finfo).grid.Ny;
	Nzg = ginfo->nz; //(*finfo).grid.Nz;

	bad = (double)ginfo->bad;
	badlim = (double)ginfo->badlim;
	bgfW = (double)ginfo->bgfW;

	sum   = 0.0;

	// loop on Data Grid
	for (kg = 0; kg < Nzg; kg++) {

		ZIP = 1.0; if ( (kg == 0) || (kg == (Nzg -1)) ) ZIP = 0.0; //HCB

		for (jg = 0; jg < Nyg; jg++) {
			for (ig = 0; ig < Nxg; ig++) {
       
				pg = gp(Nyg, Nzg, ig, jg, kg);
				value = ginfo->bgf[pg];


				if ( fabs(value) < badlim) {
               
					svalue = vel[pg];

					//
					// weighted contribution to missfit.
					//
					missfit = (value - svalue) * (value - svalue);


					//
					// make ppropiate contributions to the gradient. 
					//

					cgf[pg] += ((svalue - value) * bgfW);


					// contribution to functional.
					sum   += missfit;

				} //good data
			} //i-loop
		} //j-loop
	} //k-loop

	return((double) 0.5 * bgfW * sum); 
  
}
