#include "ngcmdargs.h"
#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>
#include <string.h>
#include <math.h>

void ngParseArgs(int argc, char** argv, ngCMDArgs* dest, char LA[][CLINE], int *LAcnt)
{
  int la = 0;
  ngCalcField* field = NULL;
  int optionIndex = 0;
  int c;
  static struct option longOptions[] = 
    {
      {"nfe", required_argument, 0, 'n'},
      {"phs", required_argument, 0, 'p'},
      {"pvs", required_argument, 0, 'q'},
      {"gph", required_argument, 0, 'g'},
      {"idx", required_argument, 0, 'i'},
      {"lon", required_argument, 0, 'o'},
      {"lat", required_argument, 0, 'a'},
      {"alt", required_argument, 0, 'l'},
      //{"st", required_argument, 0, 's'},
      {NULL, 0, NULL, 0}
    };


  dest->gfitTries = 100000;
  dest->lon = NULL;
  dest->lat = NULL;
  dest->pres = NULL; //pressure --default index
  dest->gph = NULL; //Geopotential Height name
  dest->idx = NULL; //Index name
  dest->phs = 0;
  dest->pvs = 0;
  dest->svx = 0;
  dest->svy = 0;
  dest->reftime = 0;
  dest->fields = NULL;
  dest->numFields = 0;

  /* parse named arguments. */
  char usage[200];
  sprintf(usage,"ngsonde [-h] [--nfe val] [--phs val] [--pvs val] [-s svx:svy:reftime] [--idx name] [--gph name] --lon lon0:dlon:Nlon --lat lat0:dlat:Nlat --alt alt0:dalt:Nalt   var1 var2 ...  < stdin > stdout \n");

  if (argc < 8) {
    fprintf(stderr,"%s\n",usage); 
    exit(1);
  }

  while((c = getopt_long(argc, argv, "n:p:q:s:g:i:o:a:l:h", longOptions, &optionIndex)) != -1)
    {
      switch(c)
	{
	case 'h':
	  fprintf(stderr, "%s",usage); exit(1);
	  break;
	case 'n': /* the user wants to set gfitTries */
	  if(sscanf(optarg,"%i", &dest->gfitTries) != 1 || dest->gfitTries < 0){
	    fprintf(stderr, "ngsonde: --nfe val --positive integer\n"); exit(1);
	  }
	  strcpy(LA[la],"--nfe ");
	  strcat(LA[la++],optarg);
	  break;
	case 'p': /* the user has specified horizontal smoothing */
	  if(sscanf(optarg, "%f", &dest->phs) != 1) {
	    fprintf(stderr, "--phs val'\n"); exit(1);
	  }
	  strcpy(LA[la],"--phs ");
	  strcat(LA[la++],optarg);
	  break;
	case 'q': /* the user has specified vertical smoothing */
	  if(sscanf(optarg, "%f", &dest->pvs) != 1) {
	    fprintf(stderr, "--pvs val'\n"); exit(1);
	  }
	  strcpy(LA[la],"--pvs ");
	  strcat(LA[la++],optarg);
	  break;
	case 'g': /* the user wants to set geopotential height. */
	  strcpy(LA[la],"--gph ");
	  strcat(LA[la++],optarg);
	  dest->gph = malloc(sizeof(char) * strlen(optarg) + 1);
	  strcpy(dest->gph, optarg);

	  break;
	case 'i': /* the user wants to set the index field. */
	  strcpy(LA[la],"--idx ");
	  strcat(LA[la++],optarg);
	  dest->idx = malloc(sizeof(char) * strlen(optarg) + 1);
	  strcpy(dest->idx, optarg);

	  break;
	case 'o': /* the user wants to set longitude. */
	  if(sscanf(optarg, "%f:%f:%i", &dest->sx, &dest->dx, &dest->nx) != 3){
	    fprintf(stderr, "ngsonde: --lon lon0:dlon:Nlon\n"); exit(1);
	  }
	  strcpy(LA[la],"--lon ");
	  strcat(LA[la++],optarg);
	  dest->lon = malloc(sizeof(char) * 4); //Assumed name
	  strcpy(dest->lon, "lon");
	  break;
	case 'a': /* the user wants to set latitude. */
	  if(sscanf(optarg, "%f:%f:%i", &dest->sy, &dest->dy, &dest->ny) != 3){
	    fprintf(stderr, "ngsonde: --lat lat0:dlat:Nlat\n"); exit(1);
	  } 
	  strcpy(LA[la],"--lat ");
	  strcat(LA[la++],optarg);
	  dest->lat = malloc(sizeof(char) * 4); //Assumed name
	  strcpy(dest->lat, "lat");
	  break;
	case 'l': /* the user wants to set altitude. */
	  if(sscanf(optarg, "%f:%f:%i", &dest->sz, &dest->dz, &dest->nz) != 3){
	    fprintf(stderr, "ngsonde: --alt alt0:dalt:Nalt\n"); exit(1);
	  } 
	  strcpy(LA[la],"--alt ");
	  strcat(LA[la++],optarg);
	  dest->alt = malloc(sizeof(char) * 4); //Assumed name
	  strcpy(dest->alt, "z");
	  break;
	case 's': /* the user has specified the storm velocity */
	  if(sscanf(optarg, "%f:%f:%f", &dest->svx, &dest->svy, &dest->reftime) != 3) {
	    fprintf(stderr, "- s svx:svy:reftime'\n");
	    exit(1);
	  }
	  strcpy(LA[la],"-s ");
	  strcat(LA[la++],optarg);
	  break;
	default:
	  fprintf(stderr, "%s",usage); exit(1);
	  break;
	}
    }
  // Add time for compatibility 
  dest->time = malloc(sizeof(char) * 10); //Assumed name
  strcpy(dest->time, "monotime");

  // If index was not given assume pressure is the index by default.
  if (dest->idx == NULL) {
    dest->idx = malloc(sizeof(char) * 10); 
    strcpy(dest->idx, "pres");  //Assumed name

  } 
  //else { //index is other than pres, so check if gph is given.
  //Regardless of who is the index, we need to know if pressure
  // is needed to get the altitude.
  if ( dest->gph == NULL) {
    // In this case, We need the pressure to get altitude
    // -- this is the default.
    dest->pres = malloc(sizeof(char) * 10); 
    strcpy(dest->pres, "pres");  //Assumed name
  }
  //}

  /* parse positional arguments. */
  for(; optind < argc; optind++) {

    strcpy(LA[la++],argv[optind]);

    dest->numFields++;
    dest->fields = realloc(dest->fields,sizeof(ngCMDArgs) * dest->numFields);
    field = &dest->fields[dest->numFields - 1];
		
    field->inName = malloc(sizeof(char) * (strlen(argv[optind]) + 1));
    strcpy(field->inName, argv[optind]);

    //
    // ->outName, ->hsmooth, ->vsmooth are here for compatibility.
    //
    field->outName = malloc(sizeof(char) * (strlen(argv[optind]) + 1));
    strcpy(field->outName, argv[optind]);
    field->hsmooth = dest->phs;
    field->vsmooth = dest->pvs;

  }

  *LAcnt = la;

}

void ngFreeCMDArgs(ngCMDArgs* args)
{
  unsigned int i;

  free(args->lon);
  free(args->lat);
  free(args->idx);
  free(args->time);
  if (args->gph  != NULL) free(args->gph);
  if (args->pres != NULL) free(args->pres);
  if(args->fields)
    {
      for(i = 0; i < args->numFields; i++)
	{
	  ngFreeCalcField(args->fields + i);
	}

      free(args->fields);
      args->fields = 0;
      args->numFields = 0;
    }
}

void ngFreeCalcField(ngCalcField* field)
{

  free(field->inName);
  free(field->outName);
}

/**
 * If value is 0 then msg will be printed and exit(1) will be called.
 */
void ngAssert(int value, const char* msg)
{
  if(!value)
    {
      fprintf(stderr, "%s\n", msg);
      exit(1);
    }
}

/*
 * Same as ngAssert but negates value.
 */
void ngAssertF(int value, const char* msg)
{
  ngAssert(value == 0, msg);
}

/**
 * Builds an optional paramter, used by TestCMDArgs.
 */
void ngBuildOptParam(char* dest, const char* inName, const char* outName, float hsmooth, float vsmooth)
{
  sprintf(dest, "%s,%s,%f,%f", inName, outName, hsmooth, vsmooth);
}

void ngCheckField(const ngCalcField* field, const char* fldName, const char* inName, const char* outName, float hsmooth,
		  float vsmooth)
{
  int error = 0;
  if(strcmp(inName, field->inName) != 0)
    {
      fprintf(stderr, "For the %s field inName got '%s' expected '%s'\n", fldName, field->inName, inName);
      error = 1;
    }

  if(strcmp(outName, field->outName) != 0)
    {
      fprintf(stderr, "For the %s field outName got '%s' expected '%s'\n", fldName, field->outName, outName);
      error = 1;
    }

  if(field->hsmooth != hsmooth)
    {
      fprintf(stderr, "For the %s field hsmooth got %f expected %f\n", fldName, field->hsmooth, hsmooth);
      error = 1;
    }

  if(field->vsmooth != vsmooth)
    {
      fprintf(stderr, "For the %s field vsmooth got %f expected %f\n", fldName, field->vsmooth, vsmooth);
      error = 1;
    }

  if(error)
    {
      exit(1);
    }
}
