#ifndef NG_CMDARGS_H_INCLUDED
#define NG_CMDARGS_H_INCLUDED

#include <string.h>

#define CLINE 80
/**
 * A field that will be processed.
 */
typedef struct _ngCalcField
{
	char* inName;  /**< The name of the field in the input file. */
	char* outName; /**< The name of the field in the output file. */
	float hsmooth; /**< The horizontal smoothing factor for the global fit. */
	float vsmooth; /**< The vertical smoothing factor for the global fit. */
} ngCalcField;

/**
 * Contains parsed commandline arguments.
 */
typedef struct _ngCMDArgs
{
	int gfitTries;  /* Number of iterations for the conmin algorithm */
	float svx, svy; /* Storm vx and vy */
	float reftime; /* Storm reference time */
	float phs, pvs; // smoothing parameters
	float sx, sy, sz; /* grid start x, y and z */
	float dx, dy, dz; /* grid delta x, y, and z */
	int nx, ny, nz; /* number of grid points in the x, y, and z directions */
	char *gph;  // geopotential height
	char *idx;  //  index for the individual souldings.
	char *lon;   /**< Name of the input longitude field. */
	char *lat;  /**< Name of the input lattitude field. */
	char *alt;  // vertical grid axis-- default name z:compatible with ng3dvar
	char *pres; /**< Name of the input pressure field. Used to determine the z value. */
	char *time; /**< Name of the input time field. */
	ngCalcField *fields; /**< An array of length numFields containing fields to process. */
	unsigned int numFields; /**< The number of elements in CalcFields. */
} ngCMDArgs;

/**
 * Parse the commandline arguments.
 * On failure an error message will be printed and exit(1) will be called.
 *
 * @param argc Length of argv (as passed to main).
 * @param argv Argument strings (as passed to main).
 * @param dest Location to place the results, will be overwritten.
 */
void ngParseArgs(int argc, char** argv, ngCMDArgs* dest, char LA[][CLINE], int *LAcnt);

/**
 * Frees all internal memory for a CMDArgs structure.
 * free must still be called on the structure it's self.
 *
 * @args The args to free internal memory for.
 */
void ngFreeCMDArgs(ngCMDArgs* args);

/**
 * Frees all internal memory for a CalcField structure.
 * free must still be called on the structure it's self.
 *
 * @param field The CalcField to free interal memory for.
 */
void ngFreeCalcField(ngCalcField* field);

/**
 * Runs tests on ParseArgs, FreeCMDArgs, and FreeCaclField.
 * On failure an error message will be printed.
 */
void ngTestCMDArgs();

#endif
