#include "nggfit.h"
#include "cgm_conmin.h"
#include "ng3dvar_tf_smootht.h"
#include "ngsonde_tf_misfit.h"
#include "ng3dvar_tf_bgf.h"
#include "ngutil.h"
#include <cdfhdr.h>
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <stdbool.h>

//
//-------------------------------------------------------------------
//
static int calcfg(funInfo *finfo,int N, double *x, double *f, double *g);
void fun_cspline(double badlim, double dxx, int n, double *xx, double *f);
void fun_ini(ngFitInfo *ginfo, double *x, double *have);

//
//-------------------------------------------------------------------
//
/* this allocates and frees space for double precision arrays */
double *grid_alloc(int n)
{
  double *fld;

  if ((fld = (double*)malloc(sizeof(double)*n)) ==
      (double*)NULL) {
    fprintf(stderr,"Grid: Can't get space for n %d\n",n);
    exit(1);
  }
  return(fld);
}
//
//-------------------------------------------------------------------
//
/* this allocates and frees space for arrays of integers*/
int *int_alloc(int n)
{
  int *fld;

  if ((fld = (int*)malloc(sizeof(int)*n)) == (int*)NULL) {
    fprintf(stderr,"int_alloc: Can't get space\n");
    exit(1);
  }
  return(fld);
}
//
//-------------------------------------------------------------------
//
void grid_free(double *fld)
{
  free(fld);
  fld = (double*)NULL;
}
//
//-------------------------------------------------------------------
//
void int_free(int *fld)
{
  free(fld);
  fld = (int*)NULL;
}
//
//-------------------------------------------------------------------
//
void ngGlobalFit(ngCMDArgs *args, unsigned int findex, float *data, float bad, float badlim)
{
  funInfo finfo;
  ngFitInfo ginfo;
  int ix,jy,kz,pg;
  int n, i;
  double *x, *f, *g, *auxG, *origData, *bgf, *aProf;
  double bgfW = -1; // weight for the backgraund field.
                    // larger values acentuate the soundings.

  // allocate space for arrays -- fix by D. J. Raymond --
  // previously automatic allocation was used, resulting
  // in seg fault for big files, probably because of
  // a stack overflow
  n = args->nx * args->ny * args->nz;
  x = grid_alloc(n);
  f = grid_alloc(n);
  g = grid_alloc(n);
  auxG = grid_alloc(n);
  origData = grid_alloc(n);
  bgf = grid_alloc(n);
  aProf = grid_alloc(args->nz);

  //init parameters to cgmconmin
  for(i = 0; i < n; i++)
    {
      x[i] = (double)data[i];
      origData[i] = x[i];
    }
  ginfo.nx = args->nx;
  ginfo.ny = args->ny;
  ginfo.nz = args->nz;
  ginfo.dx = args->dx;
  ginfo.dy = args->dy;
  ginfo.dz = args->dz;
  ginfo.PHS = args->fields[findex].hsmooth;
  ginfo.PVS = args->fields[findex].vsmooth;
  ginfo.data = origData;
  ginfo.bgf  = bgf;
  ginfo.auxG = auxG;
  ginfo.bgfW = bgfW; 
  ginfo.bad = bad; //BAD;
  ginfo.badlim = badlim; //BADLIM;

  finfo.userData = &ginfo;
  finfo.CC.g = 0;
  finfo.CC.x = 0;
  finfo.CC.f = 0;
  finfo.CC.c = 0;
  finfo.cgi.MXFUN = args->gfitTries;
  finfo.cgi.eps = sqrt(FLT_EPSILON);
  finfo.cgi.acc = FLT_EPSILON * 100;

  

  //
  // Do initialization
  //
  fun_ini(&ginfo,x,aProf);
  for ( kz = 0; kz < ginfo.nz ; kz++) {
    for (jy = 0; jy < ginfo.ny ; jy++) {
      for (ix = 0; ix < ginfo.nx ; ix++) {
	pg = gp(ginfo.ny,ginfo.nz,ix,jy,kz);
	x[pg] = aProf[kz];
	bgf[pg] = x[pg];
      }
    }
  }
  // fill data voids with average: this is different from
  // only initializing with the average.  
  // This variation, strongly concentratesthe influence of
  // the actual data to the sounding locations. The results are
  // not optimeal. I think what we need is a background field
  // constricted to be the average of the soundings, but with
  // smaller weigths than the soundings, so it only kicks in
  // in data void regions.
  //for(i = 0; i < n; i++) { 
  //		if (origData[i] >= badlim) origData[i] = x[i];
  //}
  free(aProf);

  // Done with initialization

  // Let's search
  cg_CONMINc(n, x, f, g, &finfo, &calcfg);
  //finfo.CC.c = 1; //for testing

  //TODO should we copy back over if c is false?
  for(i = 0; i < n; i++) {
    data[i] = (float)x[i];
  }
	
  if(!finfo.CC.c) {
    fprintf(stderr, "ngsonde: Convergance criteria not met for %s\n", args->fields[findex].outName);
    fprintf(stderr, "ngsonde: CC.g=%g CC.x=%g CC.f=%g CC.c=%g\n\n", finfo.CC.g, finfo.CC.x, finfo.CC.f, finfo.CC.c);
  }

  // free the initial arrays
  grid_free(bgf);
  grid_free(origData);
  grid_free(auxG);
  grid_free(g);
  grid_free(f);
  grid_free(x);
}

//
//-------------------------------------------------------------------
//
int calcfg(funInfo *finfo,int N, double *x, double *f, double *g)
{
  double PHS,PVS,bgfW;
  double fmis, flap, fbgf;//,fmc,fbc,flap,fxc,fbg;
  int i;
  double Nrobs = 0;
  double *auxG;
  int Nxg,Nyg,Nzg;
  ngFitInfo *ginfo = finfo->userData;
  bool have_bad = false;

  Nxg = ginfo->nx;
  Nyg = ginfo->ny;
  Nzg = ginfo->nz;
  //TODO init dx, dy, and dz

  PHS  = ginfo->PHS; 
  PVS  = ginfo->PVS; 
  bgfW = ginfo->bgfW; 

  //
  // Initialize function and its gradient, 
  // as well as the auxiliar holder. 
  //
  auxG = ginfo->auxG;
  *f = 0.0;
  for (i = 0; i< N; i++)
    {
      auxG[i] = 0.0;
      g[i] = 0.0;

      if(!have_bad && !isfinite(x[i]))
	{
	  have_bad = true; //prevents the user from being spamed with errors
	  fprintf(stderr, "ngsonde: WARNING!!! Data contains bad values x[%i]=%g more may follow\n", i, x[i]);
	}
    }

  //
  // Get the contribution from the data misfit
  // Then update the function, gradient and reset the holder
  //
  fmis = if_misfit(finfo,N,&Nrobs,x,auxG);

  if(!(fmis >= 0))
    {
      fprintf(stderr, "ngsonde: WARNING!! fmis=%g not >= 0\n", fmis);
    }

  *f += fmis;
  for(i = 0; i< N; i++)
    {
      g[i] += auxG[i];//(auxG[i]*ScaRad);
      auxG[i] = 0.0;
    }

  //
  // Get the contribution from the background field
  //
  if(bgfW > DBL_EPSILON ) {

    fbgf = if_bgf(finfo,N,x,auxG);

    if(!(fbgf >= 0))
      {
	fprintf(stderr, "ngsonde: WARNING!! fbgf=%g not >= 0\n", fbgf);
      }

    *f += fbgf;
    for(i = 0; i< N; i++) {
      g[i] += auxG[i];
      auxG[i] = 0.0;
    }
  }

  //
  // Get the contribution from the Smoothing
  //
  if(PHS > DBL_EPSILON || PVS > DBL_EPSILON)
    {

      flap = Flap(finfo,N,x,auxG);

      if(!(flap >= 0))
	{
	  fprintf(stderr, "ngsonde: WARNING!! flap=%g not >= 0\n", flap);
	}

      *f += flap;
      for(i = 0; i < N; i++)
	{
	  g[i] += auxG[i];
	  auxG[i] = 0.0;
	}
    } 

  if(!(*f >= 0.))
    {
      fprintf(stderr,"ngsonde: WARNING!! f=%g not >= 0\n", *f);
    }

  return(1);
}
//
//-------------------------------------------------------------------
//
void fun_ini(ngFitInfo *ginfo, double *x, double *have) 
{
  // Initialize the search to the horizontal average
  // of the fields from the gridded data.
  //
  float aux,numx;
  int ix,jy,kz,pg;
  double *height;
  height = grid_alloc(ginfo->nz);
	
  for ( kz = 0; kz < ginfo->nz ; kz++) {
    aux = 0.0;
    numx = 0.0;
    for (jy = 0; jy < ginfo->ny ; jy++) {
      for (ix = 0; ix < ginfo->nx ; ix++) {
	pg = gp(ginfo->ny,ginfo->nz,ix,jy,kz);
	if (fabs(x[pg]) < ginfo->badlim) {
	  aux  += x[pg];
	  numx += 1.0;
	}
      }
    }
    if (numx > 0.5) have[kz] = aux/numx;
    else          have[kz] = ginfo->bad; 
    //else          have[kz] = 0.0; //~toZero norm

    height[kz] = (double) kz * ginfo->dz;
  }
  //
  //Fill data gaps by either interpolation or by filling with the last
  // good value.
  // 
  fun_cspline(ginfo->badlim, ginfo->dz,ginfo->nz,height,have);

  grid_free(height);

}
//
//-------------------------------------------------------------------
//
void fun_cspline(double badlim, double dxx, int n, double *xx, double *f) 
{

  int k0,k,kf,l,N,*kg;
  double *a,*b,*c,*d,*h,*x,*alpha,*L,*M,*Z,delta;
  a = grid_alloc(n);
  b = grid_alloc(n);
  c = grid_alloc(n);
  d = grid_alloc(n);
  h = grid_alloc(n);
  x = grid_alloc(n);
  L = grid_alloc(n);
  M = grid_alloc(n);
  Z = grid_alloc(n);
  alpha = grid_alloc(n);
  kg = int_alloc(n);
	
  // get to the first good value
  k0 = 0;
  for (k = 0; k < n; k++) {
    if (f[k] < badlim) {k0 = k; break;}
  }
  // get to the last good value
  kf = n-1;
  for (k = n-1; k >= 0; k--) {
    if (f[k] < badlim) {kf = k; break;}
  }

  // collect good data and their separations.
  delta = dxx;
  l = 0;
  for (k = k0; k <= kf; k++) {
    if (f[k] < badlim) {
      x[l] = xx[k]; 
      a[l] = f[k]; 
      h[l] = delta; 
      kg[k] = l; 
      l++; //<-- 
      delta = dxx;
    }

    // If l == 0, there are no soundings within grid bounds
    if (l == 0) {
      //fprintf(stderr,"ngsonde: No soundings exist within grid domain\n");
      //exit(1);
    }
    else { delta +=dxx; kg[k] = l;} //keep last good
  }
  N = l-1; //good data points (??? why not l???)

  for (k = 1; k < N; k++) {
    alpha[k] = 3.0*(a[k+1]*h[k-1] - a[k]*(x[k+1]-x[k-1]) + a[k-1]*h[k]);
    alpha[k] /= (h[k-1]*h[k]); 
  }		

  M[0] = 0.0;
  Z[0] = 0.0;
  for (k = 1; k < N; k++) {
    L[k] = 2.0*(x[k+1] - x[k-1]) - h[k-1]*M[k-1];
    M[k] = h[k]/L[k];
    Z[k] = (alpha[k] - h[k-1]*Z[k-1])/L[k];
  }

  L[N] = 1.0;
  Z[N] = 0.0;
  c[N] = 0.0;
  for (k = N-1; k >= 0; k--) {
    c[k] = Z[k] - M[k]*c[k+1];
    b[k] = (a[k+1] - a[k])/h[k] - h[k] *(c[k+1] + 2.0 *c[k])/3.0;
    d[k] = (c[k+1] -c[k])/(3.0*h[k]);
  }

  //do the interpolation
  for (k = k0; k < N; k++) {
    if ( f[k] > badlim) {
      delta = (xx[k] - x[kg[k]]);
      f[k] = a[kg[k]] + b[kg[k]]*delta + \
	c[kg[k]]*delta*delta + \
	d[kg[k]]*delta*delta*delta;
    }
  }

  //Now extrapolate using last good point
  //first the bottom
  for (k = 0; k < k0; k++)  f[k] = a[0];
  //Then the top
  for (k = N+1; k < n; k++)  f[k] = a[N];

  //free up memory
  grid_free(a);
  grid_free(b);
  grid_free(c);
  grid_free(d);
  grid_free(h);
  grid_free(x);
  grid_free(L);
  grid_free(M);
  grid_free(Z);
  grid_free(alpha);
  int_free(kg);
}
