#ifndef LFIT_H_INCLUDED
#define LFIT_H_INCLUDED

#include "ngcmdargs.h"
#include <string.h>
#include <stdio.h>

/**
 * Information about a drop sonde as read from an input file.
 */
typedef struct _ngSonde
{
	float *x; /**< The lon positions of the drop sonde for each data point */
	float *y; /**< The lat positions of the drop sonde for each data point */
	float *z; /**< The height of the drop sonde for each data point */
	float **data; /**< A two dimensional array containing data at each given point. The first index has dimensions
					 of count. The second index has dimensions of the ngCMDArgs.numFields used to create this. */
	size_t count; /**< The number of data points taken by the drop sonde. */
} ngSonde;

/**
 * Frees the data allocated for the sonde, not the sonde its self.
 *
 * @param sonde The sonde to free the data for.
 */
void ngFreeSonde(ngSonde *sonde);

/**
 * Read data about drop sondes from a candis file.
 *
 * An example call folows:
 * ngSonde *sondes = NULL;
 * size_t count;
 * ngCMDArgs args;
 * //read in args
 * ngReadData(stdin, &args, &sondes, &count);
 * //now args remains unchanged, sondes represents the data read, and count is the length of sondes.
 *
 * @param fin The input candis file, normally stdin.
 * @param args The parsed commandline args.
 * @param dest A pointer to the detination array, will be writen to and it is up to the caller to free.
 * @param destCount A pointer to the size of the destination array.
 */
void ngReadData(FILE *fin, ngCMDArgs *args, ngSonde **dest, size_t *destCount, float *BaD, float *BadLim);

/**
 * Preform a local fit for a set of data.
 *
 * @param args Parsed commandline args.
 * @param findex The index in args->fields to preform the fit on.
 * @param sondes The drop sondes to preform the fit for.
 * @param numSondes The size of the sondes array.
 */
float* ngLocalFit(ngCMDArgs *args, unsigned int findex, ngSonde *sondes, size_t numSondes,float bad, float badlim);

#endif
