#include "nglfit.h"
#include "nggfit.h"
#include "ngcmdargs.h"
#include <quickcandis.h>
#include <string.h>
#include <stdio.h>

/* Constant output field names */
#define LON "lon"
#define LAT "lat"
#define HEIGHT "z"

//
int main(int argc, char** argv)
{
  int i;
  float bad,badlim;
  char LAr[80][CLINE];
  int LArCnt,cnt;
  char pline[CLINE];

  /* variable for parsing the commandline */
  ngCMDArgs args;

  /* variables for reading the sondes */
  ngSonde *sondes = NULL;
  size_t numSondes = 0;

  /* variables for writing the resulting candis file */
  int handle, londim, latdim, zdim;
  float *data = NULL;

  /* run the test cases */
  //if(argc == 2 && strcmp(argv[1], "test") == 0)
  //{
  //		ngTestCMDArgs();
  //		return 0;
  //}

  /* parse the commandline arguments */
  ngParseArgs(argc, argv, &args, LAr, &LArCnt);

  int checkParams = 0;
  if (checkParams != 0 ) {
    fprintf(stderr,"nfe = %i\n",args.gfitTries);
    fprintf(stderr,"phs = %f\n",args.phs);
    fprintf(stderr,"pvs = %f\n",args.pvs);
    fprintf(stderr,"svx = %f\n",args.svx);
    fprintf(stderr,"svy = %f\n",args.svy);
    fprintf(stderr,"reftime = %f\n",args.reftime);
    fprintf(stderr,"--lon = %f:%f:%i\n",args.sx,args.dx,args.nx);
    fprintf(stderr,"--lat = %f:%f:%i\n",args.sy,args.dy,args.ny);
    fprintf(stderr,"--alt = %f:%f:%i\n",args.sz,args.dz,args.nz);
    fprintf(stderr,"numfields = %i\n",args.numFields);
    for(i = 0; i < args.numFields; i++)
      {
	fprintf(stderr,"v[%i]-in[%s]-out[%s] [h:v]->[%f:%f]\n", \
		i+1,args.fields[i].inName,args.fields[i].outName, \
		args.fields[i].hsmooth,args.fields[i].vsmooth);
      }	
    fprintf(stderr,"names\n");
    fprintf(stderr,"lon=%s\n",args.lon);
    fprintf(stderr,"lat=%s\n",args.lat);
    fprintf(stderr,"alt=%s\n",args.alt);
    fprintf(stderr,"pres=%s\n",args.pres);
    fprintf(stderr,"gph=%s\n",args.gph);
    fprintf(stderr,"time=%s\n",args.time);
    exit(1);
  }

  /* read in the drop sondes */
  ngReadData(stdin, &args, &sondes, &numSondes, &bad, &badlim);

  fprintf(stderr,"ngReadData ... Done!\n");
  //exit(1);


  /* create output file */
  handle = candiscreate("-", "");
  candiscomment(handle, "ngsonde:");
  for (cnt = 0; cnt < LArCnt; cnt++) {
    strcpy(pline,LAr[cnt]);
    candiscomment(handle, pline);
    //fprintf(stderr,"cnt=%i:<%s>\n",cnt,pline);
  }
  candisparam(handle,"bad",bad);
  candisparam(handle,"badlim",badlim);
  candisparam(handle,"svx",args.svx);
  candisparam(handle,"svy",args.svy);
  candisparam(handle,"reftime",args.reftime);
  londim = candisdim(handle, LON,    args.nx, args.sx, args.dx);
  latdim = candisdim(handle, LAT,    args.ny, args.sy, args.dy);
  zdim   = candisdim(handle, HEIGHT, args.nz, args.sz, args.dz);

  /* create an output field for each calc field */
  for(i = 0; i < args.numFields; i++)
    {
      data = ngLocalFit(&args, i, sondes, numSondes,bad,badlim);
      if(args.gfitTries)
	{

	  /* don't attempt a gfit if the user specified 0 tries */
	  ngGlobalFit(&args, i, data, bad, badlim);
	}

      candisfld(handle, args.fields[i].outName, 3, londim, latdim, zdim, 0, data);

      free(data);
      data = NULL;
    }

  /* write the candis file to stdout */
  candiswrite(handle);
	
  /* clean up */
  ngFreeCMDArgs(&args);
  for(i = 0; i < numSondes; i++)
    {
      ngFreeSonde(sondes + i);
    }
  free(sondes);
  sondes = NULL;
  return 0;
}
