/**
 * The implmenetation of general purpose functions.
 */

#ifndef NGUTIL_H_INCLUDED
#define NGUTIL_H_INCLUDED

/**
 * Access to storage in RowMajor order.
 * Used to access scalars on a 3D grid.
 * 
 * Ny and Nz is the size in the y and z dimensions.
 * i,j,k are the indexes of the grid point(numbering starts from zero).
 */
int gp(int Ny, int Nz, int i, int j, int k);

/**
 * Access to storage in RowMajor order.
 * Used to access vectors on a 3D grid where c is the component number.
 * 
 * Nx, Ny, and Nz is the size in the x, y, and z dimensions.
 * i,j,k are the indexes of the grid point(numbering starts from zero).
 * c     -- component number [0,1,2] 
 */
int gpe(int Nx, int Ny, int Nz, int c, int i, int j, int k);

/**
 * Calculate the norm of a vector.
 *
 * @param N the length of the vector.
 * @param fld The vector to calculate the norm of.
 */
double FieldNorm2(int N, double *fld);

/**
 * Check to make sure a vector is finite.
 * Will print to stderr if a component is infinte.
 *
 * @param N The length of the vector.
 * @param V The vector.
 * @return 1 if finite, 0 if contains any infinte values.
 */
int CheckVector(int N,double *V);

#endif
