/* getfld.c -- This routine scans through a common data format header
 * buffer looking for a field in position pos.  If found, it returns a pointer
 * to a field structure that contains the field information.  Otherwise
 * it returns NULL.  At most, max lines
 * of the buffer will be scanned.  If the end of the buffer is actually
 * reached, a format error in the header exists.
 * If type = 's', the static area is searched.  Otherwise the
 * variable field area is searched.
 */
#include <stdio.h>
#include "cdfhdr.h"
#include "string.h"
static struct field fb;
static int fatal();
static char b1[LINE],b2[LINE],b3[LINE],b4[LINE],b5[LINE],bb[LINE];
 
struct field *getfld(buffer,max,type,pos)
int max,pos;
char buffer[][LINE],type;
{
  int nf,count,loop;
  count = 0;
/* find appropriate field area of buffer */
  for (;;) {
    if (count >= max) fatal();
    if (ETEST(count)) fatal();
    sscanf(buffer[count],"%s",b1);
    count++;
    if (type == 's') {
      if (strcmp(b1,SSH) == 0) break;
    }
    else {
      if (strcmp(b1,VSH) == 0) break;
    }
  }
/* search through field list */
  fb.ptr = 0;
  for (loop = 0; loop <= pos; loop++) {
    if (count >= max) fatal();
    if (ETEST(count)) fatal();
    nf = sscanf(buffer[count],"%s %f %f %s %d %s %d %s %d %s %d %s %d",
      b1,&fb.smul,&fb.sadd,bb,&fb.dim,b2,&fb.dsize1,b3,&fb.dsize2,b4,
      &fb.dsize3,b5,&fb.dsize4);
    fb.bytes = bb[0];
/* check for end of field list */
    if (type == 's') {
      if (strcmp(b1,VSH) == 0) return(NULL);
    }
    else {
      if (strcmp(b1,FSH) == 0) return(NULL);
    }
/* check for dimension match */
    if (loop == pos) {
      if (nf < (5 + 2*fb.dim)) {
        fprintf(stderr,"getfld: inconsistent field dimension for %s\n",b1);
        exit(1);
      }
      strcpy(fb.fname,b1);
/* copy dimension names and compute total size */
      fb.tsize = 1;
      if (fb.dim >= 1) {
        strcpy(fb.dname1,b2);
        fb.tsize *= fb.dsize1;
      }
      if (fb.dim >= 2) {
        strcpy(fb.dname2,b3);
        fb.tsize *= fb.dsize2;
      }
      if (fb.dim >= 3) {
        strcpy(fb.dname3,b4);
        fb.tsize *= fb.dsize3;
      }
      if (fb.dim >= 4) {
        strcpy(fb.dname4,b5);
        fb.tsize *= fb.dsize4;
      }
/* return if we are in the correct position */
      return(&fb);
   }
/* increment pointers for next pass */
    if (fb.dim == 0) fb.ptr += 1;
    else if (fb.dim == 1) fb.ptr += fb.dsize1;
    else if (fb.dim == 2) fb.ptr += fb.dsize1*fb.dsize2;
    else if (fb.dim == 3) fb.ptr += fb.dsize1*fb.dsize2*fb.dsize3;
    else if (fb.dim == 4) fb.ptr += fb.dsize1*fb.dsize2*fb.dsize3*fb.dsize4;
    count++;
  }
}
/* fatal error -- format problem in header buffer */
static int fatal()
{
  fprintf(stderr,"getfld: format error in header buffer\n");
  exit(1);
}
