// godecim.go -- Randomly decimates variable candis fields
package main

import (
	"fmt"
	"strconv"
	"os"
	"gocandis"
	"math/rand"
)

func main() {

	// command line arguments are ranseed dfrac ifield
	args := os.Args
	if len(args) != 3 {
		fmt.Fprintln(os.Stderr,"Usage: godecim ranseed dfrac")
		os.Exit(1)
	}

	// convert command line arguments
	ranseed, err := strconv.ParseInt(args[1],0,0)
	if err != nil {
		fmt.Fprintln(os.Stderr,"Godecim: malformed command line arg 1")
		os.Exit(1)
	}
	dfrac, err := strconv.ParseFloat(args[2],64)
	if err != nil {
		fmt.Fprintln(os.Stderr,"Godecim: malformed command line arg 2")
		os.Exit(1)
	}

	// initialize with ranseed
	rand.Seed(ranseed)

	// get Candis structure from standard input
	a := gocandis.GetCandis("-")

	// set up output
	b := gocandis.NullCandis()

	// copy a to b
	comments := a.GetComments()
	for _, comment := range comments {
		b.AddComment(comment)
	}
	pnames := a.GetParamNames()
	for _, pname := range pnames {
		pval := a.GetParamValue(pname)
		b.AddParam(pname,pval)
	}
	dims := a.GetDimNames()
	for _, dim := range dims {
		dimdata := a.GetDimData(dim)
		b.AddDim(dim,dimdata)
	}
	fields := a.GetVFieldNames()
	for _, field := range fields {
		fielddims := a.GetFieldDims(field)
		fielddata := a.GetFieldData(field)
		b.AddVField(field,fielddims,fielddata)
	}

	// put the first variable slice to standard output
	b.PutCandis("-")

	// loop on write and read
	for {

		// get next variable slice and exit if eof
		ssize := a.GetNextVSlice()
		if ssize == 0 {break}

		// transfer slice if rnum in range [0.0,1.0) is less than dfrac
		rnum := rand.Float64()
		if rnum < dfrac {

			// transfer variable fields from a to b
			for _, field := range fields {
				fielddata := a.GetFieldData(field)
				b.UpdateVField(field,fielddata)
			}

			// put the new variable slice
			b.PutNextVSlice()
		}
	}
	
	// cleanup
	a.CloseCandis()
	b.CloseCandis()
}
