// goflat.go -- Program to flatten multi-dimensional fields.
// Does only first variable slice of input file.
package main

import (
	"fmt"
	"os"
	"gocandis"
)

func main() {

	// command line arguments
	args := os.Args
	numargs := len(args)
	if numargs < 4 || numargs > 6 {
		fmt.Fprintln(os.Stderr,"Usage: goflat flatdim dim1 dim2 [dim3 [dim4]]")
		os.Exit(1)
	}

	// get flat dimension name
	fdim := args[1]

	// get the input dimensions to be used
	dims := args[2:]

	// get input Candis file from standard input
	a := gocandis.GetCandis("-")

	// get other input file information

	// start creation of output Candis file
	c := gocandis.NullCandis()

	// transfer comments and add our comment
	incomments := a.GetComments()
	for _, comment := range incomments {
		c.AddComment(comment)
	}
	newcomment := ""
	for j, x := range args {
		if j == 0 {
			newcomment = "goflat:"
		} else {
			newcomment = newcomment + " " + x
		}
	}

	// add comments to new file
	c.AddComment(newcomment)

	// get parameters and extract bad and badlim if they exist
	inparams := a.GetParamNames()
	bad := ""
	badlim := ""
	for _, x := range inparams {
		if x == "bad" {
			bad = a.GetParamValue("bad")
		}
		if x == "badlim" {
			badlim = a.GetParamValue("badlim")
		}
	}

	// add bad and badlim to output file if they exist
	if bad != "" {
		c.AddParam("bad", bad)
	}
	if badlim != "" {
		c.AddParam("badlim", badlim)
	}

	// compute the length of the flattened dimension
	flatlen := 1
	for _, x := range dims {
		dimdata := a.GetDimData(x)
		flatlen = flatlen*len(dimdata)
	}

	// create data for flat dimension
	// the dimension data values are just 0, 1, 2, 3, ...
	flatdimdata := make([]float64, flatlen)
	for j := 0; j < flatlen; j++ {
		flatdimdata[j] = float64(j)
	}
	
	// create the new dims list, fdim is last
	tmp := a.GetDimNames()
	newdims := tmp[0:len(tmp) - len(dims)]
	newdims = append(newdims,fdim)
	
	// add the dimensions to the output file
	for _, y := range newdims {
		if y == fdim {
			c.AddDim(y, flatdimdata)
		} else {
			c.AddDim(y, a.GetDimData(y))
		}
	}

	// loop on input field names
	infields := a.GetVFieldNames()
	for _, x := range infields {

		// get dimension list for field
		vdims := a.GetFieldDims(x)
		
		// number of field dimensions must be >= dims
		if len(vdims) < len(dims) {

			// if no overlap in dims, just pass the field
			overlap := false
			for _, d := range dims {
				for _, v := range vdims {
					if d == v {
						overlap = true
					}
				}
			}
			if !overlap {
				vdata := a.GetFieldData(x)
				c.AddVField(x,vdims,vdata)
			}

			// go to next variable field
			continue
		}

		// the dims must match the end of the vdims list
		match := true
		for j, _ := range dims {
			k := j + len(vdims) - len(dims)
			if dims[j] != vdims[k] {
				match = false
				break
			}
		}

		// if there is a match, add new field
		if match {

			// construct the new dimension list for this field
			outdims := vdims[0:len(vdims) - len(dims)]
			outdims = append(outdims,fdim)
			
			// add field to output
			vdata := a.GetFieldData(x)
			c.AddVField(x,outdims,vdata)
		}
	}

	// write output file to standard output and clean up
	c.PutCandis("-")
	a.CloseCandis()
	c.CloseCandis()
}
