/* goreport
	This program will print the data for any static or variable
	field in a candis file. If the array has two dimmensions, 
	the array will be unflattened before printed.
*/

package main
import (
	"gocandis"
	"fmt"
	"os"
)

func main() {

	// Error checking for input
	if len(os.Args) < 2 {
		fmt.Println("Usage: goreport variable_1 variable_2...")
		os.Exit(0)
	}

    c := gocandis.GetCandis("-")

	// For each of the variable fields entered	
	for i := 1; i < len(os.Args); i++{
		fmt.Printf("%s: ",os.Args[i])

		data := c.GetFieldData(os.Args[i]) //Data contained in variable field 
		dims := c.GetFieldDims(os.Args[i]) // The dimensions of variable field
		
		numDim := len(dims)
		fmt.Println(dims)

		// If 2D array, unflatten. Otherwise leave in unflattened form
		if numDim == 2 {
			
			// Length of the 2 dimensions and unflatten the array
			dim1 := c.GetDimData(dims[0])
			len1 := len(dim1)
			dim2 := c.GetDimData(dims[1])
			len2 := len(dim2)
			unflat := gocandis.Unflatten2(len1, len2, data)

			// Print the 2D array
			for i := 0; i < len1; i++{
				for j := 0; j < len2; j++{
					fmt.Printf("%0.5e  ", unflat[i][j])
				}
			}
		}

		// Print the array in flattened form
		if numDim != 2 {
			for i := 0; i < len(data); i++{
				fmt.Printf(" %0.5e", data[i])
			}
		}
		fmt.Printf("\n")
	}
	c.CloseCandis()
 }
