/* sistogo.go This program will take the a record of candis type in
	fibre format and will convert the information and store it in
	a candis structure in go. The candis file will output the
	infomation into a candis file names newCandis

	Usage:
		sistogo < fibre_output filename
 */




package main
import (
	"gocandis"
	"fmt"
	"os"
	"strings"
	"strconv"
)


/* Function for getting rid of extra white space and quotations */
func cleanString(s string) string {
	parts := strings.Split(s, "\"")
	return parts[1]
}

/* Function for reading a line at a time from an open file pointer */
func getline(fptr *os.File) (string, int) {
	stuff := make([]byte, 1)
	lbuff := make([]byte, 0)
	for {
		num, err := fptr.Read(stuff)
		if err != nil || num == 0 {
			if len(lbuff) != 0 {
				return string(lbuff), 0
			} else {
				return "", 1
			}
		} else if stuff[0] == '\n' {
			return string(lbuff), 0
		} else {
			lbuff = append(lbuff, stuff[0])
		}
	}
}

func main() {
	in := os.Stdin
	c := gocandis.NullCandis()
	tag := 'c'  // Designate what section that is be read in

	/* For getting the parameters*/
	pname := "" 
	value := ""
	ptag := 'n'

	/* For getting the dimensions*/
	skipped := false // For skipping over unneeded lines in the input
	DimName := ""
	DimAdded := 0
	DimCount := 0  // Number of dimensions

	/* For getting variable fields*/
	vtag := 'n' // Designate what section that is be read in
	vname := ""
	dlist := make([]string, 0, 50)

	// loop until kicked out by getline()
	for { 

		// grab a line
		myline, lastline := getline(in)
		if lastline != 0 {
			break
		}

		// Getting comments
		if tag == 'c' {
			if strings.Contains(myline, "\"") {
				c.AddComment(cleanString(myline))
			}
			if strings.Contains(myline, "]") {
				tag = 'p'
				continue
			}
		}

	 	// Getting parameters
		if tag == 'p' {
			if strings.Contains(myline, "\"") {
				if ptag == 'n' {
					pname = cleanString(myline)
					ptag = 'v'
				} else {
					value = cleanString(myline)
					c.AddParam(pname, value)
					ptag = 'n'
				}
			}
			if strings.Contains(myline, "]") {

				tag = 'd'
				continue
			}
		}

		// Getting the dimensions
		if tag == 'd' {
			// Skips over lines of fibre that are unessary
			if strings.Contains(myline, "<") {
				skipped = true

			// Counts the number of dimensions from the dlist element
			// of the record
			} else if !skipped && strings.Contains(myline, "\"") {
				DimCount++

			// Gets the infomation for each of the dimensions
			} else if skipped {
				if strings.Contains(myline, "\"") {
					DimName = cleanString(myline)
					for i := 0; i < 3; i++ {
						myline, lastline = getline(in)
						if lastline != 0 {
							fmt.Fprintln(os.Stderr, "Sistogo: unexpected end of file\n")
						}
					}
				} else {
					parts := strings.Split(myline, " ")
					DimData := make([]float64,0,50)
					for j := range parts {
						if strings.Contains(parts[j], "."){
							num, _ := strconv.ParseFloat(parts[j], 64)
							DimData = append(DimData, num)
						}
					}
					if len(DimData) != 0 {
						c.AddDim(DimName, DimData)
						DimAdded++
					}

				}
			}

			// If have gotten the expected number of dimensions
			if DimCount == DimAdded && DimCount != 0 {
				tag = 'v'
				continue
			}
	 	}

	 	// Getting the variable fields
		if tag == 'v' {
			// Get the name of the field
			if vtag == 'n' && strings.Contains(myline, "\"") {
				vname = cleanString(myline)
				vtag = 'l'

			// Get the dimension list for the field
			} else if vtag == 'l' { 
				if strings.Contains(myline, "\"") {
					edit := cleanString(myline)
					dlist = append(dlist, edit)
				} else if strings.Contains(myline, "]") {
					vtag = 'd'
				}

			// Get the data for the field
			} else if vtag == 'd' {
				parts := strings.Split(myline, " ")
				VData := make([]float64,0,50)
				for j := range parts {
					if strings.Contains(parts[j], "."){
						num, _ := strconv.ParseFloat(parts[j], 64)
						VData = append(VData, num)
					}
				}

				if len(VData) != 0 {
					c.AddVField(vname, dlist, VData)
					dlist = nil
					vtag = 'n'
				}
			}
	 	}

	}
	if len(os.Args) > 1 {
		filename := os.Args[1] 
		if filename == "-" {
			c.PutCandis("-")	
		} else {
			c.PutCandis(filename)
		}
	} else {
		c.PutCandis("newCandis")
	}

	
	in.Close()
}
